#ifndef __AIO_H
#define __AIO_H

typedef enum {
  AIO_ADC=0, ADIO_DAC=1
} AIOType;

typedef struct AIO AIO;
typedef struct AIOAPI AIOAPI;
typedef struct VoltageRange VoltageRange;
typedef struct VoltageSubRange VoltageSubRange;
typedef struct PeriodRange PeriodRange;

struct VoltageSubRange {
  float start, step;
  unsigned count;
};
struct VoltageRange {
  VoltageSubRange low,high;
};
struct ActualVoltageRange {
  float low, high;
};

struct PeriodRange {
  unsigned start, step, count;
};
#define AIO_PR(start,step,count) ((PeriodRange){start,step,count});
#define AIO_VR(lowstart,lowstep,lowcount,highstart,highstep,highcount) ((VoltageRange){(VoltageSubRange){lowstart,lowstep,lowcount},(VoltageSubRange){,highstart,highstep,highcount}})
struct AIOAPI {
  int (*Init)(void *);
  void (*Fini)(void *);
  int (*Lock)(void *,unsigned num,int flags);
  int (*Unlock)(void *,unsigned num,int flags);
  int (*Preempt)(void *);
  AIOType (*Type)(AIO *);
  int (*VoltageRangeListLength)(AIO *);
  int (*VoltageRange)(AIO *,int i,VoltageRange *rr);
  int (*PrecisionList)(AIO *);
  int (*PeriodRangeListLength)(AIO *);
  int (*PeriodRange)(AIO *,int i,PeriodRange *r);
  int (*TriggerListLength)(AIO *);
  int (*TriggerList)(AIO *,int *tlist);
  int (*Channels)(AIO *);
  int (*Configuration)(AIO *,float *low,float *high,int *prec,int *rate,int *trigger,int *itrig);
  int (*ConfigureTest)(AIO *,int count,float *low,float *high,int *prec,int *rate,int *trigger,int itrig);
  int (*Configure)(AIO *,int count,float *low,float *high,int *prec,int *rate,int *trigger,int itrig);
  int (*ITrig)(AIO *,int itrig);
  int (*Get)(AIO *,int channel);
  int (*Put)(AIO *,int channel,int value);
  int (*Ready)(AIO *,int channel);
  int (*Gets)(AIO *,int channelMask,int samples,void *buf);
  int (*Puts)(AIO *,int channelMask,int samples,void *buf);
  int (*Readys)(AIO *,int channelMask,int samples,void *buf);
};

struct AIO {
  AIOAPI *API;
  int InitStatus;
};

#define AIO_HZ(hz) ((int)(1000000000/hz))
enum {
  NetAIO_Lock=0,
  NetAIO_VoltageRangeListLength=1,
  NetAIO_VoltageRange=2,
  NetAIO_PrecisionList=3,
  NetAIO_RateRangeListLength=4,
  NetAIO_RateRange=5,
  NetAIO_TriggerListLength=6,
  NetAIO_TriggerList=7,
  NetAIO_Channels=8,
  NetAIO_Configuration=9,
  NetAIO_ConfigureTest=10,
  NetAIO_Configure=11,
  NetAIO_Get=12,
  NetAIO_Put=13,
  NetAIO_Ready=14,
  NetAIO_Gets=15,
  NetAIO_Puts=16,
  NetAIO_Readys=17
};
#define AIO_APICount (1+NetAIO_Readys)

#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.

