#include <stdio.h>
#include <string.h>
#include "AggregateDIO.h"

static DIOAPI AggregateDIOAPI = {
  .Init = (void *)AggregateDIOInit,
  .Fini = (void *)AggregateDIOFini,
  .Lock = (void *)AggregateDIOLock,
  .Unlock = (void *)AggregateDIOUnlock,
  .Preempt = (void *)AggregateDIOPreempt,
  .Refresh = (void *)AggregateDIORefresh,
  .Commit = (void *)AggregateDIOCommit,
  .Set = (void *)AggregateDIOSet,
  .Get = (void *)AggregateDIOGet,
  .SetAsync = (void *)AggregateDIOSetAsync,
  .GetAsync = (void *)AggregateDIOGetAsync,
  .Count = (void *)AggregateDIOCount,
  .Capabilities = (void *)AggregateDIOCapabilities
};

void *AggregateDIOInit(AggregateDIO *dio,unsigned Count,void *pin1,
		       int *SubCount,DIO **pDIO) {
  int i;
  Pin *pin = pin1;

  if (dio->InitStatus > 0) return dio;
  dio->API = &AggregateDIOAPI;
  dio->pin = pin;
  if (dio->pin && dio->pin->InitStatus <= 0) {
    dio->InitStatus = -1;
    return dio;
  }
  dio->Count = Count;
  dio->SubCount = SubCount;
  dio->dio = pDIO;
  for (i=0;i<Count;i++) {
    if (dio->dio[i]) {
      dio->SubCount[i] = dio->dio[i]->API->Count(dio->dio[i]);
    } else {
      dio->SubCount[i] = 0;
    }
  }
  for (i=0;i<Count;i++) {
    if (dio->dio[i] && dio->dio[i]->InitStatus <= 0) {
      dio->API->Fini(dio);
      dio->InitStatus = -1;
      return dio;
    }
  }
  dio->InitStatus = 1;
  return dio;
}

void AggregateDIOFini(AggregateDIO *dio) {
  int i;

  if (dio->pin) {
    dio->pin->API->Fini(dio->pin);
  }
  for (i=0;i<dio->Count;i++) {
    if (dio->dio[i]) {
      dio->dio[i]->API->Fini(dio->dio[i]);
    }
  }
  if (dio->InitStatus > 0) dio->InitStatus = 0;
}

void AggregateDIORefresh(AggregateDIO *dio) {
  int i;
  for (i=0;i<dio->Count;i++) {
    if (dio->dio[i]) {
      dio->dio[i]->API->Refresh(dio->dio[i]);
    }
  }
}

void AggregateDIOCommit(AggregateDIO *dio,int forceall) {
  int i;
  for (i=0;i<dio->Count;i++) {
    if (dio->dio[i]) {
      dio->dio[i]->API->Commit(dio->dio[i],forceall);
    }
  }
}

static inline int DIOSub(AggregateDIO *dio,int *DIONum,int *i) {
  if (*DIONum < 0) return 0;
  for (*i=0;*i<dio->Count && *DIONum >= dio->SubCount[*i];(*i)++) {
    *DIONum -= dio->SubCount[*i];
  }
  return (*i < dio->Count);
}

void AggregateDIOSet(AggregateDIO *dio,int DIONum,DIOState State) {
  int i;

  if (!DIOSub(dio,&DIONum,&i)) return;
  dio->dio[i]->API->Set(dio->dio[i],DIONum,State);
}

DIOState AggregateDIOGet(AggregateDIO *dio,int DIONum) {
  int i;

  if (!DIOSub(dio,&DIONum,&i)) return INPUT;
  return dio->dio[i]->API->Get(dio->dio[i],DIONum);
}

void AggregateDIOSetAsync(AggregateDIO *dio,int DIONum,DIOState State) {
  int i;

  if (!DIOSub(dio,&DIONum,&i)) return;
  dio->dio[i]->API->SetAsync(dio->dio[i],DIONum,State);
}

DIOState AggregateDIOGetAsync(AggregateDIO *dio,int DIONum) {
  int i;

  //printf("%d\n",DIONum);
  if (!DIOSub(dio,&DIONum,&i)) return INPUT;
  //printf("%d:%d\n",i,DIONum);
  return dio->dio[i]->API->GetAsync(dio->dio[i],DIONum);
}

int AggregateDIOLock(AggregateDIO *dio,unsigned num,int flags) {
  unsigned j,num0=num;
  int ret;

  if (!DIOSub(dio,(int *)&num,&j)) return -1;
  if (dio->pin) {
    // shared:
    //   acquire read lock
    //   if pin not DIO
    //     acquire write lock
    //     set pin to DIO
    //     drop write lock
    // exclusive:
    //   acquire write lock
    //   set pin to DIO
    if (flags & SHARED) {
      ret = dio->pin->API->Lock(dio->pin,num0,flags);
      if (dio->pin->API->ModeGet(dio->pin,num0) != MODE_DIO) {
	ret = dio->pin->API->Lock(dio->pin,num0,flags & ~SHARED);
	if (ret < 0) {
	  dio->pin->API->Unlock(dio->pin,num0,flags); //assert
	  return ret;
	}
	dio->pin->API->ModeSet(dio->pin,num0,MODE_DIO);
	dio->pin->API->Unlock(dio->pin,num0,flags & ~SHARED);
      }
    } else {
      ret = dio->pin->API->Lock(dio->pin,num0,flags);
      if (ret < 0) return ret;
      dio->pin->API->ModeSet(dio->pin,num0,MODE_DIO);
    }
  }
  return ret;
}

int AggregateDIOUnlock(AggregateDIO *dio,unsigned num,int flags) {
  unsigned j,num0=num;

  if (!DIOSub(dio,(int *)&num,&j)) return -1;
  if (dio->pin) {
    return dio->pin->API->Unlock(dio->pin,num0,flags & ~SHARED);
  }
}

void AggregateDIOPreempt(AggregateDIO *dio) {
}

int AggregateDIOWait(AggregateDIO *dio,int *match,int min,int max,int nh,
		     int *h,int nl,int *l) {
  int i,j;
  int h2[nh],l2[nl];

  for (j=0;j<nh;j++) {
    h2[j] = h[j];
    if (!DIOSub(dio,h2+i,&i)) return -1;
  }
  for (j=0;j<nl;j++) {
    l2[j] = l[j];
    if (!DIOSub(dio,l2+i,&i)) return -1;
  }
  return dio->dio[i]->API->Wait(dio->dio[i],match,min,max,nh,h2,nl,l2);
}

unsigned AggregateDIOCount(AggregateDIO *dio) {
  unsigned ret = 0,i;

  for (i=0;i<dio->Count;i++) {
    ret += dio->SubCount[i];
  }
  return ret;
}

DIOCaps AggregateDIOCapabilities(AggregateDIO *dio,unsigned num) {
  int i;

  if (!DIOSub(dio,&num,&i)) return DIO_NOT_PRESENT;
  return dio->dio[i]->API->Capabilities(dio->dio[i],num);
}

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
