# make variables required
# all = list of variables defining what to build
# PathToRoot=path from source directory to common makefiles such as this one

# environment variables recognized (e.g. from the command line)
# only=arch causes us to only build the specified arch
# exclude=list causes use to exclude the specified arches
# verbose=anything causes us to rsync verbosely


SHELL := /bin/bash

-include $(SRCDIR)$(PathToRoot)BuildProcess.user
ARCH=$(notdir $(CURDIR))

ifeq (,$(filter _%,$(ARCH)))

ifeq ($(wildcard _*),)
     $(shell $(PathToRoot)BuildProcess.setup $(PathToRoot) 1>&2)
     $(info Run 'make' again to start the actual build)
endif

.SUFFIXES:

#==============================================================================
# All sub-directories starting with an underscore are treated as arches to
# build.
# only=arch causes us to only build the specified arch
# exclude=list causes use to exclude the specified arches

ifeq ($(only),)
all_arches = $(filter-out $(foreach x,$(exclude),_$x),$(wildcard _*))
else
all_arches = $(wildcard _$(only))
endif
#$(info all_arches=$(all_arches))

MAKETARGET = $(MAKE) --no-print-directory -C $@ -f $(CURDIR)/Makefile \
                     SRCDIR=$(CURDIR)/ $(MAKECMDGOALS)

.PHONY: $(all_arches) all build release tarball setup sync $(subst @,@@,$(foreach x,$(synchosts),$x))

tarball:
	@if [ -e .tarball ]; then NOW=`date +%Y%m%d%H%M%S` && HOSTNAME=$(shell hostname) && echo "creating build tarball $(PROJECT)-$$NOW-$$HOSTNAME" && \
	DIR=$(lastword $(subst /, ,$(shell pwd))) && \
	echo "$(PROJECT)-$$NOW-$$HOSTNAME.tar.gz:" >> buildlog.txt && \
	for i in `ls .build-*`; do TMP=`echo "$$i:"|cut -f2- -d-`; echo -n -e "\t$$TMP" >> buildlog.txt &&  >> buildlog.txt ; V=`cat $$i|cut -f1 -d-`;H=`cat $$i|cut -f2 -d-`;echo "$$(($$V-1))-$$H" >> buildlog.txt ; done && \
	X=`pwd`;mkdir -p tarballs && cd .. && tar --exclude='tarballs' --exclude='*/CVS' --exclude='_*' --exclude='product_*' -hczf $$X/tarballs/$(PROJECT)-$$NOW-$$HOSTNAME.tar.gz $$DIR/ && cd $$DIR && \
	if [ "$(build_mirror)" != "" -a -e "$(build_mirror)" ]; then cp tarballs/$$DIR-$$NOW-$$HOSTNAME.tar.gz $(build_mirror); fi && rm -f .tarball; fi

ifdef verbose
rsync_opts=-v
else
rsync_opts=-q
endif

#$(info SYNCINFO $(syncdirs_tsctl) -- $(PROJECT) -- $(syncdirs_$(PROJECT)))
$(patsubst %,%/phony,$(foreach x,$(syncdirs_$(PROJECT)),$x)):
	@if [ -e "$(patsubst %/phony,%,$@)" ]; then echo "Syncing to $(patsubst %/phony,%,$@)"; rsync --exclude tarballs --exclude CVS --inplace -l -t $(rsync_opts) -r  --exclude '/product_*' --exclude '/_*' . $(patsubst %/phony,%,$@); rsync --inplace -l -t $(rsync_opts) -r --include='/product_*' --exclude='/*' . $(patsubst %/phony,%,$@); else echo "Not syncing to $(patsubst %/phony,%,$@)"; fi
#	@if [ -e "$(patsubst %/phony,%,$@)" ]; then echo "Syncing to $(patsubst %/phony,%,$@)"; rsync --exclude tarballs --exclude CVS --delete --inplace -l -t $(rsync_opts) -r  --exclude '/product_*' --exclude '/_*' . $(patsubst %/phony,%,$@); rsync --inplace -l -t $(rsync_opts) -r --include='/product_*' --exclude='/*' . $(patsubst %/phony,%,$@); else echo "Not syncing to $(patsubst %/phony,%,$@)"; fi

$(foreach x,$(synchosts),$x):
	@echo "Syncing to $(subst @,:,$@)"; rsync --exclude CVS --delete --inplace -l -t $(rsync_opts) -r --exclude '/product_*' --exclude '/_*' . $(subst @,:,$@); rsync --inplace -l -t $(rsync_opts) -r --include='/product_*' --exclude='/*' . $(subst @,:,$@);

setup:
	@$(PathToRoot)BuildProcess.setup $(PathToRoot)

sync: $(patsubst %,%/phony,$(foreach x,$(syncdirs_$(PROJECT)),$x)) $(foreach x,$(synchosts),$x)

syncback: $(subst @,@@,$(foreach x,$(synchosts),$x))
	@cd .. && rsync $(rsync_opts) -l -r -t $(subst @@,:,$<) .

all: $(all_arches) tarball sync
	@#echo "Build complete"

ifeq ($(MAKECMDGOALS),release)
export PROJECT PathToRoot
endif

native:
	mkdir native

native/BuildProcess.release: BuildProcess.release.c shell.c HashTable.c
	gcc -o $@ $^

release: native native/BuildProcess.release
	@#HOSTNAME=$(shell hostname);REMAKE=`cat .build-* 2>/dev/null | cut -f1 -d- | sort -n | uniq | wc -l`;if [ "$$REMAKE" != "1" ]; then HV=`cat .build-* 2>/dev/null | cut -f1 -d- | sort -n | tail -1` && HV=$$(($$HV+0)) && for i in `ls -d .build-*`; do echo $$HV-$$HOSTNAME > $$i; done; echo "$$HV-$$HOSTNAME:all architectures synchronized to this build number for release" >> buildlog.txt; make clean all; fi
	@#$(PathToRoot)BuildProcess.release
	@native/BuildProcess.release
	@#if [ -x release-script ]; then export PathToRoot=$(PathToRoot) && ./release-script; fi

$(all_arches):
	+@echo arch $@
	+@$(MAKETARGET)
	+@TMP=`echo $@ | cut -f2- -d_` && echo "Build for $$TMP complete"
	
Makefile:
$(PathToRoot)BuildProcess.user:
$(PathToRoot)BuildProcess.make:

ifneq ($(MAKECMDGOALS),tarball)
ifneq ($(MAKECMDGOALS),setup)
ifneq ($(MAKECMDGOALS),release)
ifneq ($(MAKECMDGOALS),sync)
ifneq ($(MAKECMDGOALS),syncback)
% :: $(all_arches) ;
endif
endif
endif
endif
endif

# NOW=`date +%Y%m%d%H%M%S` && \

else
-include $(SRCDIR)$(PathToRoot)BuildProcess.flags
CMDARCH=$(word 1,$(subst -, ,$(ARCH)))

FULLARCH=$(shell echo $(ARCH) | cut -f2- -d_)

# ARCHLIST = list of base names of primary and all sub-arches
ARCHLIST=$(subst -, ,$(ARCH))

# PARCH = primary architecture
# SARCH = list of sub-architectures
FULL_PARCH=$(subst _, ,$(word 1,$(ARCHLIST)))
PARCH=$(word 1,$(FULL_PARCH))
PFLAGS=$(wordlist 2,99,$(FULL_PARCH))
SARCH=$(wordlist 2,99,$(ARCHLIST))

ifeq ($(origin prefix_$(PARCH)), undefined)
     $(shell read -e -p "Enter cross-compiler prefix for $(PARCH):" PREFIX ;\
       echo "prefix_$(PARCH)=$$PREFIX" >> ../$(PathToRoot)BuildProcess.user)
endif


CFLAGS+=-DARCHCOUNT=$(words $(ARCHLIST)) -DARCHLIST="ARCH_FIRST($(PARCH)) $(foreach x,$(SARCH),ARCH_NEXT($x))"

ifeq ($(CFLAGS$(CMDARCH)),)
	CFLAGS+=$(CFLAGS_$(PARCH))
else
	CFLAGS+=$(CFLAGS$(CMDARCH))
endif
#$(info debug1:CFLAGS_$(PARCH)--$(CFLAGS_$(PARCH)))
#$(info debug2:CFLAGS$(CMDARCH)--$(CFLAGS$(CMDARCH)))

ifeq ($(LDFLAGS$(CMDARCH)),)
	LDFLAGS+=$(LDFLAGS_$(PARCH))
else
	LDFLAGS+=$(LDFLAGS$(CMDARCH))
endif
#$(info debug3:LDFLAGS_$(PARCH)--$(LDFLAGS_$(PARCH)))
#$(info debug4:LDFLAGS$(CMDARCH)--$(LDFLAGS$(CMDARCH)))

CFLAGS+=$(foreach x,$(PFLAGS),$(CFLAGS_$(x)))
LDFLAGS+=$(foreach x,$(PFLAGS),$(LDFLAGS_$(x)))
CFLAGS+=-DARCH=$(PARCH) $(foreach x,$(SARCH),$(shell if [ -e ../$x.h ]; then echo '-include ../$x.h'; fi))

#$(info ARCH=$(ARCH) FULLARCH=$(FULLARCH) CMDARCH=$(CMDARCH) ARCHLIST=$(ARCHLIST) FULL_PARCH=$(FULL_PARCH) PARCH=$(PARCH) PFLAGS=$(PFLAGS) SARCH=$(SARCH) all$(CMDARCH)=$(all$(CMDARCH)))
#$(info CFLAGS=$(CFLAGS))
#$(info LDFLAGS=$(LDFLAGS))

VPATH = $(SRCDIR)
CC=$(prefix_$(PARCH))gcc
AR=$(prefix_$(PARCH))ar

%.a:
	@echo "Creating archive $@";$(AR) cr $@ $^

%: %.o
	@echo "Building executable $@";$(CC) $^ $(LOADLIBES) $(LDLIBS) -o $@ $(LDFLAGS_$@) $(LDFLAGS)

%.auto.o: %.auto.c
	@$(CC) $(CFLAGS) -c $<

%.o: %.c
	@echo "Compiling $<";$(CC) $(CFLAGS) $(CFLAGS_$(lastword $(subst /, ,$(patsubst %.o,%,$@)))) -c $<

%.so:
	@echo "Building shared object $@";$(CC) $(CFLAGS) -shared $(LDLIBS) -Wl,-soname,$@ -o $@ $^ $(LDFLAGS) $(LDFLAGS_$@)

%.auto.c:
	@T=`date +%s`;echo "const char copyright[] __attribute__ ((weak)) = \"Copyright (c) Technologic Systems - \" __DATE__ ; const char hoststring[] __attribute__ ((weak)) = \"CompileHost - $(shell uname -a)\"; const char build[] __attribute__ ((weak)) =  \"\$$build: $< $$(cat ../.build-$*-$(FULLARCH))\"; const char compiledate[] __attribute__ ((weak)) = \"\$$date: $$(date --date=@$$T)\"; const char archstr[] __attribute__ ((weak)) = \"$(ARCH)\"; const char dummystr[] __attribute__ ((weak)) = \"\"; const int compiletime __attribute__ ((weak)) = $$T;" > $@
	@#echo "--> $@";cat $@

%.md5.c: %
	@echo "const char md5a_$(patsubst %.c.md5.c,%,$@)[] = \"\$$md5:\" __DATE__ \" \" __TIME__ \" $(shell md5sum $<) \";" > $@
	@echo "const char md5b_$(patsubst %.c.md5.c,%,$@)[] = \"\$$md5:\" __DATE__ \" \" __TIME__ \"" $(shell $(CC) -E $(CFLAGS) $< | md5sum) "$<\";" >> $@
	@#echo "--> $@"

%.d: %.c
	@set -e; rm -f $@; \
	$(CC) -MM -MG $(CFLAGS) $< | \
	sed 's,\($*\)\.o[ :]*,\1.o $@ : ,g'  > $@;

#arch_objects=$(foreach x,$(SARCH),$x.o)
arch_objects=$(foreach x,$(SARCH),$(foreach x,$(SARCH),$(shell if [ -e ../$x.h ]; then echo '$x.o'; fi)))
#$(info arch_objects=$(arch_objects))

%.de: Makefile	
	@echo "$(patsubst %.de,%,$@):  $(patsubst %.de,%,$@).auto.o  $(patsubst %.c,%.o,$(foreach x,$(patsubst %.de,%,$@),$($(x)))) $(patsubst %,%.md5.o,$(foreach x,$(patsubst %.de,%,$@),$($(x)))) $(if $(findstring .o,$@),,$(arch_objects))" > $@
	@echo "$(patsubst %.de,%,$@).auto.o: $(patsubst %.c,%.o,$(foreach x,$(patsubst %.de,%,$@),$($(x)))) | ../.build-$(patsubst %.de,%,$@)-$(FULLARCH)" >> $@

%.ds: Makefile
	@echo "product$(ARCH)/$(patsubst %.ds,%,$@): $(patsubst %.ds,%,$@)" >$@
	@echo '	@HV=`cat ../.build-$*-$(FULLARCH)` && cp $(patsubst %.ds,%,$@) ../product$(ARCH)/$(patsubst %.ds,%,$@)-$$$$HV && ln -sf $(patsubst %.ds,%,$@)-$$$$HV ../product$(ARCH)/$(patsubst %.ds,%,$@) && echo "Build $$$$HV of $* for $(FULLARCH) complete" && HV=$$$$(($$$$HV+1)) && echo $$$$HV-$(shell hostname) > ../.build-$*-$(FULLARCH) && touch ../.tarball' >> $@

clean:
	@echo "Cleaning";rm -f $(SRCDIR)*~ *.auto.c *.o *.so $(products) $(products2) *.exe *~ *.d *.de *.ds *.md5.c *.gcda *.gcno gmon.out *.gcov *.gch && find . -name "*.a" -type f -delete

#arch_sources=$(foreach x,$(SARCH),$x.c)
arch_sources=$(foreach x,$(SARCH),$(foreach x,$(SARCH),$(shell if [ -e ../$x.h ]; then echo '$x.c'; fi)))
#$(info arch_sources=$(arch_sources))

all_sources = $(sort $(foreach x,$(all),$($(x))) $(arch_sources) $(md5_sources))
products =  $(foreach x,$(all),$(x)) $(foreach x,$(all$(CMDARCH)),$(x)) 
products2 =  $(foreach x,$(all),product$(ARCH)/$(x)) $(foreach x,$(all$(CMDARCH)),product$(ARCH)/$(x))
#$(info all_sources=$(all_sources))
#$(info products=$(products))
#$(info products2=$(products2))
#$(info all_sources=$(all_sources))

all: $(products) $(products2)

.PHONY: $(patsubst %,../.build-%-$(FULLARCH),$(all))

$(patsubst %,../.build-%-$(FULLARCH),$(all)):
	@if [ ! -e $@ ]; then HV=`cat ../.build-* 2>/dev/null | cut -f1 -d- | sort -n | tail -1`; HV=$$(($$HV-1)); else HV=`cat $@ | cut -f1 -d-`; fi; HV=$$(($$HV+0)); if (( $$HV < 1 )); then HV=1; fi; echo "$$HV-$(shell hostname)" > $@;

ifneq ($(MAKECMDGOALS),clean)
-include $(all_sources:.c=.d)
-include $(products:=.de)
-include $(products:=.ds)
endif
endif
# The purpose of this makefile:
# 1. build each architecture in its own target directory
# 2. automatically generate header file dependencies
# 3. sync sources and products to snapshot directories
# 4. make it really simple to create a makefile
# 5. count build numbers
# 6. put build info in binaries

# Author: Michael Schmidt (michael@embeddedARM.com)
# Copyright (c) 2012, Technologic Systems, All Rights Reserved
# Refer to the COPYRIGHT file provided with this project for licensing terms.
