#ifndef __Bus_H
#define __Bus_H

typedef struct Bus Bus;
typedef struct BusAPI BusAPI;

struct BusAPI {
  void *(*Init)(void *,...);
  void (*Fini)(void *);
  int (*Lock)(void *,unsigned num,int flags);
  int (*Unlock)(void *,unsigned num,int flags);
  int (*Preempt)(void *);
  unsigned char (*Peek8)(void *,int adrs);
  void (*Poke8)(void *,int adrs,unsigned char val);
  unsigned short (*Peek16)(void *,int adrs);
  void (*Poke16)(void *,int adrs,unsigned short val);
  unsigned (*Peek32)(void *,int adrs);
  void (*Poke32)(void *,int adrs,unsigned val);
  int (*BitGet8)(void *bus,int adrs,int bit);
  void (*BitAssign8)(void *bus,int adrs,int bit,int val);
  void (*BitSet8)(void *bus,int adrs,int bit);
  void (*BitClear8)(void *bus,int adrs,int bit);
  int (*BitGet16)(void *bus,int adrs,int bit);
  void (*BitAssign16)(void *bus,int adrs,int bit,int val);
  void (*BitSet16)(void *bus,int adrs,int bit);
  void (*BitClear16)(void *bus,int adrs,int bit);
  int (*BitGet32)(void *bus,int adrs,int bit);
  void (*BitAssign32)(void *bus,int adrs,int bit,int val);
  void (*BitSet32)(void *bus,int adrs,int bit);
  void (*BitClear32)(void *bus,int adrs,int bit);
  void (*PeekStream)(void *bus,int adr,int dir,char *buf,int len);
  void (*PokeStream)(void *bus,int adr,int dir,char *buf,int len);
  void (*Refresh)(void *);
  void (*Commit)(void *,int forceall);
  int (*BitToggle8)(void *bus,int adrs,int bit);
  int (*BitToggle16)(void *bus,int adrs,int bit);
  int (*BitToggle32)(void *bus,int adrs,int bit);
  unsigned char (*Assign8X)(void *,int adrs,int bit1,int bit0,int val);
  unsigned short (*Assign16X)(void *,int adrs,int bit1,int bit0,int val);
  unsigned (*Assign32X)(void *,int adrs,int bit1,int bit0,int val);
  unsigned char (*BitsGet8)(void *,int adrs,int bit1,int bit0);
  unsigned short (*BitsGet16)(void *,int adrs,int bit1,int bit0);
  unsigned (*BitsGet32)(void *,int adrs,int bit1,int bit0);
};

enum {
  NetBus_Lock=0,
  NetBus_Peek8=1,
  NetBus_Poke8=2,
  NetBus_Peek16=3,
  NetBus_Poke16=4,
  NetBus_Peek32=5,
  NetBus_Poke32=6,
  NetBus_BitGet8=7,
  NetBus_BitAssign8=8,
  NetBus_BitSet8=9,
  NetBus_BitClear8=10,
  NetBus_BitGet16=11,
  NetBus_BitAssign16=12,
  NetBus_BitSet16=13,
  NetBus_BitClear16=14,
  NetBus_BitGet32=15,
  NetBus_BitAssign32=16,
  NetBus_BitSet32=17,
  NetBus_BitClear32=18,
  NetBus_PeekStream=19,
  NetBus_PokeStream=20,
  NetBus_Refresh=21,
  NetBus_Commit=22,
  NetBus_BitToggle8=23,
  NetBus_BitToggle16=24,
  NetBus_BitToggle32=25,
  NetBus_Assign8X=26,
  NetBus_Assign16X=27,
  NetBus_Assign32X=28,
  NetBus_BitsGet8=29,
  NetBus_BitsGet16=30,
  NetBus_BitsGet32=31
};

#define Bus_APICount (1+NetBus_BitsGet32)
struct Bus {
  BusAPI *API;
  int InitStatus;
  unsigned LockBase;
  int deferlock;
};
#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
