#include <stdio.h>
#include <string.h>
#include "DIOMap.h"
#include "Thread.h"

enum {
  NONBLOCKING=1, SHARED=2, NOUNLOCK=4
};

#define DIOMapMax 1024
struct DIOMapInfo {
  char map[256];
} info[1024];
static int DIOMapEntries = 0;

static unsigned diomaplock;

int DIOMapInit() {
  static int initialized = 0;
  if (!initialized) {
    initialized = 1;
    diomaplock = ThreadLockAllocate(1);
    DIOMapLoadFromFile("/etc/dioctl.config");
  }
}

int DIOMapLock(int flags) {
  int mode = (flags & NONBLOCKING) ? 
    LOCK_TRY :
    (flags & NOUNLOCK) ? LOCK_FOD : LOCK_SOD;
  if (flags & SHARED) {
    return ThreadLockR(diomaplock,mode);
  } else {
    return ThreadLockW(diomaplock,mode);
  }
}

int DIOMapUnlock(int flags) {
  if (flags & SHARED) {
    return ThreadUnlockR(diomaplock);
  } else {
    return ThreadUnlockW(diomaplock);
  }
}

int DIOMapLength(int *max) {
  int i,n=0;

  for (i=0;i<DIOMapEntries;i++) {
    if (info->map[256*i+1]) n++;
  }
  if (max) *max = DIOMapEntries;
  return n;
}

char *DIOMapGet(int i,int *value) {
  if (!info->map[256*i+1]) {
    return 0;
  }
  *value = (int)info->map[256*i];
  return &info->map[256*i+1];
}

int DIOMapLookup(char *string) {
  int i;

  for (i=0;i<DIOMapEntries;i++) {
    if (!strcmp(&info->map[256*i+1],string)) {
      return info->map[256*i];
    }
  }
  return -1;
}

char *DIOMapLookupPartial(char *string,int value) {
  int i;
  for (i=0;i<DIOMapEntries;i++) {
    if (!strncmp(&info->map[256*i+1],string,strlen(string)) && info->map[256*i]==(char)value) {
      return &info->map[256*i+1];
    }
  }
  return "";
}

static int compar(const void *a, const void *b) {
  const char *a1 = a, *b1 = b;
  return strncmp(a1+1,b1+1,255);
}

int DIOMapAdd(char *string,int value) {
  int i;
  if (DIOMapEntries == DIOMapMax) {
    return -1;
  }
  for (i=0;i<DIOMapEntries && info->map[256*i+1];i++);
  strncpy(&info->map[256*i+1],string,255);
  info->map[256*i] = value;
  if (i >= DIOMapEntries) {
    DIOMapEntries++;
  }
  qsort(&info,DIOMapEntries,256,compar);
  return DIOMapEntries;
}

int DIOMapDelete(char *string) {
  int i = DIOMapLookup(string);
  for (i=0;i<DIOMapEntries;i++) {
    if (!strcmp(&info->map[256*i+1],string)) {
      break;
    }
  }
  if (i == DIOMapEntries) {
    return -1;
  }
  info->map[256*i+1] = 0;
  return 1;
}

void DIOMapDump() {
  int i;
  for (i=0;i<DIOMapEntries;i++) {
    //fprintf(stderr,"%s=%d\n",&info->map[256*i+1],info->map[256*i]);
  }
}

// parse str according to "name=value" and replace in map
// if value is a string, interpret as "name.value=1"
// returns 1 on success, or 0 on failure
static int ConfigLineAssign(char *str) {
  char *str0 = str;
  char *name,*cname;
  char tmp[256];
  unsigned value;
  int connectorNum,connectorPin;

  while (*str && *str != '=') str++;
  if (*str != '=') return 0;
  *str = 0;
  str++;
  value = strtoul(str,0,0);
  name = str0;
  if (value == 0 && str[0] != '0') {
    str[-1] = '.';
    value = 1;
  } else {
  }
  // automatic connector to connector name assignment
  if (2==sscanf(name,"attrib.Connector.%d.%d",
		&connectorNum,&connectorPin)) {
    cname = DIOMapLookupPartial("attrib.Connector.Name.",connectorNum);
    if (cname[0] != 0) {
      sprintf(tmp,"%s%d",cname+22,connectorPin);
      //fprintf(stderr,"%s=%d\n",tmp,value);
      DIOMapAdd(tmp,value);
    }
  }
  DIOMapAdd(name,value);
  return 1;
}

int DIOMapLoadFromFile(char *filename) {
  char buf[301],*str,*inst_str,*wc;
  int lines=0,n;
  FILE *f;

  f = fopen(filename,"r");
  if (!f) return 0;
  DIOMapLock(0);
  while (!feof(f)) {
    fgets(buf,300,f);
    if (buf[0] == 0) goto ConfigFileProcessEnd;
    str = buf;
    while (*str) str++;
    str--;
    while(str >= buf && (*str == '\n' || *str == '\r' || *str == ' ' || *str == '\t')) {
      *str = 0;
      str--;
    }
    lines++;
    if (buf[0] != '#') {
      ConfigLineAssign(buf);
    }
  }
 ConfigFileProcessEnd:
  DIOMapUnlock(0);
  fclose(f);
  return 1;  // or length of map array?
}
