#include <time.h>
#include "LocalSystem.h"
#include "DIOMap.h"
#include "Thread.h"
#include "ts.h"

SystemAPI LocalSystemAPI = {
  .Init = (void *)LocalSystemInit,
  .Fini = (void *)LocalSystemFini,
  .ClassCount = (void *)LocalSystemClassCount,
  .InstanceCount = (void *)LocalSystemInstanceCount,
  .APICount = (void *)LocalSystemAPICount,
  .LockCount = (void *)LocalSystemLockCount,
  .LockHolderInfo = (void *)LocalSystemLockHolderInfo,
  .ConnWaitInfo = (void *)LocalSystemConnWaitInfo,
  .CANBusGet = (void *)LocalSystemCANBusGet,
  .BuildInfo = (void *)LocalSystemBuildInfo,
  .ModelId = (void *)LocalSystemModelId,
  .BaseBoardId = (void *)LocalSystemBaseBoardId,
  .MapLength = (void *)LocalSystemMapLength,
  .MapGet = (void *)LocalSystemMapGet,
  .MapLookup = (void *)LocalSystemMapLookup,
  .MapLookupPartial = (void *)LocalSystemMapLookupPartial,
  .MapAdd = (void *)LocalSystemMapAdd,
  .MapDelete = (void *)LocalSystemMapDelete
};

void *LocalSystemInit(System *sys) {
  sys->API = &LocalSystemAPI;
  DIOMapInit();
  sys->InitStatus = 1;
  return sys;
}

void LocalSystemFini(System *sys) {
}

int LocalSystemClassCount(System *sys) {
  return ClassesCount;
}

int LocalSystemInstanceCount(System *sys,int class) {
  int i,n=0;
  if (class < 0 || class > ClassesCount) return 0;
  for (i=0;i<ARCHCOUNT;i++) {
    n += ArchInfos[i]->Instances[class];
  }
  return n;  
}

int LocalSystemAPICount(System *sys,int class) {
  switch (class) {                             
  case ClassSystem:
    return System_APICount;
  case ClassBus:
    return Bus_APICount;
  case ClassTime:
    return Time_APICount;
  case ClassPin:
    return Pin_APICount;
  case ClassDIORaw:
    return DIORaw_APICount;
  case ClassDIO:
    return DIO_APICount;
  case ClassTWI:
    return TWI_APICount;
  case ClassCAN:
    return CAN_APICount;
  case ClassSPI:                        
    return SPI_APICount;
  default:                                 
    return 0;
  }
}

int LocalSystemLockCount(System *sys) {
  return ThreadMutexCount();
}

int LocalSystemLockHolderInfo(System *sys,LockHolderInfo *lhi) {
}

int LocalSystemConnWaitInfo(System *sys,int max,ConnectionWaitInfo *info) {
  // TO DO: implement this using Thread moduloe
  // but, thread module does not expose this information, so add that!
  /*
  ThreadData *cur;
  int conns=0,n=0,i=max;

  pthread_mutex_lock(&watchlock);
  cur = head_thread;
  while (cur) {
    conns++;
    cur = cur->next;
  }
  if (conns > i) conns = i;
  cur = head_thread;
  while (cur && i--) {
    info[n].pid = cur->pid;
    info[n++].wait = cur->wait;
    cur = cur->next;
  }
  pthread_mutex_unlock(&watchlock);
  return conns;
  */
  return 0;
}

int LocalSystemCANBusGet(System *sys,int CANInst) {
  if (CANInst >= LocalSystemInstanceCount(sys,ClassCAN)) {
    return -1;
  } else {
    int i;
    for (i=0;i<ARCHCOUNT;i++) {
      if (ArchInfos[i]->Instances[ClassCAN] > CANInst) {
	return ArchInfos[i]->CANBusNum[CANInst];
      }
      CANInst -= ArchInfos[i]->Instances[ClassCAN];
    }
  }
  return -1; // should not reach here!
}

extern const char build[];
extern const char archstr[];
extern const int compiletime;
extern int startuptime;

void LocalSystemBuildInfo(System *sys,struct BuildInfo *buildinfo) {
  char *str;
  unsigned buildNumber = strtoul(build+7,&str,10);

  buildinfo->buildNumber = buildNumber;
  buildinfo->buildTime = compiletime ;
  buildinfo->uptime = time(0) - startuptime ;
  strcpy(buildinfo->hostname,str+1);
  strcpy(buildinfo->arch,archstr+1);
}

int LocalSystemModelId(System *sys) {
  return TSModelGet();
}

int BaseBoardIdGet();
int LocalSystemBaseBoardId(System *sys) {
  int bbid=0,ret;

  ThreadEnterBlock();
  bbid = BaseBoardIdGet();
  ThreadLeaveBlock();
  return bbid;
}

int LocalSystemMapLength(System *sys) {
  int i;
  int ret = DIOMapLock(SHARED);
  if (ret < 0) return ret;
  i = DIOMapLength(0);
  DIOMapUnlock(SHARED);
  return i;
}

int LocalSystemMapGet(System *sys,int num,char *dest,int *value) {
  char *s;
  int ret = DIOMapLock(SHARED);
  if (ret < 0) return ret;
  s = DIOMapGet(num,value);
  memcpy(dest,s,256);
  DIOMapUnlock(SHARED);
  return strlen(dest);
}

int LocalSystemMapLookup(System *sys,char *name) {
  int i;
  int ret = DIOMapLock(SHARED);
  if (ret < 0) return ret;
  i = DIOMapLookup(name);
  DIOMapUnlock(SHARED);
  return i;
}

int LocalSystemMapLookupPartial(System *sys,char *name,int value,char *dest) {
  char *s;
  int ret = DIOMapLock(SHARED);
  if (ret < 0) return ret;
  s = DIOMapLookupPartial(name,value);
  memcpy(dest,s+strlen(name),256);
  DIOMapUnlock(SHARED);
  return strlen(dest);
}

int LocalSystemMapAdd(System *sys,char *name,int value) {
  int i=0;
  int ret = DIOMapLock(0);
  if (ret < 0) return ret;
  if (DIOMapLookup(name) != -1) {
    DIOMapDelete(name);
    i++;
  }
  DIOMapAdd(name,value);
  i++;
  DIOMapUnlock(0);
  return i;
}

int LocalSystemMapDelete(System *sys,char *name) {
  int i;
  int ret = DIOMapLock(0);
  if (ret < 0) return ret;
  i = DIOMapDelete(name);
  DIOMapUnlock(0);
  return i;
}
/*
LocalSystem depends on DIOMap functions
DIOMapLength
DIOMapGet
DIOMapLookup
DIOMapLookupPartial
DIOMapDelete
DIOMapAdd

server
  ConfigLoadFile
    ConfigLineAssign
      DIOMapLookupPartial
      DIOMapAdd
  DIOMapAdd

dioctlServer
  dioctlDIOLookup:
    DIOMapLookupPartial
  dioctlDIONameLookup:
    DIOMapLookup
  dioctlDIONameAssign:
    DIOMapLookup
    DIOMapAdd
  dioctlDIONameRemove:
    DIOMapDelete
  dioctlDIONamesList:
    DIOMapLength
    DIOMapGet

BaseBoardIdGet
  DIOMapLookup

All operations on the DIOMap must be protected by a mutex.
In order to encapsulate the mutex in the DIOMap itself, we must
provide additional operations that do more work.
1. load map from file (we don't mutex protect this at the moment)
2. DIOMapAssign (Lookup + Add)
3. DIONamesList (ug.)

Defining network protocols using a Declarative Language
1. Define the protocol mapping the bytes in a structure to a function
   call and return.

There is only one other mutex in tsctl to worry about, and that one
controls access to the log.
We really should go through the Thread mutex allocation process to get
both the DIOMap lock and the log lock.

 */
