//#include <sched.h>
//#include "Obj.h"
#include "MMapBus.h"
#include "peekpoke.h"
#include "Bit.h"

#define LOCK_TYPE_SHARED
#define LOCK_CLASS MMapBus
#include "TSLock.h"

static BusAPI MMapBusAPI = {
  .Init = (void *)MMapBusInit,
  .Fini = (void *)MMapBusFini,
  .Lock = (void *)MMapBusLock,
  .Unlock = (void *)MMapBusUnlock,
  .Preempt = (void *)MMapBusPreempt,
  .Peek8 = (void *)MMapBusPeek8,
  .Poke8 = (void *)MMapBusPoke8,
  .Peek16 = (void *)MMapBusPeek16,
  .Poke16 = (void *)MMapBusPoke16,
  .Peek32 = (void *)MMapBusPeek32,
  .Poke32 = (void *)MMapBusPoke32,
  .BitGet8 = (void *)MMapBusBitGet8,
  .BitAssign8 = (void *)MMapBusBitAssign8,
  .BitSet8 = (void *)MMapBusBitSet8,
  .BitClear8 = (void *)MMapBusBitClear8,
  .BitGet16 = (void *)MMapBusBitGet16,
  .BitAssign16 = (void *)MMapBusBitAssign16,
  .BitSet16 = (void *)MMapBusBitSet16,
  .BitClear16 = (void *)MMapBusBitClear16,
  .BitGet32 = (void *)MMapBusBitGet32,
  .BitAssign32 = (void *)MMapBusBitAssign32,
  .BitSet32 = (void *)MMapBusBitSet32,
  .BitClear32 = (void *)MMapBusBitClear32,
  .PeekStream = (void *)MMapBusPeekStream,
  .PokeStream = (void *)MMapBusPokeStream,
  .Refresh = (void *)MMapBusRefresh,
  .Commit = (void *)MMapBusCommit,
  .BitToggle8 = (void *)MMapBusBitToggle8,
  .BitToggle16 = (void *)MMapBusBitToggle16,
  .BitToggle32 = (void *)MMapBusBitToggle32,
  .Assign8X = (void *)MMapBusAssign8X,
  .Assign16X = (void *)MMapBusAssign16X,
  .Assign32X = (void *)MMapBusAssign32X,
  .BitsGet8 = (void *)MMapBusBitsGet8,
  .BitsGet16 = (void *)MMapBusBitsGet16,
  .BitsGet32 = (void *)MMapBusBitsGet32
};

void *MMapBusInit2(MMapBus *bus,void *mem,int Pages) {
  if (bus->InitStatus > 0) return bus;
  MMapBusLockInit(bus,1);
  bus->API = &MMapBusAPI;
  bus->Mem = mem;
  bus->InitStatus = 1;
  return bus;
}

void *MMapBusInit(MMapBus *bus,int MemBase,int Pages) {
  if (bus->InitStatus > 0) return bus;
  bus->API = &MMapBusAPI;
  
  MMapBusLockInit(bus,1);
  if (Pages <= 0) {
    Pages = 1;
  }
  if (bus->Mem == 0) { // make multiple calls to Init safe
    bus->Mem = MemMap(MemBase,Pages);
  }
  bus->InitStatus = (bus->Mem == 0) ? MapBusErrorMapping : 1;
  return bus;
}

void MMapBusFini(MMapBus *bus) {
  if (bus->Mem) {
    MemUnmap(bus->Mem);
    bus->Mem = 0;
  }
  if (bus->InitStatus > 0) bus->InitStatus = 0;
}

void MMapBusRefresh(MMapBus *bus) {
  // nothing to do
}

void MMapBusCommit(MMapBus *bus,int forceall) {
  // nothing to do
}

//-----------------------------------------------------------------------------
#define PTR(x) ((char *)(MEM))

#ifdef DEBUG
#include <stdio.h>
#endif

static inline unsigned char _MMapBusPeek8(MMapBus *bus,int adrs) {
#ifdef DEBUG
  //  fprintf(stderr,"%p:Peek8 %d\n",bus,adrs);
#endif
  return bus->Mem[adrs];
}

static inline void _MMapBusPoke8(MMapBus *bus,int adrs,unsigned char val) {
#ifdef DEBUG
  //  fprintf(stderr,"%p:Poke8 %d,%X\n",bus,adrs,val);
#endif
  bus->Mem[adrs] = val;
}

static inline unsigned short _MMapBusPeek16(MMapBus *bus,int adrs) {
#ifdef DEBUG
  //  fprintf(stderr,"%p:Peek16 %d\n",bus,adrs);
#endif
  return ((volatile unsigned short *)(bus->Mem+adrs))[0];
}

static inline void _MMapBusPoke16(MMapBus *bus,int adrs,unsigned short val) {
#ifdef DEBUG
  //fprintf(stderr,"%p:Poke16 %d,%X\n",bus,adrs,val);
#endif
  ((volatile unsigned short *)(bus->Mem+adrs))[0] = val;
}

static inline unsigned _MMapBusPeek32(MMapBus *bus,int adrs) {
#ifdef DEBUG
  //  fprintf(stderr,"%p:Peek32 %d\n",bus,adrs);
#endif
  return ((volatile unsigned *)(bus->Mem+adrs))[0];
}

static inline void _MMapBusPoke32(MMapBus *bus,int adrs,unsigned val) {
#ifdef DEBUG
  //  fprintf(stderr,"%p:Poke32 %d,%X\n",bus,adrs,val);
#endif
  ((volatile unsigned *)(bus->Mem+adrs))[0] = val;
}
//-----------------------------------------------------------------------------
DefinitionFunctionsBit(MMapBus,unsigned char,8)
DefinitionFunctionsBit(MMapBus,unsigned short,16)
DefinitionFunctionsBit(MMapBus,unsigned,32)

void MMapBusPeekStream(MMapBus *bus,int adr,int dir,char *buf,int len) {
  int i;
  //fprintf(stderr,"PeekStream(%d,%d by %d)\n",adr,len,dir);
  for (i=0;i<len;i++) {
    buf[i] = _MMapBusPeek8(bus,adr);
    adr += dir;
  }
}

void MMapBusPokeStream(MMapBus *bus,int adr,int dir,char *buf,int len) {
  //((volatile unsigned char *)(PTR(MEM)+adr))[0] = val;
  int i;
  //fprintf(stderr,"PokeStream(%d,%d by %d)\n",adr,len,dir);
  for (i=0;i<len;i++) {
    _MMapBusPoke8(bus,adr,buf[i]);
    adr += dir;
  }
}
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
