#include "NetCANctl.h"
#include "Packet.h"

static CANAPI NetCANctlAPI = {
  .Init = (void *)NetCANctlInit,
  .Fini = (void *)NetCANctlFini,
  .Rx = (void *)NetCANctlRx,
  .Tx = (void *)NetCANctlTx,
  .BaudSet = (void *)NetCANctlBaudSet,
  .BaudGet = (void *)NetCANctlBaudGet
};

void *NetCANctlInit(NetCANctl *can,int socket) {
  if (can->InitStatus > 0) return can;
  can->API = &NetCANctlAPI;

  can->socket = socket;
  can->InitStatus = can->socket >= 0 ? 1 : -1;
  return can;
}

void NetCANctlFini(NetCANctl *can){
  if (can->InitStatus > 0) {
    close(can->socket);
    can->InitStatus = 0;
  }
}

int NetCANctlRx(NetCANctl *can,CANMessage *msg){
  int ret;
  if (can->InitStatus < 0) return -1;
  ret = PacketRecv(can->socket,msg,sizeof(CANMessage));
  return ret; 
}

int NetCANctlTx(NetCANctl *can,CANMessage *msg){
  int ret;
  if (can->InitStatus < 0) return -1;
  return PacketSend0(can->socket,msg,sizeof(CANMessage));
}

unsigned NetCANctlBaudSet(NetCANctl *can,unsigned opt_baud) {
  CANMessage msg;
  if (can->InitStatus < 0) return;
  msg.flags = FLAG_CONTROL;
  msg.id = 0;
  msg.length = 1;
  (((unsigned *)msg.data)[0]) = opt_baud;
  NetCANctlTx(can,&msg);
  //while (NetCANctlRx(can,&msg) > 0);
  //re-connect.  wait, how?
}

unsigned NetCANctlBaudGet(NetCANctl *can) {
  if (can->InitStatus < 0) return -1;
  return 1; // unknown
}
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
