#include <stdlib.h>
#include "QueueBytes.h"

QueueBytes *QueueBytesInit(QueueBytes *q,int length) {
  q->buffer = malloc(length);
  q->tail = q->head = 0;
  q->len = length;
  return q;
}

void QueueBytesFini(QueueBytes *q) {
  free(q->buffer);
}

int QueueBytesSize(QueueBytes *q) {
  return q->len;
}

int QueueBytesResize(QueueBytes *q,int length) {
  q->buffer = realloc(q->buffer,length);
  q->len = length;
  return 1;
}

int QueueBytesReady(QueueBytes *q) {
  if (q->tail == q->head) return 0; // empty
  if (q->tail > q->head) { // no wrap around present
    return q->tail - q->head;
  } else { // data wraps around
    return q->len - q->head + q->tail;
  }
}

int QueueBytesFree(QueueBytes *q) {
  return q->len - QueueBytesReady(q) - 1;
}

#if 1
int QueueBytesEnqueue(QueueBytes *q,char *bytes,int length) {
  if (length > QueueBytesFree(q)) return -1;
  while (length--) {
    q->buffer[q->tail++] = *(bytes++);
    if (q->tail >= q->len) q->tail = 0;
  }
  return 1;
}

int QueueBytesDequeue(QueueBytes *q,char *bytes,int length) {
  if (length > QueueBytesReady(q)) return -1;
  while (length--) {
    *(bytes++) = q->buffer[q->head++];
    if (q->head >= q->len) q->head = 0;
  }
}
#else
int QueueBytesEnqueue(QueueBytes *q,char *bytes,int length) {
  int len,n=0;
  char *dest;

  do { // this loop will be executed at most twice
    dest = QueueBytesEnqueuePrep(q,length,&len);
    if (len <= 0) return n;
    memcpy(dest,bytes,len);
    length -= len;
    bytes += len;
    n += len;
  } while (length > 0);
}

int QueueBytesDequeue(QueueBytes *q,char *bytes,int length) {
  int len,n=0;
  char *src;

  do { // this loop will be executed at most twice
    src = QueueBytesDequeuePrep(q,length,&len);
    if (len <= 0) return n;
    memcpy(bytes,src,len);
    length -= len;
    bytes += len;
    n += len;
  } while (length > 0);
}
#endif

// caller requests direct pointer to buffer to push "length" bytes
// caller commits to writing *actualLength bytes returned, which will
// be less than or equal to length
// if caller is uanble to write this many bytes, it must call
// QueueBytesUnEnqueue to unwind the bytes it did not push
char *QueueBytesEnqueuePrep(QueueBytes *q,int length,int *actualLength) {
  char *ret = q->buffer + q->tail;
  if (q->tail >= q->head) { // no wrap around present
    if (q->head > 0) {
      *actualLength = length > q->len-q->tail ? q->len-q->tail : length;
    } else {
      *actualLength = length > q->len-q->tail-1 ? q->len-q->tail-1 : length;
    }
  } else {
    *actualLength = length > q->head-q->tail-1 ? q->head-q->tail-1 : length;
  }
  q->tail += *actualLength;
  if (q->tail == q->len) q->tail = 0;
  return ret;
}

// caller requests direct pointer to buffer to pull "length" bytes
// caller commits to reading *actualLength bytes returned, which will
// be less than or equal to length
// if caller is uanble to read this many bytes, it must call
// QueueBytesUnDequeue to rewind the bytes it did not pull
char *QueueBytesDequeuePrep(QueueBytes *q,int length,int *actualLength) {
  char *ret = q->buffer + q->head;
  if (q->tail >= q->head) { // no wrap around present
    *actualLength = length > q->tail-q->head ? q->tail-q->head : length;
  } else {
    *actualLength = length > q->len-q->head ? q->len-q->head : length;
  }
  q->head += *actualLength;
  if (q->head == q->len) q->head = 0;
  return ret;
}

// assumes length is correct
int QueueBytesUnEnqueue(QueueBytes *q,int length) {
  q->tail -= length;
  return length;
}

// assumes length is correct
int QueueBytesUnDequeue(QueueBytes *q,int length) {
  q->head -= length;
  return length;
}
