#ifndef __SPI_H
#define __SPI_H

typedef struct SPI SPI;
typedef struct SPIAPI SPIAPI;

struct SPIAPI {
  void *(*Init)(void *,...);
  void (*Fini)(void *);
  int (*Lock)(void *,unsigned num,int flags);
  int (*Unlock)(void *,unsigned num,int flags);
  int (*Preempt)(void *);
  // "adrs" parameter indicates which CS to assert
  // if adrs < 0, then -adrs is used, and CS left asserted when done
  // otherwise, CS is de-asserted with last transfer
  int (*Write)(void *ob,int adrs,unsigned char *buf,int len);
  int (*Read)(void *ob,int adrs,unsigned char *buf,int len);
  int (*ReadWrite)(void *ob,int adrs,unsigned char *wbuf,unsigned char *rbuf, int len);
  int (*ClockSet)(void *ob,unsigned hz);
  int (*EdgeSet)(void *ob,int posedge);
};

struct SPI {
  SPIAPI *API;
  int InitStatus;
};

enum {
  NetSPI_Lock=0,
  NetSPI_Write=1,
  NetSPI_Read=2,
  NetSPI_ReadWrite=3,
  NetSPI_ClockSet=4,
  NetSPI_EdgeSet=5
};
#define SPI_APICount (1+NetSPI_EdgeSet)

#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
