#ifndef __SYSTEM_H
#define __SYSTEM_H

typedef struct System System;
typedef struct SystemAPI SystemAPI;

typedef struct LockHolderInfo {
  int pid;
  int count;
} LockHolderInfo;
typedef struct ConnectionWaitInfo {
  int pid;
  int wait; // lock waited for, or -1 if none
} ConnectionWaitInfo;

struct BuildInfo {
  unsigned buildNumber;
  int buildTime;
  unsigned uptime;
  char hostname[256], arch[256];
};

struct SystemAPI {
  void *(*Init)(void *,...);
  void (*Fini)(void *);
  int (*ClassCount)(void *);
  int (*InstanceCount)(void *,int);
  int (*APICount)(void *,int);
  int (*LockCount)(void *);
  int (*LockHolderInfo)(void *,LockHolderInfo *);
  int (*ConnWaitInfo)(void *,int max,ConnectionWaitInfo *info);
  int (*CANBusGet)(void *,int);
  void (*BuildInfo)(void *,struct BuildInfo *);
  int (*ModelId)(void *);
  int (*BaseBoardId)(void *);
  int (*MapLength)(void *);
  int (*MapGet)(void *,int,char *,int *);
  int (*MapLookup)(void *,char *);
  int (*MapLookupPartial)(void *,char *,int,char *);
  int (*MapAdd)(void *,char *,int);
  int (*MapDelete)(void *,char *);
};

struct System {
  SystemAPI *API;
  int InitStatus;
};

enum {
  NetSystem_ClassCount=0,
  NetSystem_InstanceCount=1,
  NetSystem_APICount=2,
  NetSystem_LockCount=3,
  NetSystem_LockHolderInfo=4,
  NetSystem_ConnWaitInfo=5,
  NetSystem_CANBusGet=6,
  NetSystem_BuildInfo=7,
  NetSystem_ModelId=8,
  NetSystem_BaseBoardId=9,
  NetSystem_MapLength=10,
  NetSystem_MapGet=11,
  NetSystem_MapLookup=12,
  NetSystem_MapLookupPartial=13,
  NetSystem_MapAdd=14,
  NetSystem_MapDelete=15
};
#define System_APICount (1+NetSystem_MapDelete)
#endif
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
