#include "WBWindowBus.h"
#include "Bit.h"

static BusAPI WBWindowBusAPI = {
  .Init = (void *)WBWindowBusInit,
  .Fini = (void *)WBWindowBusFini,
  .Lock = (void *)WBWindowBusLock,
  .Unlock = (void *)WBWindowBusUnlock,
  .Preempt = (void *)WBWindowBusPreempt,
  .Peek8 = (void *)WBWindowBusPeek8,
  .Poke8 = (void *)WBWindowBusPoke8,
  .Peek16 = (void *)WBWindowBusPeek16,
  .Poke16 = (void *)WBWindowBusPoke16,
  .Peek32 = (void *)WBWindowBusPeek32,
  .Poke32 = (void *)WBWindowBusPoke32,
  .BitGet8 = (void *)WBWindowBusBitGet8,
  .BitAssign8 = (void *)WBWindowBusBitAssign8,
  .BitSet8 = (void *)WBWindowBusBitSet8,
  .BitClear8 = (void *)WBWindowBusBitClear8,
  .BitGet16 = (void *)WBWindowBusBitGet16,
  .BitAssign16 = (void *)WBWindowBusBitAssign16,
  .BitSet16 = (void *)WBWindowBusBitSet16,
  .BitClear16 = (void *)WBWindowBusBitClear16,
  .BitGet32 = (void *)WBWindowBusBitGet32,
  .BitAssign32 = (void *)WBWindowBusBitAssign32,
  .BitSet32 = (void *)WBWindowBusBitSet32,
  .BitClear32 = (void *)WBWindowBusBitClear32,
  .PeekStream = (void *)WBWindowBusPeekStream,
  .PokeStream = (void *)WBWindowBusPokeStream,
  .Refresh = (void *)WBWindowBusRefresh,
  .Commit = (void *)WBWindowBusCommit,
  .BitToggle8 = (void *)WBWindowBusBitToggle8,
  .BitToggle16 = (void *)WBWindowBusBitToggle16,
  .BitToggle32 = (void *)WBWindowBusBitToggle32,
  .Assign8X = (void *)WBWindowBusAssign8X,
  .Assign16X = (void *)WBWindowBusAssign16X,
  .Assign32X = (void *)WBWindowBusAssign32X,
  .BitsGet8 = (void *)WBWindowBusBitsGet8,
  .BitsGet16 = (void *)WBWindowBusBitsGet16,
  .BitsGet32 = (void *)WBWindowBusBitsGet32
};

void *WBWindowBusInit(WBWindowBus *bus,void *sub,int adrs,int offset) {
  if (bus->InitStatus > 0) return bus;
  bus->sub = sub;
  bus->adrs = adrs;
  bus->offset = offset;

  if (bus->sub->InitStatus < 0) {
    bus->API->Fini(bus);
    bus->InitStatus = -1;
    return bus;
  }
  bus->API = &WBWindowBusAPI;
  bus->InitStatus = 1;
  return bus;
}

void WBWindowBusFini(WBWindowBus *bus) {
  bus->sub->API->Fini(bus->sub);
  if (bus->InitStatus > 0) bus->InitStatus = 0;
}

int WBWindowBusLock(WBWindowBus *bus,unsigned num,int flags) {
  return 1;
}
int WBWindowBusUnlock(WBWindowBus *bus,unsigned num,int flags) {
  return 1;
}
int WBWindowBusPreempt(WBWindowBus *bus) {
  return 1;
}

static unsigned char _WBWindowBusPeek8(WBWindowBus *bus,int adr) {
  unsigned char ret;

  adr += bus->offset;
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke16(bus->sub,bus->adrs,adr>>11);
  bus->sub->API->Poke16(bus->sub,bus->adrs+2,0xC000+(adr & 0x7FF));
  ret = bus->sub->API->Peek16(bus->sub,bus->adrs+4) & 0xFF;
  bus->sub->API->Unlock(bus->sub,0,0);
  return ret;
}

static void _WBWindowBusPoke8(WBWindowBus *bus,int adr, unsigned char dat) {
  adr += bus->offset;
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke16(bus->sub,bus->adrs,adr>>11);
  bus->sub->API->Poke16(bus->sub,bus->adrs+2,0xC000+(adr & 0x7FF));
  bus->sub->API->Poke16(bus->sub,bus->adrs+4,dat);
  bus->sub->API->Unlock(bus->sub,0,0);
}

static unsigned short _WBWindowBusPeek16(WBWindowBus *bus,int adr) {
  unsigned short ret;

  adr += bus->offset;
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke16(bus->sub,bus->adrs,adr>>11);
  bus->sub->API->Poke16(bus->sub,bus->adrs+2,0x8000+(adr & 0x7FF));
  ret = bus->sub->API->Peek16(bus->sub,bus->adrs+4);
  bus->sub->API->Unlock(bus->sub,0,0);
  return ret;
}

static void _WBWindowBusPoke16(WBWindowBus *bus,int adr, unsigned short dat) {
  adr += bus->offset;
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke16(bus->sub,bus->adrs,adr>>11);
  bus->sub->API->Poke16(bus->sub,bus->adrs+2,0x8000+(adr & 0x7FF));
  bus->sub->API->Poke16(bus->sub,bus->adrs+4,dat);
  bus->sub->API->Unlock(bus->sub,0,0);
}

static unsigned _WBWindowBusPeek32(WBWindowBus *bus,int adr) {
  unsigned ret;

  adr += bus->offset;
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke16(bus->sub,bus->adrs,adr>>11);
  bus->sub->API->Poke16(bus->sub,bus->adrs+2,0x6000+(adr & 0x7FF));
  ret = bus->sub->API->Peek16(bus->sub,bus->adrs+4);
  ret += (bus->sub->API->Peek16(bus->sub,bus->adrs+4) >> 16);
  bus->sub->API->Unlock(bus->sub,0,0);
  return ret;
}

static void _WBWindowBusPoke32(WBWindowBus *bus,int adr, unsigned dat) {
  adr += bus->offset;
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke16(bus->sub,bus->adrs,adr>>11);
  bus->sub->API->Poke16(bus->sub,bus->adrs+2,0x6000+(adr & 0x7FF));
  bus->sub->API->Poke16(bus->sub,bus->adrs+4,dat & 0xFFFF);
  bus->sub->API->Poke16(bus->sub,bus->adrs+4,dat >> 16);
  bus->sub->API->Unlock(bus->sub,0,0);
}

void WBWindowBusPeekStream(WBWindowBus *bus,int adr,int dir,char *buf,int len) {
  adr += bus->offset;
  // TO DO: optimize using split mode
  bus->sub->API->Lock(bus->sub,0,0);
  while (len > 0) {
    *buf++ = bus->sub->API->Peek8(bus->sub,adr);
    len --;
    adr += dir;
  }
  bus->sub->API->Unlock(bus->sub,0,0);
}

void WBWindowBusPokeStream(WBWindowBus *bus,int adr,int dir,char *buf,int len) {
  adr += bus->offset;
  // TO DO: optimize using split mode
  bus->sub->API->Lock(bus->sub,0,0);
  while (len > 0) {
    bus->sub->API->Poke8(bus->sub,adr, *buf++);
    adr += dir;
    len --;
  }
  bus->sub->API->Unlock(bus->sub,0,0);
}

void WBWindowBusRefresh(WBWindowBus *bus) {
  bus->sub->API->Refresh(bus->sub);
}

void WBWindowBusCommit(WBWindowBus *bus,int forceall) {
  bus->sub->API->Commit(bus->sub,forceall);
}

DefinitionFunctionsBit(WBWindowBus,unsigned char,8)
DefinitionFunctionsBit(WBWindowBus,unsigned short,16)
DefinitionFunctionsBit(WBWindowBus,unsigned,32)
