#include "WindowBus.h"
#include "Bit.h"

#define LOCK_TYPE_SHARED
#define LOCK_CLASS WindowBus
#include "TSLock.h"

static BusAPI WindowBusAPI = {
  .Init = (void *)WindowBusInit,
  .Fini = (void *)WindowBusFini,
  .Lock = (void *)WindowBusLock,
  .Unlock = (void *)WindowBusUnlock,
  .Preempt = (void *)WindowBusPreempt,
  .Peek8 = (void *)WindowBusPeek8,
  .Poke8 = (void *)WindowBusPoke8,
  .Peek16 = (void *)WindowBusPeek16,
  .Poke16 = (void *)WindowBusPoke16,
  .Peek32 = (void *)WindowBusPeek32,
  .Poke32 = (void *)WindowBusPoke32,
  .BitGet8 = (void *)WindowBusBitGet8,
  .BitAssign8 = (void *)WindowBusBitAssign8,
  .BitSet8 = (void *)WindowBusBitSet8,
  .BitClear8 = (void *)WindowBusBitClear8,
  .BitGet16 = (void *)WindowBusBitGet16,
  .BitAssign16 = (void *)WindowBusBitAssign16,
  .BitSet16 = (void *)WindowBusBitSet16,
  .BitClear16 = (void *)WindowBusBitClear16,
  .BitGet32 = (void *)WindowBusBitGet32,
  .BitAssign32 = (void *)WindowBusBitAssign32,
  .BitSet32 = (void *)WindowBusBitSet32,
  .BitClear32 = (void *)WindowBusBitClear32,
  .PeekStream = (void *)WindowBusPeekStream,
  .PokeStream = (void *)WindowBusPokeStream,
  .Refresh = (void *)WindowBusRefresh,
  .Commit = (void *)WindowBusCommit,
  .BitToggle8 = (void *)WindowBusBitToggle8,
  .BitToggle16 = (void *)WindowBusBitToggle16,
  .BitToggle32 = (void *)WindowBusBitToggle32,
  .Assign8X = (void *)WindowBusAssign8X,
  .Assign16X = (void *)WindowBusAssign16X,
  .Assign32X = (void *)WindowBusAssign32X,
  .BitsGet8 = (void *)WindowBusBitsGet8,
  .BitsGet16 = (void *)WindowBusBitsGet16,
  .BitsGet32 = (void *)WindowBusBitsGet32
};

void *WindowBusInit(WindowBus *bus,void *sub,int adrs,int data) {
  if (bus->InitStatus > 0) return bus;
  bus->sub = sub;
  bus->adrs = adrs;
  bus->data = data;

  if (bus->sub->InitStatus < 0) {
    bus->API->Fini(bus);
    bus->InitStatus = -1;
    return bus;
  }
  bus->API = &WindowBusAPI;
  bus->InitStatus = 1;
  return bus;
}

void WindowBusFini(WindowBus *bus) {
  bus->sub->API->Fini(bus->sub);
  if (bus->InitStatus > 0) bus->InitStatus = 0;
}

static unsigned char _WindowBusPeek8(WindowBus *bus,int adr) {
  unsigned char ret;

  bus->sub->API->Lock(bus->sub,0,SHARED);
  bus->sub->API->Poke8(bus->sub,bus->adrs,adr);
  ret = bus->sub->API->Peek8(bus->sub,bus->data) & 0xFF;
  bus->sub->API->Unlock(bus->sub,0,SHARED);
  return ret;
}

static void _WindowBusPoke8(WindowBus *bus,int adr, unsigned char dat) {
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke8(bus->sub,bus->adrs,adr);
  bus->sub->API->Poke8(bus->sub,bus->data,dat);
  bus->sub->API->Unlock(bus->sub,0,0);
}

static unsigned short _WindowBusPeek16(WindowBus *bus,int adr) {
  return 0; // unsupported
}

static void _WindowBusPoke16(WindowBus *bus,int adr, unsigned short dat) {
  // unsupported
}

static unsigned _WindowBusPeek32(WindowBus *bus,int adr) {
  // unsupported
}

static void _WindowBusPoke32(WindowBus *bus,int adr, unsigned dat) {
  // unsupported
}

void WindowBusPeekStream(WindowBus *bus,int adr,int dir,char *buf,int len) {
  bus->sub->API->Lock(bus->sub,0,SHARED);
  bus->sub->API->Poke8(bus->sub,bus->adrs,adr);
  while (len > 0) {
    *buf++ = bus->sub->API->Peek8(bus->sub,bus->data);
    len --;
  }
  bus->sub->API->Unlock(bus->sub,0,SHARED);
}

void WindowBusPokeStream(WindowBus *bus,int adr,int dir,char *buf,int len) {
  bus->sub->API->Lock(bus->sub,0,0);
  bus->sub->API->Poke8(bus->sub,bus->adrs,adr);
  while (len > 0) {
    bus->sub->API->Poke8(bus->sub,bus->data, *buf++);
    len --;
  }
  bus->sub->API->Unlock(bus->sub,0,0);
}

void WindowBusRefresh(WindowBus *bus) {
  bus->sub->API->Refresh(bus->sub);
}

void WindowBusCommit(WindowBus *bus,int forceall) {
  bus->sub->API->Commit(bus->sub,forceall);
}

DefinitionFunctionsBit(WindowBus,unsigned char,8)
DefinitionFunctionsBit(WindowBus,unsigned short,16)
DefinitionFunctionsBit(WindowBus,unsigned,32)
