#include <time.h>
#include "arch.h"
#include "Log.h"

#undef ARCH_FIRST
#undef ARCH_NEXT
#define ARCH_FIRST(x) TEMPLATE(&x,_ArchInfo)
#define ARCH_NEXT(x) COMMA_TEMPLATE(&x,_ArchInfo)
ArchInfo *ArchInfos[ARCHCOUNT] = { ARCHLIST };
int startuptime;

// DIOCount is designed to be used before DIO is initialized
unsigned DIOCount() {
  int n=0;
  unsigned num=0;
  DIORaw *dio = DIORawInit(n);
  while (dio) {
    num += dio->API->Count(dio);
    dio = DIORawInit(++n);
  }
  return num;
}

#ifdef __AggregateDIO_H
DIO *DIOInit0(AggregateDIO *DIO0) {
  int i,j,total = 0;
  DIO **pDIO;
  int n = 0,offset, *SubCount;
    
  for (i=0;i<ARCHCOUNT;i++) {
    total += ArchInfos[i]->Instances[ClassDIORaw];
  }
  pDIO = malloc(sizeof(DIO *) * total);
  SubCount = malloc(sizeof(int) * total);
  for (i=0;i<ARCHCOUNT;i++) {
    offset=ArchInfos[i]->Instances[ClassDIO]
      -ArchInfos[i]->Instances[ClassDIORaw];
    for (j=offset;j<ArchInfos[i]->Instances[ClassDIO];j++) {
      pDIO[n] = ArchInfos[i]->DIOInit(0,j);
      if (pDIO[n]) n++;
    }
  }
  return AggregateDIOInit(DIO0,total,PinInit(0),SubCount,pDIO);
}
#endif

void ArchInit() {
  static int initialized = 0;
  int i;
  if (initialized) return;
  LogInit(0);
  startuptime = time(0);
  initialized = 1;
  //LockInit(0);
  for (i=0;i<ARCHCOUNT;i++) {
    ArchInfos[i]->ArchInit();
  }  
}

#define CLASSINIT(class)					\
  class *class##Init(int inst) {				\
    int i,i0=inst;						\
    class *ret=0;						\
    ArchInit();							\
    i = 0;							\
    while (i < ARCHCOUNT) {					\
      if (inst < ArchInfos[i]->Instances[Class##class]) break;	\
      inst -= ArchInfos[i]->Instances[Class##class];		\
      i++;							\
    }								\
    if (i == ARCHCOUNT) return ret;				\
    ret = ArchInfos[i]->class##Init(ret,inst);			\
    while (i < ARCHCOUNT) {					\
      ret = ArchInfos[i]->class##Init(ret,-1-inst);		\
    }								\
    return ret;							\
  }

//CLASSINIT(System)
  System *SystemInit(int inst) {				
    int i,i0=inst;						
    System *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassSystem]) break;		
      inst -= ArchInfos[i]->Instances[ClassSystem];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->SystemInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->SystemInit(ret,-1-inst);		
    }								
    return ret;							
  }
//CLASSINIT(Time)
  Time *TimeInit(int inst) {				
    int i,i0=inst;						
    Time *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassTime]) break;		
      inst -= ArchInfos[i]->Instances[ClassTime];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->TimeInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->TimeInit(ret,-1-inst);		
    }								
    return ret;							
  }

//CLASSINIT(Bus)

  Bus *BusInit(int inst) {				
    int i,i0=inst;						
    Bus *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassBus]) break;		
      inst -= ArchInfos[i]->Instances[ClassBus];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->BusInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->BusInit(ret,-1-inst);		
    }								
    return ret;							
  }
//CLASSINIT(DIO)
  DIO *DIOInit(int inst) {				
    int i,i0=inst;						
    DIO *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassDIO]) break;		
      inst -= ArchInfos[i]->Instances[ClassDIO];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->DIOInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->DIOInit(ret,-1-inst);		
    }								
    return ret;							
  }
//CLASSINIT(DIORaw)
  DIORaw *DIORawInit(int inst) {				
    int i,i0=inst;						
    DIORaw *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassDIORaw]) break;		
      inst -= ArchInfos[i]->Instances[ClassDIORaw];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->DIORawInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->DIORawInit(ret,-1-inst);		
    }								
    return ret;							
  }
//CLASSINIT(Pin)
  Pin *PinInit(int inst) {				
    int i,i0=inst;						
    Pin *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassPin]) break;		
      inst -= ArchInfos[i]->Instances[ClassPin];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->PinInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->PinInit(ret,-1-inst);		
    }								
    return ret;							
  }

//CLASSINIT(CAN)
 CAN *CANInit(int inst) {				
    int i,i0=inst;						
    CAN *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassCAN]) break;		
      inst -= ArchInfos[i]->Instances[ClassCAN];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->CANInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->CANInit(ret,-1-inst);		
    }								
    return ret;							
  }
//CLASSINIT(SPI)
 SPI *SPIInit(int inst) {				
    int i,i0=inst;						
    SPI *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassSPI]) break;		
      inst -= ArchInfos[i]->Instances[ClassSPI];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->SPIInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->SPIInit(ret,-1-inst);		
    }								
    return ret;							
  }
//CLASSINIT(TWI)
 TWI *TWIInit(int inst) {				
    int i,i0=inst;						
    TWI *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassTWI]) break;		
      inst -= ArchInfos[i]->Instances[ClassTWI];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->TWIInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->TWIInit(ret,-1-inst);		
    }	
    return ret;							
  }
    //CLASSINIT(AIO)
  AIO *AIOInit(int inst) {				
    int i,i0=inst;						
    AIO *ret=0;						
    ArchInit();							
    i = 0;							
    while (i < ARCHCOUNT) {					
      if (inst < ArchInfos[i]->Instances[ClassAIO]) break;		
      inst -= ArchInfos[i]->Instances[ClassAIO];			
      i++;							
    }								
    if (i == ARCHCOUNT) return ret;				
    ret = ArchInfos[i]->AIOInit(ret,inst);			
    while (++i < ARCHCOUNT) {					
      ret = ArchInfos[i]->AIOInit(ret,-1-inst);		
    }								
    return ret;							
  }
