#include <stdlib.h>
#include "NetBus.c"
#include "NetTime.c"
#include "NetPin.c"
#include "NetDIORaw.c"
#include "NetDIO.c"
#include "NetTWI.c"
#include "NetSPI.c"
#include "NetSPIctl.c"
#include "NetCAN.c"
#include "NetSystem.c"
#include "NetCANctl.c"
//#include "NetAIO.c"
#include "AIO.h"
#include "Net.c"
#include "HashTable.h"
#include "IteratorHashTable.h"

struct NetEntry {
  char *host;
  int socket;
  int useCount;
};
typedef struct NetEntry NetEntry;

HashTable *ht = 0;

static int NetSocketNew(char *host) {
  NetEntry *ne;
  int socket;

  if (!ht) {
    ht = HashTableNewASCIIZ();
  } else {
    if ((ne=HashTableLookup(ht,host))) {
      ne->useCount++;
      return ne->socket;
      // if multiple calls are made to the same host with different
      // modes, only the first one is used; the rest are ignored
    }
  }
  socket = ClientSocketNew(host,5001);
  if (socket < 0) return socket;
  ne = malloc(sizeof(NetEntry));
  ne->host = host; // caller can't de-allocate
  ne->socket = socket;
  ne->useCount = 1;
  HashTableInsert(ht,host,ne);
  return socket;
}

static void NetSocketFini(int socket) {
  IteratorHashTable *itr = IteratorHashTableNew(ht);
  NetEntry *entry;
  do {
    entry = IteratorHashTableValue(itr);
    if (entry->socket == socket) {
      if (--entry->useCount == 0) {
	char *key = IteratorHashTableKey(itr);
	free(HashTableRemove(ht,key));
      }
      goto NetSocketFiniDone;
    }
  } while (IteratorHashTableNext(itr));
  // should not reach here!
 NetSocketFiniDone:
  IteratorHashTableFini(itr);
}

Time *Net_TimeInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetTime *obj = malloc(sizeof(NetTime));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetTimeInit(obj,socket,inst,mode,q,cmds);
}

Bus *Net_BusInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetBus *obj = malloc(sizeof(NetBus));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetBusInit(obj,socket,inst,mode,q,cmds);
}

Pin *Net_PinInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetPin *obj = malloc(sizeof(NetPin));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetPinInit(obj,socket,inst,mode,q,cmds);
}

CAN* Net_CANInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetCAN *obj = malloc(sizeof(NetCAN));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetCANInit(obj,socket,inst,mode,q,cmds);
}

CAN *Net_CANctlInit(int inst,char *host,int mode){
  int socket = ClientSocketNew(host,7552+inst);
  if (socket < 0) return 0;
  NetCANctl *obj = malloc(sizeof(NetCANctl));
  memset(obj,0,sizeof(*obj));

  return NetCANctlInit(obj,socket);
}

DIORaw *Net_DIORawInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetDIORaw *obj = malloc(sizeof(NetDIORaw));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetDIORawInit(obj,socket,inst,mode,q,cmds);
}

DIO *Net_DIOInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetDIO *obj = malloc(sizeof(NetDIO));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetDIOInit(obj,socket,inst,mode,q,cmds);
}

TWI *Net_TWIInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetTWI *obj = malloc(sizeof(NetTWI));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetTWIInit(obj,socket,inst,mode,q,cmds);
}

SPI *Net_SPIInit(int inst,char *host,int mode){
  int socket = NetSocketNew(host);
  if (socket < 0) return 0;
  NetSPI *obj = malloc(sizeof(NetSPI));
  Packet *q = malloc(sizeof(Packet));
  int *cmds = malloc(sizeof(int));
  q->len=0; *cmds=0;
  memset(obj,0,sizeof(*obj));

  return NetSPIInit(obj,socket,inst,mode,q,cmds);
}

SPI *Net_SPIctlInit(int inst,char *host,int mode){
  int socket = ClientSocketNew(host,7552+inst);
  if (socket < 0) return 0;
  NetSPIctl *obj = malloc(sizeof(NetSPIctl));
  memset(obj,0,sizeof(*obj));

  return NetSPIctlInit(obj,socket,inst);
}

AIO *Net_AIOInit(int inst,char *host,int mode){
  return 0;
  //int socket = ClientSocketNew(host,5001);
  //if (socket < 0) return 0;
  //NetAIO *obj = malloc(sizeof(NetAIO));
  //Packet *q = malloc(sizeof(Packet));
  //int *cmds = malloc(sizeof(int));
  //memset(obj,0,sizeof(*obj));

  //return NetAIOInit(obj,socket,inst,mode,q,cmds);
}

void Net_TimeFini(NetTime *t) {
  NetSocketFini(t->socket);
  return NetTimeFini(t);
}

void Net_BusFini(NetBus *bus) {
  NetSocketFini(bus->socket);
  return NetBusFini(bus);
}

void Net_PinFini(NetPin *pin) {
  NetSocketFini(pin->socket);
  return NetPinFini(pin);
}

void Net_CANFini(NetCAN *can) {
  NetSocketFini(can->socket);
  return NetCANFini(can);
}

void Net_DIORawFini(NetDIORaw *dioraw) {
  NetSocketFini(dioraw->socket);
  return NetDIORawFini(dioraw);
}

void Net_DIOFini(NetDIO *dio) {
  NetSocketFini(dio->socket);
  return NetDIOFini(dio);
}

void Net_TWIFini(NetTWI *twi) {
  NetSocketFini(twi->socket);
  return NetTWIFini(twi);
}

void Net_SPIFini(NetSPI *spi) {
  NetSocketFini(spi->socket);
  return NetSPIFini(spi);
}

//void NetAIOFini(NetAIO *aio) {
//}

