#!/usr/bin/python

# usage:
# TestBus.py <host>

# The hosts to be specified must implement a "test bus" in bus instance 6
# which provides a page of read/write memory. 

# The test exercises all the Bus API functions

# Commit and Refresh functions require the TS-4500 due to assumptions
# about the format of the CacheBus

# During the test progress is displayed in the form of:
# server-behavior#-command#
#
# The behavior# will increment while command# will decrement

from Bus import *
import sys
import random
import time
import os
bus = Bus(tsctl(sys.argv[1]),1)
bus0 = Bus(tsctl(sys.argv[1]),0)
bus1 = Bus(tsctl(sys.argv[1]),3)
#bus.tsctl.s.settimeout(1)
#bus0.tsctl.s.settimeout(1)
#bus1.tsctl.s.settimeout(1)

server=sys.argv[0]

def cmd():
    global cmdn
    sys.stderr.write(server+"-"+str(test)+"-"+str(total-cmdn)+"\r")
    cmdn = cmdn + 1

test=0
cmdn=0
total=766
cmd()
n=random.randint(0,4294967295)
rc=bus.Lock(WaitLock)
if rc < 0:
    print "WaitLock returned "+str(rc)
    exit(1)
bus.Poke32(0,n)
cmd()
n2=bus.Peek32(0)
cmd()
if n != n2:
    print("Peek32 failed to read back what Poke32 wrote (got "+hex(n2)+
          " not "+hex(n))
    exit(1)
n2=bus.Peek16(0)
if n&0xFFFF != n2:
    print("Peek16 failed to read back what Poke32 wrote (got "+hex(n2)+
          " not "+hex(n&0xFFFF))
    exit(1)
n2=bus.Peek8(0)
if n&0xFF != n2:
    print("Peek8 failed to read back what Poke32 wrote (got "+hex(n2)+
          " not "+hex(n&0xFF))
    exit(1)
test=2
n=random.randint(0,65535)
bus.Poke16(0,n)
cmd()
n2=bus.Peek16(0)
cmd()
if n != n2:
    print("Peek16 failed to read back what Poke16 wrote (got "+hex(n2)+
          " not "+hex(n))
    exit(1)
n2=bus.Peek8(0)
cmd()
if n&0xFF != n2:
    print("Peek8 failed to read back what Poke16 wrote (got "+hex(n2)+
          " not "+hex(n&0xFF))
    exit(1)
test=3
n=random.randint(0,255)
bus.Poke8(0,n)
cmd()
n2=bus.Peek8(0)
cmd()
if n != n2:
    print("Peek8 failed to read back what Poke8 wrote (got "+hex(n2)+
          " not "+hex(n))
    exit(1)
test=4
n=random.randint(0,4294967295)
bus.Poke32(0,n)
cmd()
for i in range(32):
    bit=bus.BitGet32(0,i)
    cmd()
    tmp=((n & (1<<i))>>i)
    if (tmp != bit):
        print("BitGet32 "+str(i)+" of "+hex(n)+" got "+str(bit)+
              ", expected "+str(tmp))
        exit(1)
for i in range(16):
    bit=bus.BitGet16(0,i)
    cmd()
    tmp=((n & (1<<i))>>i)
    if (tmp != bit):
        print("BitGet16 "+str(i)+" of "+hex(n)+" got "+str(bit)+
              ", expected "+str(tmp))
        exit(1)

for i in range(8):
    bit=bus.BitGet8(0,i)
    cmd()
    tmp=((n & (1<<i))>>i)
    if (tmp != bit):
        print("BitGet8 "+str(i)+" of "+hex(n)+" got "+str(bit)+
              ", expected "+str(tmp))
        exit(1)

test=5
for i in range(32):
    bus.Poke32(0,0)
    cmd()
    bus.BitSet32(0,i)
    cmd()
    n=bus.Peek32(0)
    cmd()
    tmp=1<<i
    if (n != tmp):
        print "BitSet32 "+str(i)+" got "+hex(n)+", expected "+hex(tmp)
        exit(1)
for i in range(16):
    bus.Poke16(0,0)
    cmd()
    bus.BitSet16(0,i)
    cmd()
    n=bus.Peek16(0)
    cmd()
    tmp=1<<i
    if (n != tmp):
        print "BitSet16 "+str(i)+" got "+hex(n)+", expected "+hex(tmp)
        exit(1)
for i in range(8):
    bus.Poke8(0,0)
    cmd()
    bus.BitSet8(0,i)
    cmd()
    n=bus.Peek8(0)
    cmd()
    tmp=1<<i
    if (n != tmp):
        print "BitSet8 "+str(i)+" got "+hex(n)+", expected "+hex(tmp)
        exit(1)

test=6
for i in range(32):
    bus.Poke32(0,0xFFFFFFFF)
    cmd()
    bus.BitClear32(0,i)
    cmd()
    n=bus.Peek32(0)
    cmd()
    tmp=~(1<<i)&0xFFFFFFFF
    if (n != tmp):
        print "BitClear32 "+str(i)+" got "+hex(n)+", expected "+hex(tmp)
        exit(1)
for i in range(16):
    bus.Poke16(0,0xFFFF)
    cmd()
    bus.BitClear16(0,i)
    cmd()
    n=bus.Peek16(0)
    cmd()
    tmp=~(1<<i)&0xFFFF
    if (n != tmp):
        print "BitClear16 "+str(i)+" got "+hex(n)+", expected "+hex(tmp)
        exit(1)
for i in range(8):
    bus.Poke8(0,0xFF)
    cmd()
    bus.BitClear8(0,i)
    cmd()
    n=bus.Peek8(0)
    cmd()
    tmp=~(1<<i)&0xFF
    if (n != tmp):
        print "BitClear8 "+str(i)+" got "+hex(n)+", expected "+hex(tmp)
        exit(1)

test=7
for i in range(32):
    n=bus.BitGet32(0,i)
    cmd()
    n2=bus.BitToggle32(0,i)
    cmd()
    if n != (not n2):
        print "BitToggle32 "+str(i)+" got "+str(n2)+", expected "+str(not n)
        exit(1)
    n3=bus.BitGet32(0,i)
    cmd()
    if n2 != n3:
        print "BitToggle/Get32 "+str(i)+" got "+str(n3)+", expected "+str(n2)
        exit(1)

test=8
n=random.randint(0,4294967295)
for i in range(32):
    bus.BitAssign32(0,i,((n&(1<<i))>>i))
    cmd()
n2=bus.Peek32(0)
cmd()
if (n != n2):
    print("Peek32 failed to read back what BitAssign32 wrote (got "+
          hex(n2)+", not "+hex(n))
    exit(1)
n=random.randint(0,65535)
for i in range(16):
    bus.BitAssign16(0,i,((n&(1<<i))>>i))
    cmd()
n2=bus.Peek16(0)
cmd()
if (n != n2):
    print("Peek16 failed to read back what BitAssign16 wrote (got "+
          hex(n2)+", not "+hex(n))
    exit(1)
n=random.randint(0,255)
for i in range(8):
    bus.BitAssign8(0,i,((n&(1<<i))>>i))
    cmd()
n2=bus.Peek8(0)
cmd()
if (n != n2):
    print("Peek8 failed to read back what BitAssign8 wrote (got "+
          hex(n2)+", not "+hex(n))
    exit(1)

test=9
data1="\x00\x01\x02\x03\x04\x05\x06\x07"
bus.PokeStream(0,1,data1)
cmd()
n=bus.Peek32(0)
cmd()
if n != 0x03020100:
    print "PokeStream failed to read back with Peek32 at offset 0"
    exit()
n=bus.Peek32(4)
cmd()
if n != 0x07060504:
    print "PokeStream failed to read back with Peek32 at offset 4"
    exit()
data2=bus.PeekStream(0,1,8)
cmd()
if (data1 != data2):
    print "PokeStream failed to read back with Peekstream"
    exit()

bus.Lock(DoRelease)

test=10
for i in range(2):
    rc=bus1.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    bus1.Refresh()
    bus1.Lock(DoRelease)
    cmd()
    rc=bus0.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n1=bus0.BitGet16(0x62,15)
    bus0.Lock(DoRelease)
    cmd()
    rc=bus1.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n2=bus1.BitToggle16(~0x62 & 0xFFFFFFFF,15)
    n4=bus1.BitGet16(0x62,15)
    bus1.Lock(DoRelease)
    cmd()
    if n2 == n4:
        print "Cache/Bus diagreement (1)"
        exit(1)

    if n1 != (not n2):
        print "Cache/Bus disagreement, another process may have interfered"
        exit(1)
    rc=bus0.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n3=bus0.BitGet16(0x62,15)
    bus0.Lock(DoRelease)
    if n1 != n3:
        print "BitToggle16 in cache propogated to Bus with no commit"
        exit(1)
    rc=bus1.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    bus1.Commit(0)
    bus1.Lock(DoRelease)
    rc=bus0.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n3=bus0.BitGet16(0x62,15)
    bus0.Lock(DoRelease)
    if n2 != n3:
        print "Commit "+str(i)+" failed to propogate BitToggle16 to Bus"
        print str(n1)+":"+str(n2)+":"+str(n3)
        exit(1)

test=11
for i in range(2):
    rc=bus0.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n1=bus0.BitToggle16(0x62,15)
    bus0.Lock(DoRelease)
    cmd()
    rc=bus1.Lock(WaitLock)
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n2=bus1.BitGet16(~0x62 & 0xFFFFFFFF, 15)
    cmd()
    if n1 != (not n2):
        print "BitToggle16 in Bus propogate to Cache without Refresh"
        exit(1)
    bus1.Refresh()
    cmd()
    if rc < 0:
        print "WaitLock returned "+str(rc)
        exit(1)
    n2=bus1.BitGet16(~0x62 & 0xFFFFFFFF, 15)
    bus1.Lock(DoRelease)
    cmd()
    if n1 != n2:
        print "BitToggle16 in Bus failed propogate to Cache with Refresh"
        exit(1)


# Author: Michael Schmidt (michael@embeddedARM.com)
# Copyright (c) 2011, Technologic Systems, All Rights Reserved
# Refer to the COPYRIGHT file provided with this project for licensing terms.
