#!/usr/bin/python

# usage:
# TestBus.py <host>

# This program tests the non-blocking and queued protocol modes.

# For non-blocking mode, server delays are inserted between opcodes
# and we check (via a separate connection) that the delayed opcodes
# do not get executed until after the delay.

# For queued mode, we send non-blocking opcodes and observe (via a
# separate connection) that the opcodes are not executed until after
# we return from a subsequent blocking opcode.

from Bus import *
import sys

# non-blocking Bus functions are:
# Poke8/16/32, BitAssign8/16/32, BitSet8/16/32, BitClear8/16/32
# PokeStream, Refresh, Commit

bus1 = Bus(tsctl(sys.argv[1], NetModeQueued),1)
bus2 = Bus(tsctl(sys.argv[1]),1)

bus2.Poke8(0,0)
bus1.Poke8(0,1)
n=bus2.Peek8(0)
if n != 0:
    print "Error: queued Poke8 prematurely set [0] to (1?) "+hex(n)
    exit(1)
n=bus1.Peek8(0)
if n != 1:
    print "Error: queued Poke8 did not flush, expected 1, got "+hex(n)
    exit(1)
n=bus2.Peek8(0)
if n != 1:
    print "Error: queued Poke8 flush not seen, expected 1, got "+hex(n)
    exit(1)

bus2.Poke16(0,0)
bus1.Poke16(0,0x1234)
n=bus2.Peek16(0)
if n != 0:
    print "Error: queued Poke16 prematurely set [0] to (0x1234?) "+hex(n)
    exit(1)
n=bus1.Peek16(0)
if n != 0x1234:
    print "Error: queued Poke16 did not flush, expected 0x1234, got "+hex(n)
    exit(1)
n=bus2.Peek16(0)
if n != 0x1234:
    print "Error: queued Poke16 flush not seen, expected 0x1234, got "+hex(n)
    exit(1)

bus2.Poke32(0,0)
bus1.Poke32(0,0x0155AA44)
n=bus2.Peek32(0)
if n != 0:
    print "Error: queued Poke32 prematurely set [0] to (0x0155AA44?) "+hex(n)
    exit(1)
n=bus1.Peek32(0)
if n != 0x0155AA44:
    print "Error: queued Poke32 did not flush 0x0155AA44, got "+hex(n)
    exit(1)
n=bus2.Peek32(0)
if n != 0x0155AA44:
    print "Error: queued Poke32 flush 0x0155AA44 not seen, got "+hex(n)
    exit(1)

bus2.Poke8(0,0)
for i in range(8):
    bus1.BitAssign8(0,i,1)
n=bus2.Peek8(0)
if n != 0:
    print "Error: queued BitAssign8 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek8(0)
if n != 0xFF:
    print "Error: queued BitAssign8 did not flush, expected 0xFF, got "+hex(n)
    exit(1)
n=bus2.Peek8(0)
if n != 0xFF:
    print "Error: queued BitAssign8 flush not seen, expected 0xFF, got "+hex(n)
    exit(1)

bus2.Poke16(0,0)
for i in range(16):
    bus1.BitAssign16(0,i,1)
n=bus2.Peek16(0)
if n != 0:
    print "Error: queued BitAssign16 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek16(0)
if n != 0xFFFF:
    print "Error: queued BitAssign16 did not flush, expected 0xFFFF, got "+hex(n)
    exit(1)
n=bus2.Peek16(0)
if n != 0xFFFF:
    print "Error: queued BitAssign16 flush not seen, expected 0xFFFF, got "+hex(n)
    exit(1)

bus2.Poke32(0,0)
for i in range(32):
    bus1.BitAssign32(0,i,1)
n=bus2.Peek32(0)
if n != 0:
    print "Error: queued BitAssign32 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek32(0)
if n != 0xFFFFFFFF:
    print "Error: queued BitAssign32 did not flush, expected 0xFFFFFFFF, got "+hex(n)
    exit(1)
n=bus2.Peek32(0)
if n != 0xFFFFFFFF:
    print "Error: queued BitAssign32 flush not seen, expected 0xFFFFFFFF, got "+hex(n)
    exit(1)


bus2.Poke8(0,0)
for i in range(8):
    bus1.BitSet8(0,i)
n=bus2.Peek8(0)
if n != 0:
    print "Error: queued BitSet8 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek8(0)
if n != 0xFF:
    print "Error: queued BitSet8 did not flush, expected 0xFF, got "+hex(n)
    exit(1)
n=bus2.Peek8(0)
if n != 0xFF:
    print "Error: queued BitSet8 flush not seen, expected 0xFF, got "+hex(n)
    exit(1)

bus2.Poke16(0,0)
for i in range(16):
    bus1.BitSet16(0,i)
n=bus2.Peek16(0)
if n != 0:
    print "Error: queued BitSet16 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek16(0)
if n != 0xFFFF:
    print "Error: queued BitSet16 did not flush, expected 0xFFFF, got "+hex(n)
    exit(1)
n=bus2.Peek16(0)
if n != 0xFFFF:
    print "Error: queued BitSet16 flush not seen, expected 0xFFFF, got "+hex(n)
    exit(1)

bus2.Poke32(0,0)
for i in range(32):
    bus1.BitSet32(0,i)
n=bus2.Peek32(0)
if n != 0:
    print "Error: queued BitSet32 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek32(0)
if n != 0xFFFFFFFF:
    print "Error: queued BitSet32 did not flush, expected 0xFFFFFFFF, got "+hex(n)
    exit(1)
n=bus2.Peek32(0)
if n != 0xFFFFFFFF:
    print "Error: queued BitSet32 flush not seen, expected 0xFFFFFFFF, got "+hex(n)
    exit(1)


bus2.Poke8(0,0xFF)
for i in range(8):
    bus1.BitClear8(0,i)
n=bus2.Peek8(0)
if n != 0xFF:
    print "Error: queued BitClear8 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek8(0)
if n != 0:
    print "Error: queued BitClear8 did not flush, expected 0, got "+hex(n)
    exit(1)
n=bus2.Peek8(0)
if n != 0:
    print "Error: queued BitClear8 flush not seen, expected 0, got "+hex(n)
    exit(1)

bus2.Poke16(0,0xFFFF)
for i in range(16):
    bus1.BitClear16(0,i)
n=bus2.Peek16(0)
if n != 0xFFFF:
    print "Error: queued BitClear16 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek16(0)
if n != 0:
    print "Error: queued BitClear16 did not flush, expected 0, got "+hex(n)
    exit(1)
n=bus2.Peek16(0)
if n != 0:
    print "Error: queued BitClear16 flush not seen, expected 0, got "+hex(n)
    exit(1)

bus2.Poke32(0,0xFFFFFFFF)
for i in range(32):
    bus1.BitClear32(0,i)
n=bus2.Peek32(0)
if n != 0xFFFFFFFF:
    print "Error: queued BitClear32 prematurely set [0] to "+hex(n)
    exit(1)
n=bus1.Peek32(0)
if n != 0:
    print "Error: queued BitClear32 did not flush, expected 0, got "+hex(n)
    exit(1)
n=bus2.Peek32(0)
if n != 0:
    print "Error: queued BitClear32 flush not seen, expected 0, got "+hex(n)
    exit(1)

bus2.Poke32(0,0)
bus1.PokeStream(0,1,"\x00\x01\x02\x03")
n=bus2.Peek32(0)

if n != 0:
    print "Error: queued PokeStream prematurely set to "+hex(n)
    exit(1)
n=bus1.PeekStream(0,1,4)
if n != "\x00\x01\x02\x03":
    print "Error: PeekStream did not flush, got "+repr(n)
    exit(1)

# Author: Michael Schmidt (michael@embeddedARM.com)
# Copyright (c) 2011, Technologic Systems, All Rights Reserved
# Refer to the COPYRIGHT file provided with this project for licensing terms.
