#!/usr/bin/python

# usage:
# TestCAN.py <host1> <host2> [show stats]

# This program requires two boards connected via CAN
# Instance 0 of CAN is assumed on both boards.
# A message is queued to transmit on both sides simultaneously,
# and then we wait to receive on both sides.  We repeat this
# process for the number of times specified in the "runs" variable,
# then if the optional 3rd parameter is present, we display
# statistics on how long it took for various things per loop

# if the 3rd parameter is omitted, this is a proper regression
# test

import sys
from CAN import *


sys.stderr.write("startup\r")

arg1=sys.argv[1].split(":")
host1 = arg1[0]
if len(arg1) > 1:
    inst1 = int(arg1[1])
else:
    inst1 = 0

arg2=sys.argv[2].split(":")
host2 = arg2[0]
if len(arg2) > 1:
    inst2 = int(arg2[1])
else:
    inst2 = 0

t=time.time()
if False:
    canT1 = canctl(host1,inst1)
    canT2 = canctl(host2,inst2)
    canR1 = canctl(host1,inst1)
    canR2 = canctl(host2,inst2)
    canR1.s.settimeout(1)
    canR2.s.settimeout(1)
else:
    canT1 = CAN(tsctl(host1),inst1)
    sys.stderr.write("startup2\r")
    canT2 = CAN(tsctl(host2),inst2)
    sys.stderr.write("startup3\r")
    canR1 = CAN(tsctl(host1),inst1)
    sys.stderr.write("startup4\r")
    canR2 = CAN(tsctl(host2),inst2)
    sys.stderr.write("startup5\r")
    canR1.tsctl.s.settimeout(1)
    canR2.tsctl.s.settimeout(1)
    canT1.tsctl.s.settimeout(1)
    canT2.tsctl.s.settimeout(1)

#canT1.BaudSet(1000000)
#canT2.BaudSet(1000000)
#canR1.Tx(canR1.FlagControl,77,"") # re-prime connection
#canR2.Tx(canR2.FlagControl,77,"") # re-prime connection

# Send and wait to receive a message several times,
# then print statistics
data1 = "\x01\x02\x03\x04\x05\x06\x07\x08"
data2 = "\xFF\xFE\xFD\xFC\xFB\xFA\xAA\x55"
min = 1000
max = 0
tot = 0
runs = 100
lost = 0
t0=time.time()
for x in range(runs):
    sys.stderr.write(str(x)+"A     \r")
    try:
        canT1.Tx(canT1.FlagExtId,0x1234,data1)
        sys.stderr.write(str(x)+"BBBBBBBB\r")
        canT2.Tx(canT1.FlagExtId,0x5678,data2)
        sys.stderr.write(str(x)+"CCCCCCCC\r")
    except socket.timeout:
        print "timeout0"
        pass

    t1=time.time()
    try:
        pkt = canR2.Rx()
        sys.stderr.write(str(x)+"D              \r")
        #print repr(pkt)
        if (pkt['data'] != data1):
            print "mismatch 1 0x%X:" % (pkt['id'],),
            for i in range(len(pkt['data'])):
                sys.stdout.write("%02X " % ord(pkt['data'][i]))
            print
            exit(1)
    except socket.timeout:
        lost += 1
        print "timeout2"
        #exit(1)
        
    t2=time.time()
    try:
        pkt = canR1.Rx()
        sys.stderr.write(str(x)+"E\r")
        if (pkt['data'] != data2):
            sys.stdout.write("mismatch 2 0x%X:" % (pkt['id']))
            for i in range(len(pkt['data'])):
                sys.stdout.write("%02X" % ord(pkt['data'][i]))
            print
            exit(1)
    except socket.timeout:
        lost += 1
        print "timeout1"
        #exit(1)
            
    t3=time.time()
    dt1 = 1000*(t2-t1)
    dt2 = 1000*(t3-t2)
    if dt1 > max: max = dt1
    if dt2 > max: max = dt2
    if dt1 < min: min = dt1
    if dt2 < min: min = dt2
    tot = tot + dt1 + dt2
    #print "Time1: " + str(dt1) + "ms"
    #print "Time2: " + str(1000*(t3-t2)) + "ms"
t1=time.time()
#print "Lost="+str(lost)

if (len(sys.argv) > 3):
    print "Startup:" + str(1000*(t0-t)) + "ms"
    print "Min: " + str(min) + "ms"
    print "Max: " + str(max) + "ms"
    print "Avg: " + str(tot / (2*runs)) + "ms"
    print "Avg2:" + str(1000*(t1-t0)/runs) + "ms"

"""
can1.Tx(can1.FlagExtId,0x1234,"\x01\x02\x10\x20")
try:
    pkt = can2.Rx()
    print "0x%X:" % (pkt['id']),
    for i in range(len(pkt['data'])):
        print "%02X" % ord(pkt['data'][i]),
except socket.timeout:
    print "timeout"
"""

# Author: Michael Schmidt (michael@embeddedARM.com)
# Copyright (c) 2011, Technologic Systems, All Rights Reserved
# Refer to the COPYRIGHT file provided with this project for licensing terms.
