#!/usr/bin/python

# usage:
# TestTWI.py <host>

# This regression test verifies that our locking mechanism is
# working correctly.  We open two connections to the server.
# On the first connection, we expect to lock immediately. On
# the second connection, we expect to fail to lock until we
# release the first lock.

from TWI import *

twi1 = TWI(tsctl(sys.argv[1]),0)
twi2 = TWI(tsctl(sys.argv[1]),0)
twi1.tsctl.s.settimeout(1)
twi2.tsctl.s.settimeout(1)

# lock 1
try:
    tmp = twi1.Lock(WaitLock)
    if tmp < 1:
        print "Error " + str(tmp) + " on lock 1"
        exit(1)
except socket.timeout:
    print "Error: lock 1 timeout"
    exit(1)

# make sure lock 2 blocks
try:
    tmp = twi2.Lock(WaitLock)
    print "Error: lock 2 returned " + str(tmp)
    exit(1)
except socket.timeout:
    ok=1

# close conn 1
del twi1

# now, try to pick up the return from Lock again...
try:
    tmp = twi2.LockResult()
    if tmp < 1:
        print "Error " + str(tmp) + " on lock 2"
        exit()
except socket.timeout:
    print "Error: lock 2 timeout"
    exit(1)

tmp = twi2.Lock(DoRelease)
if tmp < 1:
    print "Error " + str(tmp) + " on unlock 2"
    exit(1)

# Author: Michael Schmidt (michael@embeddedARM.com)
# Copyright (c) 2011, Technologic Systems, All Rights Reserved
# Refer to the COPYRIGHT file provided with this project for licensing terms.
