#!/usr/bin/python

# usage:
# TestTWI2.py <host>

# This program requires tsctl server to be running on a host
# connected to a TS-8160.

# This code issues the command to
# read the input voltage, and checks if it is in range.

from TWI import *

twi1 = TWI(tsctl(sys.argv[1]),0)
twi1.tsctl.s.settimeout(1)


# test reading the Voltage from the TS-8160 AVR
try:
    tmp = twi1.Lock(WaitLock)
    if tmp < 1:
        print "Error " + tmp + " on lock 1"
        exit(1)
except socket.timeout:
    print "Error: Lock timeout"
    exit(1)

try:
    data = twi1.Read(7,1,0x40,4)
except socket.timeout:
    print "Error: Read timeout"
    exit(1)

if len(data) == 0:
    print "Error "+str(twi1.Status())+" reading"
    exit(1)
if len(data) < 4:
    print "read back error: "+repr(data)
    exit(1)

if ord(data[0]) != 0x55:
    print "Signature not found: Got "+repr(data)+", expected 0x55..."
    exit(1)
if ord(data[1]) < 2:
    print "AVR software revision "+repr(data[0])+", expected >=2"
    exit(1)
mV=(ord(data[2])*256 + ord(data[3])) * 3690 / 4097
if (mV > 2800):
    print "Voltage is too high: "+str(mV)+"mV"
    exit(1)
if (mV < 450):
    print "Voltage is too low: "+str(mV)+"mV"
    exit(1)
twi1.Lock(DoRelease)
del twi1

# Author: Michael Schmidt (michael@embeddedARM.com)
# Copyright (c) 2011, Technologic Systems, All Rights Reserved
# Refer to the COPYRIGHT file provided with this project for licensing terms.
