#include "MMapBus.c"
#include "Cavium2132SBus.c"
#include "Cavium2132SBusWindowBus.c"
#include "WBWindowBus.c"
#include "TSMuxBus.c"
#include "CacheBus.c"
#include "ts4500DIORaw.c"
#include "Cavium2132DIORaw.c"
#include "ts4500Pin.c"
#include "PhysicalDIO.c"
#include "AggregateDIO.c"
#include "DIOTWI.c"
#include "SystemTime.c"
#include "Cavium2132Time.c"
#include "SJA1000CAN.c"
#include "LocalSystem.c"
#include "WBSPI.c"
#include "baudtab.h"
#include "ts4500.h"
#include "arch.h"

#ifndef __MODEL_ID
int ModelId=0x4500;
#endif

// Locks:
// 0     SBus
// 1     TWI
// 2     CAN
// 3     CANBus
// 7     SPI
// 10-65 DIO 1
// 68-99 DIO 2
#define NLOCKS 256 // HACK: 132 with TS-81X0

MMapBus cpubus,testbus;
CacheBus ts4500DIOCacheBus,Cavium2132DIOCacheBus;
TSMuxBus muxbus;
WBWindowBus wbwin;
ts4500DIORaw ts4500DIORaw0;
Cavium2132DIORaw Cavium2132DIORaw0;
ts4500Pin ts4500Pin0;
PhysicalDIO ts4500DIO,Cavium2132DIO;
AggregateDIO DIO0;
DIOTWI TWI0;
SystemTime Time0;
Cavium2132Time Time1;
unsigned char ts4500DIOCapabilities[56];
CANConn CAN0conn[16];
SJA1000CAN CAN0;
WBSPI SPI0;
System sys;

unsigned Cavium2132DIOCacheBus_WO[12];
unsigned Cavium2132DIOCacheBus_IBit[12];
unsigned Cavium2132DIOCacheBus_OBit[12];
unsigned Cavium2132DIOCacheBus_icache[12];
unsigned Cavium2132DIOCacheBus_ocache[12];
unsigned Cavium2132DIOCacheBus_ocache0[12];
unsigned char Cavium2132DIOCapabilities[32];

// Standard instances
// Bus 0 is whatever bus directly accesses the FPGA "syscon" registers
// For the TS-4500 this is the Cavium2132SBus
// Bus0

MMapBus spibus, gpiobus;
Cavium2132SBus sbus;
Cavium2132SBusWindowBus canbus;

// Bus1
unsigned ts4500DIOCacheBus_WO[15];
unsigned ts4500DIOCacheBus_IBit[15];
unsigned ts4500DIOCacheBus_OBit[15];
unsigned ts4500DIOCacheBus_icache[15];
unsigned ts4500DIOCacheBus_ocache[15];
unsigned ts4500DIOCacheBus_ocache0[15];

char testbusdata[4096];

void ts4500_ArchInit() {
  ThreadInit();
}

System *ts4500_SystemInit(System *s,int inst) {
  if (inst == 0) {
    return LocalSystemInit(&sys);
  }
  return s;
}

Time *ts4500_TimeInit(Time *t,int inst) {
  if (inst == 0) {
    return SystemTimeInit(&Time0);
  }
  if (inst == 1) {
    return Cavium2132TimeInit(&Time1);
  }
  return t;
}

Bus *ts4500_BusInit(Bus *bus,int inst) {
  if (inst == 0) { // Cavium2132SBus
    MMapBusInit(&spibus,0x71000000,1);
    return Cavium2132SBusInit(&sbus,&spibus,BusInit(6));
  }
  if (inst == 1) { // for testing
    return MMapBusInit2(&testbus,testbusdata,1);
  }

  if (inst == 2) { // MUXBus
    return TSMuxBusInit(&muxbus,BusInit(8),0x20000,BusInit(8),0x10000,
			BusInit(8),0);
  }
  if (inst == 3) { // ts4500DIOCacheBus
    ts4500DIOCacheBus_IBit[0] = 0x3000;
    ts4500DIOCacheBus_IBit[2] = 0xF001;
    ts4500DIOCacheBus_IBit[3] = 0xFFFF;
    ts4500DIOCacheBus_IBit[6] = 0xFFFF;
    ts4500DIOCacheBus_IBit[11] = 0xFC00;
    ts4500DIOCacheBus_IBit[13] = 0xFFFF;
    ts4500DIOCacheBus_IBit[14] = 0xFF00;
    ts4500DIOCacheBus_OBit[0] = 0xCF00;
    ts4500DIOCacheBus_OBit[2] = 0x0FFE;
    ts4500DIOCacheBus_OBit[4] = 0xFFFF;
    ts4500DIOCacheBus_OBit[5] = 0xFFFF;
    ts4500DIOCacheBus_OBit[7] = 0xFFFF;
    ts4500DIOCacheBus_OBit[8] = 0xFFFF;
    ts4500DIOCacheBus_OBit[11] = 0x3FFF;
    ts4500DIOCacheBus_OBit[12] = 0xFFFF;
    ts4500DIOCacheBus_OBit[14] = 0x00FF;

    ts4500DIOCacheBus.Length=15;
    ts4500DIOCacheBus.Width=16;
    ts4500DIOCacheBus.Offset=0x62;
    ts4500DIOCacheBus.WO = ts4500DIOCacheBus_WO;
    ts4500DIOCacheBus.IBit = ts4500DIOCacheBus_IBit;
    ts4500DIOCacheBus.OBit = ts4500DIOCacheBus_OBit;
    ts4500DIOCacheBus.icache = ts4500DIOCacheBus_icache;
    ts4500DIOCacheBus.ocache = ts4500DIOCacheBus_ocache;
    ts4500DIOCacheBus.ocache0 = ts4500DIOCacheBus_ocache0;
    return CacheBusInit(&ts4500DIOCacheBus,BusInit(0));
  }
  if (inst == 4) { // CPU bus
    return MMapBusInit(&cpubus,0x76000000,1);
  }
  if (inst == 5) { // Cavium2132DIOCacheBus
    Cavium2132DIOCacheBus_IBit[1] = 0xFFFFFFFF;
    Cavium2132DIOCacheBus_OBit[0] = 0xFFFFFFFF;
    Cavium2132DIOCacheBus_OBit[2] = 0xFFFFFFFF;
    Cavium2132DIOCacheBus.Length=12;
    Cavium2132DIOCacheBus.Width=32;
    Cavium2132DIOCacheBus.Offset=0;
    Cavium2132DIOCacheBus.IBit = Cavium2132DIOCacheBus_IBit;
    Cavium2132DIOCacheBus.OBit = Cavium2132DIOCacheBus_OBit;
    Cavium2132DIOCacheBus.icache = Cavium2132DIOCacheBus_icache;
    Cavium2132DIOCacheBus.ocache = Cavium2132DIOCacheBus_ocache;
    Cavium2132DIOCacheBus.ocache0 = Cavium2132DIOCacheBus_ocache0;
    return CacheBusInit(&Cavium2132DIOCacheBus,BusInit(6));
  }
  if (inst == 6) { // gpiobus
    return MMapBusInit(&gpiobus,0x7C000000,1);
  }
  if (inst == 7) { // Cavium2132SBusWindowBus
    canbus.MemBase = 0x50;
    return Cavium2132SBusWindowBusInit(&canbus,BusInit(0));
  }
  if (inst == 8) { // WBWindowBus
    return WBWindowBusInit(&wbwin,BusInit(0),0x18,0);
  }
  return bus;
}

Pin *ts4500_PinInit(Pin *pin,int inst) {
  if (inst == 0) {
    return ts4500PinInit(&ts4500Pin0,BusInit(0),BusInit(4),
			 BusInit(8),0x20000);
  }
  return pin;
}

int CANBusNum[1] = { 7 };

CAN *ts4500_CANInit(CAN *can,int inst) {
  if (inst == 0) {
    CAN0.LockNum = 2;
    CAN0.status = 0;
    CAN0.CAN_TX = 15;
    CAN0.CAN_RX = 16;
    CAN0.D.conn = CAN0conn;
    CAN0.D.maxconn = 16;
    CAN0.D.nconn = 0;
    CAN0.D.txc = 0;
    CAN0.baud = 1000000;
    CAN0.baudparms = baudparms25;
    CAN0.irq = 30;
    return SJA1000CANInit(&CAN0,BusInit(7),PinInit(0),TimeInit(1));
  }
  return can;
}

DIORaw *ts4500_DIORawInit(DIORaw *dioraw,int inst) {
  if (inst == 0) {
    return ts4500DIORawInit(&ts4500DIORaw0,BusInit(3));
  }
  if (inst == 1) {
    return Cavium2132DIORawInit(&Cavium2132DIORaw0,BusInit(5));
  }
  return dioraw;
}

DIO *ts4500_DIOInit(DIO *dio,int inst) {
  int i;

  if (inst == 0) {
    return DIOInit0(&DIO0);
  }
  if (inst == 1) {
    ts4500DIO.Capabilities = ts4500DIOCapabilities;
    for (i=0;i<=52;i++) {
      ts4500DIOCapabilities[i] = DIO_NORMAL;
    }
    ts4500DIOCapabilities[54] = DIO_OUTPUT_ONLY;
    ts4500DIOCapabilities[55] = DIO_OUTPUT_ONLY;
    return PhysicalDIOInit(&ts4500DIO,BusInit(3),DIORawInit(0));
  }
  if (inst == 2) {
    Cavium2132DIO.Capabilities = Cavium2132DIOCapabilities;
    Cavium2132DIOCapabilities[13] = DIO_NORMAL;
    Cavium2132DIOCapabilities[14] = DIO_NORMAL;
    return PhysicalDIOInit(&Cavium2132DIO,BusInit(5),DIORawInit(1));
  }
  return dio;
}

TWI *ts4500_TWIInit(TWI *twi,int inst) {
  if (inst == 0) {
    TWI0.TW_CLK=14+56;
    TWI0.TW_DAT=13+56;
    TWI0.Speed=1000000;
    TWI0.LockNum=1;
    return DIOTWIInit(&TWI0,DIOInit(0),TimeInit(0));
  }
  return twi;
}

static void ts4500SPIChipSelect(WBSPI *ob,unsigned num,int asserted) {
  if (num > 3) return;
  if (!asserted) return; // core will do this automatically
  if (num == 0) num = 1; else if (num == 1) num = 0;
  ob->bus->API->Lock(ob->bus,0,0);
  ob->bus->API->Poke16(ob->bus,ob->offset,
		  (ob->bus->API->Peek16(ob->bus,ob->offset) & 0xFC7F)
		       | (num << 8) /*| (1 << 7)*/);
  ob->bus->API->Unlock(ob->bus,0,0);
  return;
}

SPI *ts4500_SPIInit(SPI *spi,int inst) {
  if (inst == 0) {
    SPI0.LockNum = 7;
    return WBSPIInit(&SPI0,BusInit(0),BusInit(0),0x40,ts4500SPIChipSelect);
  }
  return spi;
}

AIO *ts4500_AIOInit(AIO *aio,int inst) {
  return aio;
}

#ifndef __MODEL_ID
#define __MODEL_ID ts4500
#endif

ArchInfo TEMPLATE(__MODEL_ID,_ArchInfo) = {
  .Instances={1,9,2,1,2,3,1,1,1},
  .CANBusNum=CANBusNum,
  .ArchInit=ts4500_ArchInit,
  .SystemInit=ts4500_SystemInit,
  .TimeInit=ts4500_TimeInit,
  .BusInit=ts4500_BusInit,
  .PinInit=ts4500_PinInit,
  .CANInit=ts4500_CANInit,
  .DIORawInit=ts4500_DIORawInit,
  .DIOInit=ts4500_DIOInit,
  .SPIInit=ts4500_SPIInit,
  .TWIInit=ts4500_TWIInit,
  .AIOInit=ts4500_AIOInit
  //.EDIOInit=ts4500_EDIOInit
};

#include "arch.c"

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2012, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
