//#include "MarvellPXA166TWI.c"
#include "MMapBus.c"
#include "SystemTime.c"
#include "AggregateDIO.c"
#include "DIOTWI.c"
#include "PhysicalDIO.c"
#include "ts4700Pin.c"
#include "ts4700DIORaw.c"
#include "TSMuxBus.c"
#include "MarvellPXA166DIORaw.c"
#include "CacheBus.c"
#include "SJA1000CAN.c"
#include "LocalSystem.c"
#include "WBSPI.c"
#include "baudtab.h"
#include "ts4700.h"
#include "arch.h"

int ModelId=0x4700;

// object hierarchy
MMapBus MarvellPXA166DIOBus, ts4700DIOBus, ts4700CANBus, testbus, twiregs,
  mfpregs, mux8bus, mux16bus, spi8bus, spi16bus;
TSMuxBus muxbus;
CacheBus MarvellPXA166DIOCacheBus, ts4700DIOCacheBus;
MarvellPXA166DIORaw MarvellPXA166DIORaw0;
ts4700DIORaw ts4700DIORaw0;
PhysicalDIO MarvellPXA166DIO, ts4700DIO;
AggregateDIO DIO0;
SystemTime Time0;
//MarvellPXA166TWI TWI0;
DIOTWI TWI0;
SJA1000CAN CAN0;
ts4700Pin ts4700Pin0;
WBSPI SPI0;
System sys;

CANConn CAN0conn[16];

#define ts4700DIOCacheBusLength 12
unsigned ts4700DIOCacheBus_WO[ts4700DIOCacheBusLength];
unsigned ts4700DIOCacheBus_IBit[ts4700DIOCacheBusLength];
unsigned ts4700DIOCacheBus_OBit[ts4700DIOCacheBusLength];
unsigned ts4700DIOCacheBus_icache[ts4700DIOCacheBusLength];
unsigned ts4700DIOCacheBus_ocache[ts4700DIOCacheBusLength];
unsigned ts4700DIOCacheBus_ocache0[ts4700DIOCacheBusLength];

#define MarvellPXA166DIOCacheBusLength 0x4A
unsigned MarvellPXA166DIOCacheBus_WO[MarvellPXA166DIOCacheBusLength];
unsigned MarvellPXA166DIOCacheBus_IBit[MarvellPXA166DIOCacheBusLength];
unsigned MarvellPXA166DIOCacheBus_OBit[MarvellPXA166DIOCacheBusLength];
unsigned MarvellPXA166DIOCacheBus_icache[MarvellPXA166DIOCacheBusLength];
unsigned MarvellPXA166DIOCacheBus_ocache[MarvellPXA166DIOCacheBusLength];
unsigned MarvellPXA166DIOCacheBus_ocache0[MarvellPXA166DIOCacheBusLength];

unsigned char MarvellPXA166DIOCapabilities[128];
unsigned char ts4700DIOCapabilities[62];

char testbusdata[4096];

void ts4700_ArchInit() {
  ThreadInit();
}

System *ts4700_SystemInit(System *s,int inst) {
  if (inst == 0) {
    return LocalSystemInit(&sys);
  }
  return s;
}

// object initializers
Time *ts4700_TimeInit(Time *t,int inst) {
  if (inst == 0) {
    return SystemTimeInit(&Time0);
  }
  return t;
}

Bus *ts4700_BusInit(Bus *bus,int inst) {
  if (inst == 0) { // ts4700DIOBus
    return MMapBusInit(&ts4700DIOBus,0x80004000,1);
  }
  if (inst == 1) { // for testing
    return MMapBusInit2(&testbus,testbusdata,1);
  }
  if (inst == 2) { // MUXBus
    return TSMuxBusInit(&muxbus,BusInit(0),0x4,BusInit(9),0,BusInit(10),0);
  }
  if (inst == 3) { // MarvellPXA166DIOBus
    return MMapBusInit(&MarvellPXA166DIOBus,0xD4019000,1);
  }
  if (inst == 4) { // ts4700CANBus
    return MMapBusInit(&ts4700CANBus,0x81004c00,1);
  }
  if (inst == 5) { // ts4700DIOCacheBus
    ts4700DIOCacheBus_IBit[8] = 0x7FFF;
    ts4700DIOCacheBus_IBit[9] = 0x7FF;
    ts4700DIOCacheBus_IBit[10] = 0xFFFF;
    ts4700DIOCacheBus_IBit[11] = 0xFFF;
    ts4700DIOCacheBus_OBit[0] = 0x7FFF;
    ts4700DIOCacheBus_OBit[1] = 0x1FFF;
    ts4700DIOCacheBus_OBit[2] = 0xFFFF;
    ts4700DIOCacheBus_OBit[3] = 0xFFF;
    ts4700DIOCacheBus_OBit[4] = 0x7FFF;
    ts4700DIOCacheBus_OBit[5] = 0x7FF;
    ts4700DIOCacheBus_OBit[6] = 0xFFFF;
    ts4700DIOCacheBus_OBit[7] = 0xFFF;

    ts4700DIOCacheBus.Length=ts4700DIOCacheBusLength;
    ts4700DIOCacheBus.Width=16;
    ts4700DIOCacheBus.Offset=0x10;
    ts4700DIOCacheBus.WO=ts4700DIOCacheBus_WO;
    ts4700DIOCacheBus.IBit=ts4700DIOCacheBus_IBit;
    ts4700DIOCacheBus.OBit=ts4700DIOCacheBus_OBit;
    ts4700DIOCacheBus.icache=ts4700DIOCacheBus_icache;
    ts4700DIOCacheBus.ocache=ts4700DIOCacheBus_ocache;
    ts4700DIOCacheBus.ocache0=ts4700DIOCacheBus_ocache0;
    return CacheBusInit(&ts4700DIOCacheBus,BusInit(0));
  }
  if (inst == 6) { // MarvellPXA166DIOCacheBus
    MarvellPXA166DIOCacheBus_IBit[0] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_IBit[1] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_IBit[2] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_IBit[64] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_OBit[3] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_OBit[4] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_OBit[5] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_OBit[67] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[6] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[7] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[8] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[70] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[9] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[10] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[11] = 0xFFFFFFFF;
    MarvellPXA166DIOCacheBus_WO[73] = 0xFFFFFFFF;

    MarvellPXA166DIOCacheBus.Length=MarvellPXA166DIOCacheBusLength;
    MarvellPXA166DIOCacheBus.Width=32;
    MarvellPXA166DIOCacheBus.Offset=0;
    MarvellPXA166DIOCacheBus.WO=MarvellPXA166DIOCacheBus_WO;
    MarvellPXA166DIOCacheBus.IBit=MarvellPXA166DIOCacheBus_IBit;
    MarvellPXA166DIOCacheBus.OBit=MarvellPXA166DIOCacheBus_OBit;
    MarvellPXA166DIOCacheBus.icache=MarvellPXA166DIOCacheBus_icache;
    MarvellPXA166DIOCacheBus.ocache=MarvellPXA166DIOCacheBus_ocache;
    MarvellPXA166DIOCacheBus.ocache0=MarvellPXA166DIOCacheBus_ocache0;

    return CacheBusInit(&MarvellPXA166DIOCacheBus,BusInit(3));
  }
  if (inst == 7) { // CPU TWI regs
    return MMapBusInit(&twiregs,0xD4025008,1); // 0xD4011008
  }
  if (inst == 8) { // CPU MFP regs
    return MMapBusInit(&mfpregs,0xD401E000,1); // 0xD4011008
  }
  if (inst == 9) { // mux 8
    return MMapBusInit(&mux8bus,0x81008000,8);
  }
  if (inst == 10) { // mux 16
    return MMapBusInit(&mux16bus,0x80008000,8);
  }
  if (inst == 11) { // SPI 8
    return MMapBusInit(&spi8bus,0x81004800,1);
  }
  if (inst == 12) { // SPI 16
    return MMapBusInit(&spi16bus,0x80004800,1);
  }
  return bus;
}

Pin *ts4700_PinInit(Pin *pin,int inst) {
  if (inst == 0) {
    return ts4700PinInit(&ts4700Pin0,BusInit(0),BusInit(8));
  }
  return pin;
}

int CANBusNum[1] = { 4 };

CAN *ts4700_CANInit(CAN *can,int inst) {
  if (inst == 0) {
    CAN0.LockNum = 2;
    CAN0.status = 0;
    CAN0.CAN_TX = 15;
    CAN0.CAN_RX = 16;
    CAN0.D.conn = CAN0conn;
    CAN0.D.maxconn = 16;
    CAN0.D.nconn = 0;
    CAN0.D.txc = 0;
    CAN0.baud = 1000000;
    CAN0.baudparms = baudparms25;
    CAN0.irq = 65;
    return SJA1000CANInit(&CAN0,BusInit(4),PinInit(0),TimeInit(0));
  }
  return can;
}

DIORaw *ts4700_DIORawInit(DIORaw *dioraw,int inst) {
  if (inst == 0) {
    return ts4700DIORawInit(&ts4700DIORaw0,BusInit(5));
  }
  if (inst == 1) {
    return MarvellPXA166DIORawInit(&MarvellPXA166DIORaw0,BusInit(6));
  }
  return dioraw;
}

DIO *ts4700_DIOInit(DIO *dio,int inst) {
  int i,n;

  if (inst == 0) {
    return DIOInit0(&DIO0);
  }
  if (inst == 1) {
    DIORaw *raw = DIORawInit(0);

    n = raw->API->Count(raw);
    ts4700DIO.Capabilities = ts4700DIOCapabilities;
    for (i=0;i<n;i++) {
      ts4700DIOCapabilities[i] = DIO_NORMAL;
    }
    return PhysicalDIOInit(&ts4700DIO,BusInit(5),raw);
  }
  if (inst == 2) {
    DIORaw *raw = DIORawInit(1);

    n = raw->API->Count(raw);
    MarvellPXA166DIO.Capabilities = MarvellPXA166DIOCapabilities;
    for (i=0;i<n;i++) {
      MarvellPXA166DIOCapabilities[i] = DIO_NORMAL;
    }
    return PhysicalDIOInit(&MarvellPXA166DIO,BusInit(6),raw);
  }
  return dio;
}

TWI *ts4700_TWIInit(TWI *twi,int inst) {
  if (inst == 0) {
    TWI0.TW_CLK=150;
    TWI0.TW_DAT=149;
    TWI0.Speed=1000000;
    TWI0.LockNum=1;
    return DIOTWIInit(&TWI0,DIOInit(0),TimeInit(0));
    //return MarvellPXA166TWIInit(&TWI0);
  }
  return twi;
}

static void ts4700SPIChipSelect(WBSPI *ob,unsigned num,int asserted) {
  if (num > 0) return; // CS1-3# not available
  if (!asserted) return; // core will do this automatically
  ob->bus->API->Lock(ob->bus,0,0);
  ob->bus->API->Poke16(ob->bus,ob->offset,
		  (ob->bus->API->Peek16(ob->bus,ob->offset) & 0xFC7F)
		  | (num << 8) | (1 << 7));
  ob->bus->API->Unlock(ob->bus,0,0);
  return;
}

SPI *ts4700_SPIInit(SPI *spi,int inst) {
  if (inst == 0) {
    SPI0.LockNum = 7;
    return WBSPIInit(&SPI0,BusInit(12),BusInit(11),0,ts4700SPIChipSelect);
  }
  return spi;
}

AIO *ts4700_AIOInit(AIO *aio,int inst) {
  return aio;
}

ArchInfo ts4700_ArchInfo = {
  .Instances={1,13,1,1,2,3,1,1,1},
  .CANBusNum=CANBusNum,
  .ArchInit=ts4700_ArchInit,
  .SystemInit=ts4700_SystemInit,
  .TimeInit=ts4700_TimeInit,
  .BusInit=ts4700_BusInit,
  .PinInit=ts4700_PinInit,
  .CANInit=ts4700_CANInit,
  .DIORawInit=ts4700_DIORawInit,
  .DIOInit=ts4700_DIOInit,
  .SPIInit=ts4700_SPIInit,
  .TWIInit=ts4700_TWIInit,
  .AIOInit=ts4700_AIOInit
  //.EDIOInit=ts4700_EDIOInit
};

#include "arch.c"

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2012, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
