#include "LinuxTWI.c"
#include "SystemTime.c"
#include "AggregateDIO.c"
#include "MMapBus.c"
#include "PhysicalDIO.c"
#include "ts4800Pin.c"
#include "ts4800DIORaw.c"
#include "TSMuxBus.c"
#include "FreescaleIMX51DIORaw.c"
#include "CacheBus.c"
#include "SJA1000CAN.c"
#include "WBSPI.c"
#include "baudtab.h"
#include "TSTime.c"
#include "WindowBus.c"
#include "LocalSystem.c"
#include "ts4800.h"
#include "arch.h"

int ModelId=0x4800;

// object hierarchy
MMapBus ts4800DIOBus, FreescaleIMX51DIOBus0, FreescaleIMX51DIOBus1, testbus,
  FreescaleIMX51DIOBus2, FreescaleIMX51DIOBus3, ts4800CANBus0, ts4800CANBus1,
  SPIBus;
TSMuxBus muxbus;
CacheBus ts4800DIOCacheBus, FreescaleIMX51DIOCacheBus0, 
  FreescaleIMX51DIOCacheBus1, FreescaleIMX51DIOCacheBus2,
  FreescaleIMX51DIOCacheBus3;
WindowBus WindowBus0, WindowBus1;
ts4800DIORaw ts4800DIORaw0;
FreescaleIMX51DIORaw FreescaleIMX51DIORaw0,FreescaleIMX51DIORaw1,
  FreescaleIMX51DIORaw2, FreescaleIMX51DIORaw3;
PhysicalDIO ts4800DIO, FreescaleIMX51DIO0, FreescaleIMX51DIO1,
  FreescaleIMX51DIO2,FreescaleIMX51DIO3;
AggregateDIO DIO0;
SystemTime Time0;
TSTime Time1;
LinuxTWI TWI0;
SJA1000CAN CAN0, CAN1;
ts4800Pin ts4800Pin0;
WBSPI SPI0;
System sys;

CANConn CAN0conn[16];
CANConn CAN1conn[16];

#define ts4800DIOCacheBusLength 23
unsigned ts4800DIOCacheBus_WO[ts4800DIOCacheBusLength];
unsigned ts4800DIOCacheBus_IBit[ts4800DIOCacheBusLength];
unsigned ts4800DIOCacheBus_OBit[ts4800DIOCacheBusLength];
unsigned ts4800DIOCacheBus_icache[ts4800DIOCacheBusLength];
unsigned ts4800DIOCacheBus_ocache[ts4800DIOCacheBusLength];
unsigned ts4800DIOCacheBus_ocache0[ts4800DIOCacheBusLength];

#define FreescaleIMX51DIOCacheBusLength 3
unsigned FreescaleIMX51DIOCacheBus_WO[FreescaleIMX51DIOCacheBusLength];
unsigned FreescaleIMX51DIOCacheBus_IBit[FreescaleIMX51DIOCacheBusLength];
unsigned FreescaleIMX51DIOCacheBus_OBit[FreescaleIMX51DIOCacheBusLength];
unsigned FreescaleIMX51DIOCacheBus_icache[FreescaleIMX51DIOCacheBusLength];
unsigned FreescaleIMX51DIOCacheBus_ocache[FreescaleIMX51DIOCacheBusLength];
unsigned FreescaleIMX51DIOCacheBus_ocache0[FreescaleIMX51DIOCacheBusLength];

unsigned char ts4800DIOCapabilities[66];
unsigned char FreescaleIMX51DIOCapabilities0[16];
unsigned char FreescaleIMX51DIOCapabilities1[16];
unsigned char FreescaleIMX51DIOCapabilities2[16];
unsigned char FreescaleIMX51DIOCapabilities3[16];

char testbusdata[4096];

void ts4800_ArchInit() {
  ThreadInit();
}

System *ts4800_SystemInit(System *s,int inst) {
  if (inst == 0) {
    return LocalSystemInit(&sys);
  }
  return s;
}

// object initializers
Time *ts4800_TimeInit(Time *t,int inst) {
  if (inst == 0) {
    return SystemTimeInit(&Time0);
  }
  if (inst == 1) {
    return TSTimeInit(&Time1,BusInit(0),4,6,1000000);
  }
  return t;
}

Bus *ts4800_BusInit(Bus *bus,int inst) {
  if (inst == 0) { // ts4800DIOBus (actually ts4800SysconBus)
    return MMapBusInit(&ts4800DIOBus,0xB0010000,2);
  }
  if (inst == 1) { // for testing
    return MMapBusInit2(&testbus,testbusdata,1);
  }
  if (inst == 2) { // MUXBus
    return TSMuxBusInit(&muxbus,BusInit(0),0x12,BusInit(0),0x1000,
			BusInit(0),0x1000);
  }
  if (inst == 3) { // FreescaleIMX51DIOBus0
    return MMapBusInit(&FreescaleIMX51DIOBus0,0x73F84000,1);
  }
  if (inst == 4) { // FreescaleIMX51DIOBus1
    return MMapBusInit(&FreescaleIMX51DIOBus1,0x73F88000,33); // 33 pp for Pin
  }
  if (inst == 5) { // FreescaleIMX51DIOBus2
    return MMapBusInit(&FreescaleIMX51DIOBus2,0x73F8C000,1);
  }
  if (inst == 6) { // FreescaleIMX51DIOBus3
    return MMapBusInit(&FreescaleIMX51DIOBus3,0x73F90000,1);
  }
  if (inst == 7) { // ts4800CANBus0
    return MMapBusInit(&ts4800CANBus0,0xB001A000,1);
  }
  if (inst == 8) { // ts4800CANBus1
    return MMapBusInit(&ts4800CANBus1,0xB001B000,1);
  }
  if (inst == 9) { // ts4800DIOCacheBus
    ts4800DIOCacheBus_IBit[0] = 0x3F00;
    ts4800DIOCacheBus_IBit[8] = 0xFFFF;
    ts4800DIOCacheBus_IBit[12] = 0xFFFF;
    ts4800DIOCacheBus_IBit[16] = 0xFFFF;
    ts4800DIOCacheBus_IBit[20] = 0xFFFF;

    ts4800DIOCacheBus_OBit[0] = 0xC0FF;
    ts4800DIOCacheBus_OBit[9] = 0xFFFF;
    ts4800DIOCacheBus_OBit[10] = 0xFFFF;
    ts4800DIOCacheBus_OBit[13] = 0xFFFF;
    ts4800DIOCacheBus_OBit[14] = 0xFFFF;
    ts4800DIOCacheBus_OBit[17] = 0xFFFF;
    ts4800DIOCacheBus_OBit[18] = 0xFFFF;
    ts4800DIOCacheBus_OBit[21] = 0xFFFF;
    ts4800DIOCacheBus_OBit[22] = 0xFFFF;

    ts4800DIOCacheBus.Length=ts4800DIOCacheBusLength;
    ts4800DIOCacheBus.Width=16;
    ts4800DIOCacheBus.Offset=0x10;
    ts4800DIOCacheBus.WO=ts4800DIOCacheBus_WO;
    ts4800DIOCacheBus.IBit=ts4800DIOCacheBus_IBit;
    ts4800DIOCacheBus.OBit=ts4800DIOCacheBus_OBit;
    ts4800DIOCacheBus.icache=ts4800DIOCacheBus_icache;
    ts4800DIOCacheBus.ocache=ts4800DIOCacheBus_ocache;
    ts4800DIOCacheBus.ocache0=ts4800DIOCacheBus_ocache0;
    return CacheBusInit(&ts4800DIOCacheBus,BusInit(0));
  }
  if (inst == 10) { // FreescaleIMX51DIOCacheBus0 
    FreescaleIMX51DIOCacheBus_IBit[2] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[0] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[1] = 0xFFFFFFFF;

    FreescaleIMX51DIOCacheBus0.Length=FreescaleIMX51DIOCacheBusLength;
    FreescaleIMX51DIOCacheBus0.Width=32;
    FreescaleIMX51DIOCacheBus0.Offset=0;

    FreescaleIMX51DIOCacheBus0.WO=FreescaleIMX51DIOCacheBus_WO;
    FreescaleIMX51DIOCacheBus0.IBit=FreescaleIMX51DIOCacheBus_IBit;
    FreescaleIMX51DIOCacheBus0.OBit=FreescaleIMX51DIOCacheBus_OBit;
    FreescaleIMX51DIOCacheBus0.icache=FreescaleIMX51DIOCacheBus_icache;
    FreescaleIMX51DIOCacheBus0.ocache=FreescaleIMX51DIOCacheBus_ocache;
    FreescaleIMX51DIOCacheBus0.ocache0=FreescaleIMX51DIOCacheBus_ocache0;
    return CacheBusInit(&FreescaleIMX51DIOCacheBus0,BusInit(3));
  }
  if (inst == 11) { // FreescaleIMX51DIOCacheBus1
    FreescaleIMX51DIOCacheBus_IBit[2] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[0] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[1] = 0xFFFFFFFF;

    FreescaleIMX51DIOCacheBus1.Length=FreescaleIMX51DIOCacheBusLength;
    FreescaleIMX51DIOCacheBus1.Width=32;
    FreescaleIMX51DIOCacheBus1.Offset=0;

    FreescaleIMX51DIOCacheBus1.WO=FreescaleIMX51DIOCacheBus_WO;
    FreescaleIMX51DIOCacheBus1.IBit=FreescaleIMX51DIOCacheBus_IBit;
    FreescaleIMX51DIOCacheBus1.OBit=FreescaleIMX51DIOCacheBus_OBit;
    FreescaleIMX51DIOCacheBus1.icache=FreescaleIMX51DIOCacheBus_icache;
    FreescaleIMX51DIOCacheBus1.ocache=FreescaleIMX51DIOCacheBus_ocache;
    FreescaleIMX51DIOCacheBus1.ocache0=FreescaleIMX51DIOCacheBus_ocache0;
    return CacheBusInit(&FreescaleIMX51DIOCacheBus1,BusInit(4));
  }
  if (inst == 12) { // FreescaleIMX51DIOCacheBus2
    FreescaleIMX51DIOCacheBus_IBit[2] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[0] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[1] = 0xFFFFFFFF;

    FreescaleIMX51DIOCacheBus2.Length=FreescaleIMX51DIOCacheBusLength;
    FreescaleIMX51DIOCacheBus2.Width=32;
    FreescaleIMX51DIOCacheBus2.Offset=0;

    FreescaleIMX51DIOCacheBus2.WO=FreescaleIMX51DIOCacheBus_WO;
    FreescaleIMX51DIOCacheBus2.IBit=FreescaleIMX51DIOCacheBus_IBit;
    FreescaleIMX51DIOCacheBus2.OBit=FreescaleIMX51DIOCacheBus_OBit;
    FreescaleIMX51DIOCacheBus2.icache=FreescaleIMX51DIOCacheBus_icache;
    FreescaleIMX51DIOCacheBus2.ocache=FreescaleIMX51DIOCacheBus_ocache;
    FreescaleIMX51DIOCacheBus2.ocache0=FreescaleIMX51DIOCacheBus_ocache0;
    return CacheBusInit(&FreescaleIMX51DIOCacheBus2,BusInit(5));
  }
  if (inst == 13) { // FreescaleIMX51DIOCacheBus3
    FreescaleIMX51DIOCacheBus_IBit[2] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[0] = 0xFFFFFFFF;
    FreescaleIMX51DIOCacheBus_OBit[1] = 0xFFFFFFFF;

    FreescaleIMX51DIOCacheBus3.Length=FreescaleIMX51DIOCacheBusLength;
    FreescaleIMX51DIOCacheBus3.Width=32;
    FreescaleIMX51DIOCacheBus3.Offset=0;

    FreescaleIMX51DIOCacheBus3.WO=FreescaleIMX51DIOCacheBus_WO;
    FreescaleIMX51DIOCacheBus3.IBit=FreescaleIMX51DIOCacheBus_IBit;
    FreescaleIMX51DIOCacheBus3.OBit=FreescaleIMX51DIOCacheBus_OBit;
    FreescaleIMX51DIOCacheBus3.icache=FreescaleIMX51DIOCacheBus_icache;
    FreescaleIMX51DIOCacheBus3.ocache=FreescaleIMX51DIOCacheBus_ocache;
    FreescaleIMX51DIOCacheBus3.ocache0=FreescaleIMX51DIOCacheBus_ocache0;
    return CacheBusInit(&FreescaleIMX51DIOCacheBus3,BusInit(6));
  }
  if (inst == 14) { // WindowBus0
    return WindowBusInit(&WindowBus0,BusInit(7),0,2);
  }
  if (inst == 15) { // WindowBus1
    return WindowBusInit(&WindowBus1,BusInit(8),0,2);
  }
  if (inst == 16) { // SPI
    return MMapBusInit(&SPIBus,0xB0013000,1);
  }
  return bus;
}

Pin *ts4800_PinInit(Pin *pin,int inst) {
  if (inst == 0) {
    return ts4800PinInit(&ts4800Pin0,BusInit(0),BusInit(4));
  }
  return pin;
}

int CANBusNum[2] = { 14, 15 };

CAN *ts4800_CANInit(CAN *can,int inst) {
  if (inst == 0) {
    CAN0.LockNum = 2;
    CAN0.status = 0;
    CAN0.CAN_TX = 47;
    CAN0.CAN_RX = 45;
    CAN0.D.conn = CAN0conn;
    CAN0.D.maxconn = 16;
    CAN0.D.nconn = 0;
    CAN0.D.txc = 0;
    CAN0.baud = 1000000;
    CAN0.baudparms = baudparms25;
    CAN0.irq = 273;
    return SJA1000CANInit(&CAN0,BusInit(14),PinInit(0),TimeInit(0));
  }
  if (inst == 1) {
    CAN1.LockNum = 3;
    CAN1.status = 0;
    CAN1.CAN_TX = 10;
    CAN1.CAN_RX = 11;
    CAN1.D.conn = CAN1conn;
    CAN1.D.maxconn = 16;
    CAN1.D.nconn = 0;
    CAN1.D.txc = 0;
    CAN1.baud = 1000000;
    CAN1.baudparms = baudparms25;
    CAN1.irq = 274;
    return SJA1000CANInit(&CAN1,BusInit(15),PinInit(0),TimeInit(0));
  }
  return can;
}

DIORaw *ts4800_DIORawInit(DIORaw *dioraw,int inst) {
  if (inst == 0) {
    return ts4800DIORawInit(&ts4800DIORaw0,BusInit(9));
  }
  if (inst == 1) {
    return FreescaleIMX51DIORawInit(&FreescaleIMX51DIORaw0,BusInit(10));
  }
  if (inst == 2) {
    return FreescaleIMX51DIORawInit(&FreescaleIMX51DIORaw1,BusInit(11));
  }
  if (inst == 3) {
    return FreescaleIMX51DIORawInit(&FreescaleIMX51DIORaw2,BusInit(12));
  }
  if (inst == 4) {
    return FreescaleIMX51DIORawInit(&FreescaleIMX51DIORaw3,BusInit(13));
  }
  return dioraw;
}

DIO *ts4800_DIOInit(DIO *dio,int inst) {
  int i,n;

  if (inst == 0) {
    return DIOInit0(&DIO0);
  }
  if (inst == 1) {
    DIORaw *raw = DIORawInit(0);

    n = raw->API->Count(raw);
    ts4800DIO.Capabilities = ts4800DIOCapabilities;
    for (i=0;i<n;i++) {
      ts4800DIOCapabilities[i] = DIO_NORMAL;
    }
    ts4800DIOCapabilities[64] = DIO_OUTPUT_ONLY; // GREEN_LED
    ts4800DIOCapabilities[65] = DIO_OUTPUT_ONLY; // RED_LED
    return PhysicalDIOInit(&ts4800DIO,BusInit(9),raw);
  }
  if (inst == 2) {
    DIORaw *raw = DIORawInit(1);

    n = raw->API->Count(raw);
    FreescaleIMX51DIO0.Capabilities = FreescaleIMX51DIOCapabilities0;
    FreescaleIMX51DIOCapabilities0[3] = DIO_NORMAL;
    FreescaleIMX51DIOCapabilities0[5] = DIO_NORMAL;
    FreescaleIMX51DIOCapabilities0[6] = DIO_NORMAL;
    FreescaleIMX51DIOCapabilities0[7] = DIO_NORMAL;
    return PhysicalDIOInit(&FreescaleIMX51DIO0,BusInit(10),raw);
  }
  if (inst == 3) {
    DIORaw *raw = DIORawInit(2);

    n = raw->API->Count(raw);
    FreescaleIMX51DIO1.Capabilities = FreescaleIMX51DIOCapabilities1;
    return PhysicalDIOInit(&FreescaleIMX51DIO1,BusInit(11),raw);
  }
  if (inst == 4) {
    DIORaw *raw = DIORawInit(3);

    n = raw->API->Count(raw);
    FreescaleIMX51DIO2.Capabilities = FreescaleIMX51DIOCapabilities2;
    return PhysicalDIOInit(&FreescaleIMX51DIO2,BusInit(12),raw);
  }
  if (inst == 5) {
    DIORaw *raw = DIORawInit(4);

    n = raw->API->Count(raw);
    FreescaleIMX51DIO3.Capabilities = FreescaleIMX51DIOCapabilities3;
    FreescaleIMX51DIOCapabilities3[13] = DIO_NORMAL;
    FreescaleIMX51DIOCapabilities3[14] = DIO_NORMAL;
    return PhysicalDIOInit(&FreescaleIMX51DIO3,BusInit(13),raw);
  }
  return dio;
}

TWI *ts4800_TWIInit(TWI *twi,int inst) {
  if (inst == 0) {
    // I2C_CLK = IMX51:F16
    // I2C_DAT = IMX51:C16
    //TWI0.Speed=1000000;
    //TWI0.LockNum=1;
    return LinuxTWIInit(&TWI0,"/dev/i2c-1");
  }
  return twi;
}

DIO *dio0;

static void ts4800SPIChipSelect(WBSPI *ob,unsigned num,int asserted) {
  if (num > 3) return;
  if (num == 0) {
    if (!asserted) return; // core will do this automatically
    ob->bus->API->Lock(ob->bus,0,0);
    ob->bus->API->Poke16(ob->bus,ob->offset,
			 (ob->bus->API->Peek16(ob->bus,ob->offset) & 0xFC7F)
			 | (num << 8) | (1 << 7));
    ob->bus->API->Unlock(ob->bus,0,0);
    return;
  }
  // Now use DIO
  switch (num) {
  case 1: dio0->API->SetAsync(dio0,18,asserted?LOW:INPUT); break;
  case 2: dio0->API->SetAsync(dio0,127,asserted?LOW:INPUT); break;
  case 3: dio0->API->SetAsync(dio0,128,asserted?LOW:INPUT); break;
  }
}
#include "peekpoke.h"

SPI *ts4800_SPIInit(SPI *spi,int inst) {
  // write 8 to 0x83fda00c to set up WEIM to use byte selects
  volatile unsigned *reg = MemMap(0x83fda000,1);
  reg[0xC/sizeof(unsigned)] = 8;
  MemUnmap(reg);

  if (inst == 0) {
    Bus *bus;
    dio0 = DIOInit(0); // needed by ts4800SPIChipSelect
    SPI0.LockNum = 7;
    bus = BusInit(0);
    bus->API->BitSet16(bus,0x10,7);
    return WBSPIInit(&SPI0,BusInit(16),BusInit(16),0,ts4800SPIChipSelect);
  }
  return spi;
}

AIO *ts4800_AIOInit(AIO *aio,int inst) {
  return aio;
}

ArchInfo ts4800_ArchInfo = {
  .Instances={1,17,2,1,5,6,1,2,1},
  .CANBusNum=CANBusNum,
  .ArchInit=ts4800_ArchInit,
  .SystemInit=ts4800_SystemInit,
  .TimeInit=ts4800_TimeInit,
  .BusInit=ts4800_BusInit,
  .PinInit=ts4800_PinInit,
  .CANInit=ts4800_CANInit,
  .DIORawInit=ts4800_DIORawInit,
  .DIOInit=ts4800_DIOInit,
  .SPIInit=ts4800_SPIInit,
  .TWIInit=ts4800_TWIInit,
  .AIOInit=ts4800_AIOInit
  //.EDIOInit=ts4800_EDIOInit
};

#include "arch.c"

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2012, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.

