#include "MMapBus.c"
#include "Cavium2132SBus.c"
#include "Cavium2132SBusWindowBus.c"
#include "WBWindowBus.c"
#include "TSMuxBus.c"
//#include "PThreadLock.c"
#include "CacheBus.c"
#include "ts4500DIORaw.c"
#include "Cavium2132DIORaw.c"
#include "ts4500Pin.c"
#include "PhysicalDIO.c"
#include "AggregateDIO.c"
#include "DIOTWI.c"
#include "SystemTime.c"
#include "Cavium2132Time.c"
#include "SJA1000CAN.c"
#include "WBSPI.c"
#include "baudtab.h"
#include "arch2def.h"
#include "arch2.h"
#define ts7558DIOCount 2

#ifndef __MODEL_ID
int ModelId=0x7558;
#endif

// Locks:
// 0     SBus
// 1     TWI
// 2     CAN
// 3     CANBus
// 7     SPI
// 10-65 DIO 1
// 68-99 DIO 2
#define NLOCKS 256 // HACK: 132 with TS-81X0

MMapBus cpubus,testbus;
CacheBus ts4500DIOCacheBus,Cavium2132DIOCacheBus;
TSMuxBus muxbus;
WBWindowBus wbwin;
ts4500DIORaw ts4500DIORaw0;
Cavium2132DIORaw Cavium2132DIORaw0;
ts4500Pin ts4500Pin0;
PhysicalDIO ts4500DIO,Cavium2132DIO;
AggregateDIO DIO0;
DIOTWI TWI0;
SystemTime Time0;
Cavium2132Time Time1;
unsigned char ts4500DIOCapabilities[56];
CANConn CAN0conn[16];
SJA1000CAN CAN0;
WBSPI SPI0;

#define F(arch) + TEMPLATE(arch,DIOCount)
#define ArchDIOCount SUB_ARCH_LIST

DIO *pDIO[2+ArchDIOCount];
int SubCount[2+ArchDIOCount];
int sbuscount[1],sbusholder[1];

unsigned Cavium2132DIOCacheBus_WO[12];
unsigned Cavium2132DIOCacheBus_IBit[12];
unsigned Cavium2132DIOCacheBus_OBit[12];
unsigned Cavium2132DIOCacheBus_icache[12];
unsigned Cavium2132DIOCacheBus_ocache[12];
unsigned Cavium2132DIOCacheBus_ocache0[12];
unsigned char Cavium2132DIOCapabilities[32];

// Standard instances
// Bus 0 is whatever bus directly accesses the FPGA "syscon" registers
// For the TS-4500 this is the Cavium2132SBus
// Bus0

MMapBus spibus, gpiobus;
Cavium2132SBus sbus;
Cavium2132SBusWindowBus canbus;

// Bus1
unsigned ts4500DIOCacheBus_WO[15];
unsigned ts4500DIOCacheBus_IBit[15];
unsigned ts4500DIOCacheBus_OBit[15];
unsigned ts4500DIOCacheBus_icache[15];
unsigned ts4500DIOCacheBus_ocache[15];
unsigned ts4500DIOCacheBus_ocache0[15];

char testbusdata[4096];

Lock *LockInit(int inst);
void ArchInit1() {
  static int initialized = 0;
  if (initialized) return;
  initialized = 1;
  LockInit(0);
  ArchInitX();
}

Time *TimeInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(Time,2);
  if (inst == 0) {
    return SystemTimeInit(&Time0);
  }
  if (inst == 1) {
    return Cavium2132TimeInit(&Time1);
  }
  INIT_POSTAMBLE(Time,2);
}

Bus *BusInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(Bus,9);
  if (inst == 0) { // Cavium2132SBus
    MMapBusInit(&spibus,0x71000000,1);
    sbus.LockNum = 0;
    return Cavium2132SBusInit(&sbus,&spibus,BusInit1(6));
  }
  if (inst == 1) { // for testing
    return MMapBusInit2(&testbus,testbusdata,1);
  }

  if (inst == 2) { // MUXBus
    return TSMuxBusInit(&muxbus,BusInit1(8),0x20000,BusInit1(8),0x10000,
			BusInit1(8),0);
  }
  if (inst == 3) { // ts4500DIOCacheBus
    ts4500DIOCacheBus_IBit[0] = 0x3000;
    ts4500DIOCacheBus_IBit[2] = 0xF001;
    ts4500DIOCacheBus_IBit[3] = 0xFFFF;
    ts4500DIOCacheBus_IBit[6] = 0xFFFF;
    ts4500DIOCacheBus_IBit[11] = 0xFC00;
    ts4500DIOCacheBus_IBit[13] = 0xFFFF;
    ts4500DIOCacheBus_IBit[14] = 0xFF00;
    ts4500DIOCacheBus_OBit[0] = 0xCF00;
    ts4500DIOCacheBus_OBit[2] = 0x0FFE;
    ts4500DIOCacheBus_OBit[4] = 0xFFFF;
    ts4500DIOCacheBus_OBit[5] = 0xFFFF;
    ts4500DIOCacheBus_OBit[7] = 0xFFFF;
    ts4500DIOCacheBus_OBit[8] = 0xFFFF;
    ts4500DIOCacheBus_OBit[11] = 0x3FFF;
    ts4500DIOCacheBus_OBit[12] = 0xFFFF;
    ts4500DIOCacheBus_OBit[14] = 0x00FF;

    ts4500DIOCacheBus.Length=15;
    ts4500DIOCacheBus.Width=16;
    ts4500DIOCacheBus.Offset=0x62;
    ts4500DIOCacheBus.WO = ts4500DIOCacheBus_WO;
    ts4500DIOCacheBus.IBit = ts4500DIOCacheBus_IBit;
    ts4500DIOCacheBus.OBit = ts4500DIOCacheBus_OBit;
    ts4500DIOCacheBus.icache = ts4500DIOCacheBus_icache;
    ts4500DIOCacheBus.ocache = ts4500DIOCacheBus_ocache;
    ts4500DIOCacheBus.ocache0 = ts4500DIOCacheBus_ocache0;
    return CacheBusInit(&ts4500DIOCacheBus,BusInit1(0));
  }
  if (inst == 4) { // CPU bus
    return MMapBusInit(&cpubus,0x76000000,1);
  }
  if (inst == 5) { // Cavium2132DIOCacheBus
    Cavium2132DIOCacheBus_IBit[1] = 0xFFFFFFFF;
    Cavium2132DIOCacheBus_OBit[0] = 0xFFFFFFFF;
    Cavium2132DIOCacheBus_OBit[2] = 0xFFFFFFFF;
    Cavium2132DIOCacheBus.Length=12;
    Cavium2132DIOCacheBus.Width=32;
    Cavium2132DIOCacheBus.Offset=0;
    Cavium2132DIOCacheBus.IBit = Cavium2132DIOCacheBus_IBit;
    Cavium2132DIOCacheBus.OBit = Cavium2132DIOCacheBus_OBit;
    Cavium2132DIOCacheBus.icache = Cavium2132DIOCacheBus_icache;
    Cavium2132DIOCacheBus.ocache = Cavium2132DIOCacheBus_ocache;
    Cavium2132DIOCacheBus.ocache0 = Cavium2132DIOCacheBus_ocache0;
    return CacheBusInit(&Cavium2132DIOCacheBus,BusInit1(6));
  }
  if (inst == 6) { // gpiobus
    return MMapBusInit(&gpiobus,0x7C000000,1);
  }
  if (inst == 7) { // Cavium2132SBusWindowBus
    canbus.LockNum = 3;
    canbus.MemBase = 0;
    return WBWindowBusInit(&wbwin,BusInit1(0),0x18,0);
  }
  if (inst == 8) { // WBWindowBus
    return WBWindowBusInit(&wbwin,BusInit1(0),0x18,0);
  }
  INIT_POSTAMBLE(Bus,9);
}

Pin *PinInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(Pin,1);
  if (inst == 0) {
    return ts4500PinInit(&ts4500Pin0,BusInit1(0),BusInit1(4),
			 BusInit1(8),0x20000);
  }
  INIT_POSTAMBLE(Pin,1);
}

int CANBusNum[1] = { 7 };

CAN *CANInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(CAN,1);
  if (inst == 0) {
    CAN0.LockNum = 2;
    CAN0.status = 0;
    CAN0.CAN_TX = 61;
    CAN0.CAN_RX = 55;
    CAN0.D.conn = CAN0conn;
    CAN0.D.maxconn = 16;
    CAN0.D.nconn = 0;
    CAN0.D.txc = 0;
    CAN0.baud = 1000000;
    CAN0.baudparms = baudparms25;
    CAN0.irq = 29;
    return SJA1000CANInit(&CAN0,BusInit1(7),PinInit1(0),TimeInit1(1));
  }
  INIT_POSTAMBLE(CAN,1);
}

DIORaw *DIORawInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(DIORaw,2);
  if (inst == 0) {
    return ts4500DIORawInit(&ts4500DIORaw0,BusInit1(3));
  }
  if (inst == 1) {
    return Cavium2132DIORawInit(&Cavium2132DIORaw0,BusInit1(5));
  }
  INIT_POSTAMBLE(DIORaw,2);
}

DIO *DIOInit1(int inst) {
  ArchInit1();
  int i;

  INIT_PREAMBLE(DIO,3);
  if (inst == 0) {
    for (i=0;i<4+ArchDIOCount;i++) {
      pDIO[i] = DIOInit1(i+1);
    }
    return AggregateDIOInit(&DIO0,2+ArchDIOCount,PinInit1(0),SubCount,pDIO);
  }
  if (inst == 1) {
    ts4500DIO.Capabilities = ts4500DIOCapabilities;
    for (i=0;i<=52;i++) {
      ts4500DIOCapabilities[i] = DIO_NORMAL;
    }
    ts4500DIOCapabilities[54] = DIO_OUTPUT_ONLY;
    ts4500DIOCapabilities[55] = DIO_OUTPUT_ONLY;
    return PhysicalDIOInit(&ts4500DIO,BusInit1(3),DIORawInit1(0));
  }
  if (inst == 2) {
    Cavium2132DIO.Capabilities = Cavium2132DIOCapabilities;
    Cavium2132DIOCapabilities[13] = DIO_NORMAL;
    Cavium2132DIOCapabilities[14] = DIO_NORMAL;
    return PhysicalDIOInit(&Cavium2132DIO,BusInit1(5),DIORawInit1(1));
  }
  INIT_POSTAMBLE(DIO,3);
}

TWI *TWIInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(TWI,1);
  if (inst == 0) {
    TWI0.TW_CLK=14+56;
    TWI0.TW_DAT=13+56;
    TWI0.Speed=1000000;
    TWI0.LockNum=1;
    return DIOTWIInit(&TWI0,DIOInit1(0),TimeInit1(0));
  }
  INIT_POSTAMBLE(TWI,1);
}

static void ts4500SPIChipSelect(WBSPI *ob,unsigned num,int asserted) {
  if (num > 3) return;
  if (!asserted) return; // core will do this automatically
  if (num == 0) num = 1; else if (num == 1) num = 0;
  ob->bus->API->Lock(ob->bus,WaitLock,0);
  ob->bus->API->Poke16(ob->bus,ob->offset,
		  (ob->bus->API->Peek16(ob->bus,ob->offset) & 0xFC7F)
		       | (num << 8) /*| (1 << 7)*/);
  ob->bus->API->Lock(ob->bus,DoRelease,0);
  return;
}

SPI *SPIInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(SPI,1);
  if (inst == 0) {
    SPI0.LockNum = 7;
    return WBSPIInit(&SPI0,BusInit1(0),BusInit1(0),0x40,ts4500SPIChipSelect);
  }
  INIT_POSTAMBLE(SPI,1);
}

AIO *AIOInit1(int inst) {
  ArchInit1();
  INIT_PREAMBLE(AIO,0);
  INIT_POSTAMBLE(AIO,0);
}

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
