#include <stdio.h>
#include <stdlib.h>
#include "Time.h"
#include "Bus.h"
#include "Pin.h"
#include "CAN.h"
#include "DIORaw.h"
#include "DIO.h"
#include "TWI.h"
#include "ts81x0DIORaw.c"
#include "ts81x0Pin.c"
#include "PhysicalDIO.h"
#include "CacheBus.h"
#include "arch.h"

static ts81x0DIORaw dio;
static PhysicalDIO pdio;
static unsigned char ts81x0DIOCapabilities[32];
static CacheBus ts81x0DIOCacheBus;
static ts81x0Pin pin;

unsigned ts81x0DIOCacheBus_WO[8];
unsigned ts81x0DIOCacheBus_IBit[8];
unsigned ts81x0DIOCacheBus_OBit[8];
unsigned ts81x0DIOCacheBus_icache[8];
unsigned ts81x0DIOCacheBus_ocache[8];
unsigned ts81x0DIOCacheBus_ocache0[8];

void ts81x0_ArchInit();

System *ts81x0_SystemInit(System *s,int inst) {
  return s;
}

Time *ts81x0_TimeInit(Time *t,int inst) {
  return t;
}

Bus *ts81x0_BusInit(Bus *bus,int inst) {
  if (inst == 0) {
    ts81x0DIOCacheBus_IBit[6] = 0xFFFF;
    ts81x0DIOCacheBus_IBit[7] = 0x03FF;
    ts81x0DIOCacheBus_OBit[2] = 0xFFFF;
    ts81x0DIOCacheBus_OBit[3] = 0x03FF;
    ts81x0DIOCacheBus_OBit[4] = 0xFFFF;
    ts81x0DIOCacheBus_OBit[5] = 0x03FF;
    ts81x0DIOCacheBus.Length = 8;
    ts81x0DIOCacheBus.Width = 16;
    ts81x0DIOCacheBus.Offset = 0;
    ts81x0DIOCacheBus.WO = ts81x0DIOCacheBus_WO;
    ts81x0DIOCacheBus.IBit = ts81x0DIOCacheBus_IBit;
    ts81x0DIOCacheBus.OBit = ts81x0DIOCacheBus_OBit;
    ts81x0DIOCacheBus.icache = ts81x0DIOCacheBus_icache;
    ts81x0DIOCacheBus.ocache = ts81x0DIOCacheBus_ocache;
    ts81x0DIOCacheBus.ocache0 = ts81x0DIOCacheBus_ocache0;
    return CacheBusInit(&ts81x0DIOCacheBus,BusInit(2));
  }
  return bus;
}

Pin *ts81x0_PinInit(Pin *pin0,int inst) {
  if (inst == -1) {
    static int entered = 0;
    int CAN_TX1,CAN_TX2,CAN_RX1,CAN_RX2;
    CAN *can0,*can1;
    Pin *ret;

    if (entered) return (Pin *)&pin;
    entered = 1;
    ret = ts81x0PinInit(&pin,pin0,BusInit(2));
    can0 = CANInit(0);
    can1 = CANInit(1);
    ts81x0PinPostInit(&pin,can0 ? can0->CAN_TX : -1,
			can1 ? can1->CAN_TX : -1,
			can0 ? can0->CAN_RX : -1,
			can1 ? can1->CAN_RX : -1);
    entered = 0;
    return ret;
  }
  return pin0;
}

CAN *ts81x0_CANInit(CAN *can,int inst) {
  return can;
}

DIORaw *ts81x0_DIORawInit(DIORaw *dioraw,int inst) {
  if (inst == 0) {
    return ts81x0DIORawInit(&dio,ts81x0_BusInit(0,0));
  }
  return dioraw;
}

DIO *ts81x0_DIOInit(DIO *dio,int inst) {
  if (inst == 0) {
    int i;
    for (i=0;i<27;i++) {
      ts81x0DIOCapabilities[i] = DIO_NORMAL;
    }
    pdio.Capabilities = ts81x0DIOCapabilities;
    return PhysicalDIOInit(&pdio,ts81x0_BusInit(0,0),ts81x0_DIORawInit(0,0));
  }
  return dio;
}

TWI *ts81x0_TWIInit(TWI *twi,int inst) {
  return twi;
}

SPI *ts81x0_SPIInit(SPI *spi,int inst) {
  return spi;
}

AIO *ts81x0_AIOInit(AIO *aio,int inst) {
  return aio;
}

ArchInfo ts81x0_ArchInfo = {
  .Instances={0,1,0,1,1,1,0,0,0},
  .CANBusNum=0,
  .ArchInit=ts81x0_ArchInit,
  .SystemInit=ts81x0_SystemInit,
  .TimeInit=ts81x0_TimeInit,
  .BusInit=ts81x0_BusInit,
  .PinInit=ts81x0_PinInit,
  .CANInit=ts81x0_CANInit,
  .DIORawInit=ts81x0_DIORawInit,
  .DIOInit=ts81x0_DIOInit,
  .SPIInit=ts81x0_SPIInit,
  .TWIInit=ts81x0_TWIInit,
  .AIOInit=ts81x0_AIOInit
  //.EDIOInit=ts81x0_EDIOInit
};

void ts81x0_ArchInit() {
  Bus *muxbus = BusInit(2);
  int model;
  muxbus->API->Lock(muxbus,0,SHARED);
  model = muxbus->API->Peek16(muxbus,0);
  muxbus->API->Unlock(muxbus,0,SHARED);
  if (model != 0x8100) {
    fprintf(stderr,"This binary was compiled for TS-81x0 boards\n");
    fprintf(stderr,"Error: detected model %04X\n",model);
    exit(1);
  }
}
