#include <stdio.h>
#include <stdlib.h>
#include "Time.h"
#include "Bus.h"
#include "Pin.h"
#include "CAN.h"
#include "DIORaw.h"
#include "DIO.h"
#include "TWI.h"
#include "ts8900DIORaw.c"
#include "ts8900Pin.c"
#include "PhysicalDIO.h"
#include "CacheBus.h"
#include "arch2def.h"
#include "arch2.h"

static ts8900DIORaw dio;
static PhysicalDIO pdio;
static unsigned char ts8900DIOCapabilities[41];
static CacheBus ts8900DIOCacheBus;
static ts8900Pin pin;

unsigned ts8900DIOCacheBus_WO[8];
unsigned ts8900DIOCacheBus_IBit[8];
unsigned ts8900DIOCacheBus_OBit[8];
unsigned ts8900DIOCacheBus_icache[8];
unsigned ts8900DIOCacheBus_ocache[8];
unsigned ts8900DIOCacheBus_ocache0[8];

Time *TimeInit(int inst) {
  INIT_PREAMBLE(Time,0);
  INIT_POSTAMBLE(Time,0);
}

// 0x08 [ 4] [15:0]=GPIOEN[15:0]
// 0x0A [ 5] [8]=GPIOEN[16], [7:0]=ISAEN
// 0x0C [ 6] [15:0]=GPIODATA[15:0]
// 0x0E [ 7] [8]=GPIODATA[16], [7:0]=ISADATA[7:0]
// 0x10 [ 8] [15:0]=GPIODIR[15:0]
// 0x12 [ 9] [8]=GPIODIR[16], [7:0]=ISADIR[7:0]
// 0x14 [10] [15:0]=GPIOIN[15:0]
// 0x16 [11] [8]=GPIOIN[16], [7:0]=ISAIN[7:0]

// IBit
// 0x08 [ 4] 0x0000
// 0x0A [ 5] 0x0000
// 0x0C [ 6] 0x0000
// 0x0E [ 7] 0x0000
// 0x10 [ 8] 0x0000
// 0x12 [ 9] 0x0000
// 0x14 [10] 0xFFFF
// 0x16 [11] 0x01FF
// OBit
// 0x08 [ 4] 0xFFFF
// 0x0A [ 5] 0x01FF
// 0x0C [ 6] 0xFFFF
// 0x0E [ 7] 0x01FF
// 0x10 [ 8] 0xFFFF
// 0x12 [ 9] 0x01FF
// 0x14 [10] 0x0000
// 0x16 [11] 0x0000

Bus *BusInit(int inst) {
  INIT_PREAMBLE(Bus,1);
  if (inst == 0) {

    ts8900DIOCacheBus_IBit[6] = 0xFFFF;
    ts8900DIOCacheBus_IBit[7] = 0x01FF;

    ts8900DIOCacheBus_OBit[0] = 0xFFFF;
    ts8900DIOCacheBus_OBit[1] = 0x01FF;
    ts8900DIOCacheBus_OBit[2] = 0xFFFF;
    ts8900DIOCacheBus_OBit[3] = 0x01FF;
    ts8900DIOCacheBus_OBit[4] = 0xFFFF;
    ts8900DIOCacheBus_OBit[5] = 0x01FF;
    ts8900DIOCacheBus.Length = 8;
    ts8900DIOCacheBus.Width = 16;
    ts8900DIOCacheBus.Offset = 8;
    ts8900DIOCacheBus.WO = ts8900DIOCacheBus_WO;
    ts8900DIOCacheBus.IBit = ts8900DIOCacheBus_IBit;
    ts8900DIOCacheBus.OBit = ts8900DIOCacheBus_OBit;
    ts8900DIOCacheBus.icache = ts8900DIOCacheBus_icache;
    ts8900DIOCacheBus.ocache = ts8900DIOCacheBus_ocache;
    ts8900DIOCacheBus.ocache0 = ts8900DIOCacheBus_ocache0;
    return CacheBusInit(&ts8900DIOCacheBus,BusInit1(2));
  }
  INIT_POSTAMBLE(Bus,1);
}

int isme8900;

Pin *PinInit(int inst) {
  if (inst == -1) {
    int i=0,start=0;
    DIORaw *dr;
    isme8900=0;
    do {
      dr = DIORawInit1(i++);
      if (dr && !isme8900) {
	start += dr->API->Count(dr);
      }
    } while (dr && ~isme8900);
    return ts8900PinInit(&pin,PinInit1(0),BusInit1(2),start);
  }
  INIT_PREAMBLE(Pin,0);
  INIT_POSTAMBLE(Pin,0);
}

CAN *CANInit(int inst) {
  INIT_PREAMBLE(CAN,0);
  INIT_POSTAMBLE(CAN,0);
}

DIORaw *DIORawInit(int inst) {
  INIT_PREAMBLE(DIORaw,1);
  if (inst == 0) {
    isme8900=1;
    return ts8900DIORawInit(&dio,BusInit(0));
  }
  INIT_POSTAMBLE(DIORaw,1);
}

DIO *DIOInit(int inst) {
  INIT_PREAMBLE(DIO,1);
  if (inst == 0) {
    int i;
    for (i=0;i<7;i++) {
      ts8900DIOCapabilities[i] = DIO_INPUT_ONLY;
    } 
    for (i=8;i<15;i++) {
      ts8900DIOCapabilities[i] = DIO_OUTPUT_ONLY;
    } 
    for (i=16;i<41;i++) {
      ts8900DIOCapabilities[i] = DIO_NORMAL;
    }
    pdio.Capabilities = ts8900DIOCapabilities;
    return PhysicalDIOInit(&pdio,BusInit(0),DIORawInit(0));
  }
  INIT_POSTAMBLE(DIO,1);
}

TWI *TWIInit(int inst) {
  INIT_PREAMBLE(TWI,0);
  INIT_POSTAMBLE(TWI,0);
}

SPI *SPIInit(int inst) {
  INIT_PREAMBLE(SPI,0);
  INIT_POSTAMBLE(SPI,0);
}

AIO *AIOInit(int inst) {
  INIT_PREAMBLE(AIO,0);
  INIT_POSTAMBLE(AIO,0);
}

void ArchInit() {
  Bus *muxbus = BusInit1(2);
  int model;
  muxbus->API->Lock(muxbus,WaitLock,0);
  model = muxbus->API->Peek16(muxbus,0);
  if (model != 0x8900) { // bug workaround
    model = muxbus->API->Peek16(muxbus,0);
  }
  muxbus->API->Lock(muxbus,DoRelease,0);
  if (model != 0x8900) {
    fprintf(stderr,"This binary was compiled for TS-8900 boards\n");
    fprintf(stderr,"Error: detected model %04X\n",model);
    exit(1);
  }
  ArchInitX();
}
