#include "ts8900DIORaw.h"

#define I(x) ( Num<0 ? ~(x) : (x))

static DIORawAPI ts8900DIORawAPI = {
  .Init = (void *)ts8900DIORawInit,
  .Fini = (void *)ts8900DIORawFini,
  .Lock = (void *)ts8900DIORawLock,
  .DirSet = (void *)ts8900DIORawDirSet,
  .DataSet = (void *)ts8900DIORawDataSet,
  .DirGet = (void *)ts8900DIORawDirGet,
  .DataGet = (void *)ts8900DIORawDataGet,
  .Count = (void *)ts8900DIORawCount
};

void *ts8900DIORawInit(ts8900DIORaw *dio,void *bus1) {
  if (dio->InitStatus > 0) return dio;
  dio->API = &ts8900DIORawAPI;
  dio->bus = bus1;
  if (dio->bus->InitStatus <= 0) {
    dio->InitStatus = -1;
  }
  dio->InitStatus = 1;
  return dio;
}

void ts8900DIORawFini(ts8900DIORaw *dio) {
  dio->bus->API->Fini(dio->bus);
  if (dio->InitStatus > 0) dio->InitStatus = 0;
}

int ts8900DIORawLock(ts8900DIORaw *obj,int command,unsigned num) {
  switch (command) {
  case WaitLock:
  case TryLock:
    obj->deferlock++;
    return 1;
  case DoRelease:
    if (obj->deferlock && --obj->deferlock) {
      return 1;
    } else {
      return obj->bus->API->Lock(obj->bus,command,0);
    }
  case DoPreempt:
    if (!obj->deferlock) {
      return obj->bus->API->Lock(obj->bus,command,0);
    } else {
      return 1;
    }
  }
}

// DIO  0-7=digital inputs
// DIO 8-15=digital outputs
// DIO 16-32=PC104 pins
// DIO 33-41=ISA pins
void ts8900DIORawDirSet(ts8900DIORaw *dio,int Num,int asOutput) {
  if (I(Num) > 41) return;
  while (dio->deferlock) {
    dio->bus->API->Lock(dio->bus,WaitLock,0);
    dio->deferlock--;
  }  
  if (I(Num) >= 16 && I(Num) <= 31) {
    dio->bus->API->BitAssign16(dio->bus,0x10,I(Num)-16,asOutput);
  } else if (I(Num) == 32) {
    dio->bus->API->BitAssign16(dio->bus,0x12,8,asOutput);
  } else if (I(Num) >= 33 && I(Num) <= 41) {
    dio->bus->API->BitAssign16(dio->bus,0x12,I(Num)-33,asOutput);
  } 
}

void ts8900DIORawDataSet(ts8900DIORaw *dio,int Num,int asHigh) {
  if (I(Num) > 26) return;
  while (dio->deferlock) {
    dio->bus->API->Lock(dio->bus,WaitLock,0);
    dio->deferlock--;
  }  
  if (I(Num) >= 8 && I(Num) <= 15) {
    dio->bus->API->BitAssign16(dio->bus,0x04,I(Num),asHigh);
  } else if (I(Num) >= 16 && I(Num) <= 31) {
    dio->bus->API->BitAssign16(dio->bus,0x0C,I(Num)-16,asHigh);
  } else if (I(Num) == 32) {
    dio->bus->API->BitAssign16(dio->bus,0x0E,8,asHigh);
  } else if (I(Num) >= 33 && I(Num) <= 41) {
    dio->bus->API->BitAssign16(dio->bus,0x0E,I(Num)-33,asHigh);
  }
}

int ts8900DIORawDirGet(ts8900DIORaw *dio,int Num) {
  if (I(Num) > 26) return 0;
  while (dio->deferlock) {
    dio->bus->API->Lock(dio->bus,WaitLock,0);
    dio->deferlock--;
  }  
  if (I(Num) >= 0 && I(Num) <= 7) {
    return 0;
  } else if (I(Num) >= 8 && I(Num) <= 15) {
    return 1;
  } else if (I(Num) >= 16 && I(Num) <= 31) {
    return dio->bus->API->BitGet16(dio->bus,0x10,I(Num)-16);
  } else if (I(Num) == 32) {
    return dio->bus->API->BitGet16(dio->bus,0x12,8);
  } else if (I(Num) >= 33 && I(Num) <= 41) {
    return dio->bus->API->BitGet16(dio->bus,0x12,I(Num)-33);
  } 
  return 0;
}

int ts8900DIORawDataGet(ts8900DIORaw *dio,int Num) {
  if (I(Num) > 26) return 0;
  while (dio->deferlock) {
    dio->bus->API->Lock(dio->bus,WaitLock,0);
    dio->deferlock--;
  }  
  if (I(Num) >= 0 && I(Num) <= 7) {
    return dio->bus->API->BitGet16(dio->bus,0x02,I(Num)+8);
  } else if (I(Num) >= 8 && I(Num) <= 15) {
    return dio->bus->API->BitGet16(dio->bus,0x04,I(Num));
  } else if (I(Num) >= 16 && I(Num) <= 31) {
    return dio->bus->API->BitGet16(dio->bus,0x14,I(Num)-16);
  } else if (I(Num) == 32) {
    return dio->bus->API->BitGet16(dio->bus,0x16,8);
  } else if (I(Num) >= 33 && I(Num) <= 41) {
    return dio->bus->API->BitGet16(dio->bus,0x16,I(Num)-33);
  }
}

unsigned ts8900DIORawCount(ts8900DIORaw *dio) {
  return 42;
}

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
