#include "ts8900Pin.h"
#include "Bus.h"
#include "Lock.h"

static PinAPI ts8900PinAPI = {
  .Init = (void *)ts8900PinInit,
  .Fini = (void *)ts8900PinFini,
  .Lock = (void *)ts8900PinLock,
  .ModeGet = (void *)ts8900PinModeGet,
  .ModeSet = (void *)ts8900PinModeSet
};

void *ts8900PinInit(ts8900Pin *pin,void *parent,void *bus,int start) {
  if (pin->InitStatus > 0) return pin;
  pin->API = &ts8900PinAPI;
  pin->parent = parent;
  pin->bus = bus;
  pin->InitStatus = 1;
  pin->start = start;
  return pin;
}

void ts8900PinFini(ts8900Pin *pin) { 
  if (pin->InitStatus > 0) pin->InitStatus = 0;
}

int ts8900PinLock(ts8900Pin *pin,int command,unsigned num) {
    return lock->API->Lock(lock,command,num);
}

PinMode ts8900PinModeGet(ts8900Pin *pin,int npin) {
  int ret;

  npin -= pin->start;
  if (npin >= 16 && npin <= 31) {
    pin->bus->API->Lock(pin->bus,WaitLock,0);
    ret=pin->bus->API->BitGet16(pin->bus,0x08,npin-16) ? MODE_DIO:MODE_BUS;
    pin->bus->API->Lock(pin->bus,DoRelease,0);
    return ret;
  } else if (npin == 32) {
    pin->bus->API->Lock(pin->bus,WaitLock,0);
    ret=pin->bus->API->BitGet16(pin->bus,0x0A,8) ? MODE_DIO:MODE_BUS;
    pin->bus->API->Lock(pin->bus,DoRelease,0);
    return ret;
  } else if (npin >= 33 && npin <= 41) {
    pin->bus->API->Lock(pin->bus,WaitLock,0);
    ret=pin->bus->API->BitGet16(pin->bus,0x0A,npin-33) ? MODE_DIO:MODE_BUS;
    pin->bus->API->Lock(pin->bus,DoRelease,0);
    return ret;
  }
  npin += pin->start;
  return pin->parent->API->ModeGet(pin->parent,npin);
}

int ts8900PinModeSet(ts8900Pin *pin,int npin,PinMode mode) {
  int ret;

  npin -= pin->start;
  if (npin >= 16 && npin <= 31) {
    pin->bus->API->Lock(pin->bus,WaitLock,0);
    if (mode == MODE_DIO) {
      pin->bus->API->BitSet16(pin->bus,0x08,npin-16);
      ret=1;
    } else if (mode == MODE_BUS) {
      pin->bus->API->BitClear16(pin->bus,0x08,npin-16);
      ret=1;
    } else {
      ret=-1;
    }
    pin->bus->API->Lock(pin->bus,DoRelease,0);
    return ret;
  } else if (npin == 32) {
    pin->bus->API->Lock(pin->bus,WaitLock,0);
    if (mode == MODE_DIO) {
      pin->bus->API->BitSet16(pin->bus,0x0A,8);
      ret=1;
    } else if (mode == MODE_BUS) {
      pin->bus->API->BitClear16(pin->bus,0x0A,8);
      ret=1;
    } else {
      ret=-1;
    }
    pin->bus->API->Lock(pin->bus,DoRelease,0);
    return ret;
  } else if (npin >= 33 && npin <= 41) {
    pin->bus->API->Lock(pin->bus,WaitLock,0);
    if (mode == MODE_DIO) {
      pin->bus->API->BitSet16(pin->bus,0x0A,npin-33);
      ret=1;
    } else if (mode == MODE_BUS) {
      pin->bus->API->BitClear16(pin->bus,0x0A,npin-33);
      ret=1;
    } else {
      ret=-1;
    }
    pin->bus->API->Lock(pin->bus,DoRelease,0);
    return ret;
  }
  npin += pin->start;
  return pin->parent->API->ModeSet(pin->parent,npin,mode);
}
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
