#define __USE_GNU // stdio.h indirectly includes pthread.h under uclib.  WTF!
#include <assert.h>
#include <sys/epoll.h>
#include <asm/ioctls.h>
#include <errno.h>
#include <stdio.h>
#include <signal.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/resource.h>
#include "archdef.h"
#include "Net.h"
#include ARCH_C
#include "ts.c"
#include "opt.h"
#include "log.h"
#define LOCK_NOW -1
#define USE_MUTEXES

#include "IPC.h"
#include <stdio.h>		/* Basic I/O routines          */
#include <sys/types.h>		/* standard system types       */
#include <netinet/in.h>		/* Internet address structures */
#include <sys/socket.h>		/* socket interface functions  */
#include <netdb.h>		/* host to IP resolution       */
#include <string.h>
#include <netinet/tcp.h>        // TCP_NODELAY
#include <fcntl.h>

#include "NetBus.c"
#include "NetTime.c"
#include "NetPin.c"
#include "NetDIORaw.c"
#include "NetDIO.c"
#include "NetTWI.c"
#include "NetSPI.c"
#include "NetSPIctl.c"
#include "NetLock.c"
#include "NetCAN.c"
#include "PThreadLock.c"
#include "NetSystem.c"
#include "NetCANctl.c"
#include "arch2.c"
#include "QueueBytes.c" // for spictl()
#include "ServerTCP.c" // for spictl()

typedef struct log_conn {
  int socket;
  // flags
  int flags;
} log_conn;

#define MAX_LOG_CONN 4
log_conn LC[MAX_LOG_CONN];
int logconns,s,logflags;
pthread_mutex_t loglock = PTHREAD_MUTEX_INITIALIZER;

void log9(int flags,char *format, ...) {
  int i,j,len;
  va_list ap;
  char dummy[1];

  if ((logflags & flags) == 0) {
    return;
  }
  va_start(ap, format);
  //vfprintf(stderr,format,ap);
  if (logflags & LOG_TIMESTAMP) {
  }
  len = vsnprintf(dummy,0,format,ap);
  {
    char str[len];
    vsprintf(str,format,ap);

    pthread_mutex_lock(&loglock);
    for (i=0;i<logconns;i++) {
      if (flags & LC[i].flags) {
	if (LC[i].flags & LOG_TIME) {
	}
	if (write(LC[i].socket,str,len) <= 0) {
	  close(LC[i].socket);
	  for (j=i;j<logconns-1;j++) {
	    LC[j] = LC[j+1];
	  }
	  logconns--;
	  logflags = 0;
	  for (j=0;j<logconns;j++) {
	    logflags |= LC[j].flags;
	  }
	  i--; // so i++ won't skip next element
	}
      }
    }
    pthread_mutex_unlock(&loglock);
  }
  va_end(ap);
}

static inline int FileBlockingSet(int fd,int on) {
  return fcntl(fd, F_SETFL, on ? 0 : (FNDELAY | O_NONBLOCK));
}

#ifndef NOSIG
#define NOSIG(expression)			\
  (__extension__				\
   ({ long int __result;			\
     do __result = (long int) (expression);	\
     while (__result == -1L && errno == EINTR);	\
     __result; }))
#endif

int ServerSocketNew(int port) {
  int			rc;       /* system calls return value storage  */
  int			s;        /* socket descriptor                  */
  int			cs;       /* new connection's socket descriptor */
  struct sockaddr_in	sa;       /* Internet address struct            */
  int x = 1;

  memset(&sa, 0, sizeof(sa));
  sa.sin_family = AF_INET;
  sa.sin_port = htons(port);
  sa.sin_addr.s_addr = INADDR_ANY;
  
  s = socket(AF_INET, SOCK_STREAM, 0);
  if (s < 0) {
    //perror("socket: allocation failed");
    return -1;
  }
  setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &x, 4);
  setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &x, 4);
  if (FileBlockingSet(s,0) < 0) {
    printf("ERROR: Unable to set server socket non-blocking (%m)!\n");
    NOSIG(close(s));
    return -1;
  }
  rc = bind(s, (struct sockaddr *)&sa, sizeof(sa));
  if (rc) {
    //perror("bind");
    return -1;
  }
  rc = listen(s, 5);
  if (rc) {
    //perror("listen");
    return -1;
  }  
  return s;
}

void CANMessagePrint(CANMessage *msg) {
  struct tm * tm;
  int i;

  if (msg->flags & FLAG_CONTROL) printf("*** control packet\n");
  if (msg->flags & FLAG_BUS_ERROR) printf("*** bus error\n");
  if (msg->flags & FLAG_ERROR_PASSIVE) printf("*** error passive\n");
  if (msg->flags & FLAG_DATA_OVERRUN) printf("*** data overrun error\n");
  if (msg->flags & FLAG_ERROR_WARNING) printf("*** error warning\n");
  tm = localtime(&msg->timestamp.tv_sec);
  printf("%02d:%02d:%02d.%06d %c%c ", 
	 tm->tm_hour, tm->tm_min, tm->tm_sec,msg->timestamp.tv_usec, 
	 (msg->flags & FLAG_EXT_ID) ? 'E' : 'S', 
	 (msg->flags & FLAG_RTR)     ? 'R' : '.');
  if (msg->flags & FLAG_CONTROL) {
    printf("id=0x%08x ", msg->id);
    printf("cmd=%d ",msg->length);
    printf("dat=");
    for (i = 0; i < 8 ; i++) {
      printf("0x%02x", msg->data[i]);
      if (i != (msg->length - 1)) printf(":");
    }
  } else if (msg->length != 0) {
    printf("id=0x%08x ", msg->id);
    if (msg->length < 0 || msg->length > 8) {
      printf("bad length=%d\n",msg->length);
    } else {
      printf("dat=");
      for (i = 0; i < msg->length; i++) {
	printf("0x%02x", msg->data[i]);
	if (i != (msg->length - 1)) printf(":");
      }
    }
  }
  printf("\n");
}

/*
  Possible changes:
  1. Include status for ALL requests, which may condition whether or not
     additional bytes are provided.
  1b. Return a status for an invalid API access attempt
  2. We can't support an individual API entry changing, but we should
     support adding a new API.  Therefore we might also need some way of
     determining what the max API supported is.  We might return this if an
     invalid API access is attempted (thus logically API #255 would be reserved
     for determining this!)
 */
pthread_mutex_t busmutex[8];
pthread_mutex_t pinmutex;
pthread_mutex_t diorawmutex[8];
pthread_mutex_t diomutex[8];

Bus *busses[32];
int numBusses;
Time *times[8];
int numTimes;
Pin *pin;
DIORaw *dioraws[8];
int numDIORaw;
DIO *dios[8];
int numDIO;
TWI *twis[8];
int numTWI;
SPI *spis[8];
int numSPI;
CAN *cans[8];
int numCAN;

void ArchInit0() {
  int i,mid;

  ArchInit1();
  mid = TSModelGet();
  if (mid != ModelId) {
    fprintf(stderr,"This binary was compiled for ModelId %04X\n",ModelId);
    fprintf(stderr,"Error: Detected ModelId %04X\n",mid);
    exit(1);
  }
  for (i=0;i<32;i++) {
    busses[i] = BusInit1(i);
    if (!busses[i]) break;
    //pthread_mutex_init(&busmutex[i], NULL);
  }
  //pthread_mutex_init(&pinmutex, NULL);
  numBusses = i;
  for (i=0;i<8;i++) {
    times[i] = TimeInit1(i);
    if (!times[i]) break;
  }
  numTimes = i;
  pin = PinInit1(0);
  for (i=0;i<8;i++) {
    dioraws[i] = DIORawInit1(i);
    if (!dioraws[i]) break;
    //pthread_mutex_init(&diorawmutex[i], NULL);
  }
  numDIORaw = i;
  for (i=0;i<8;i++) {
    dios[i] = DIOInit1(i);
    if (!dios[i]) break;
    //pthread_mutex_init(&diomutex[i], NULL);
  }
  numDIO = i;
  for (i=0;i<8;i++) {
    twis[i] = TWIInit1(i);
    if (!twis[i]) break;
  }
  numTWI = i;
  for (i=0;i<8;i++) {
    spis[i] = SPIInit1(i);
    if (!spis[i]) break;
  }
  numSPI = i;
  for (i=0;i<8;i++) {
    cans[i] = CANInit1(i);
    if (!cans[i]) break;
  }
  numCAN = i;
}

// tries to send a CAN message until:
// 1. it gets sent, in which case it returns sizeof(CANMessage)
// 2. connection is closed, in which case it returns 0
int CANMsgTx(int socket,CANMessage *msg) {
  return PacketSend0(socket,msg,sizeof(CANMessage));
}

static inline int CANMsgRx(int socket,CANMessage *msg) {
  return PacketRecv(socket,msg,sizeof(CANMessage));
}

int efd;
#ifndef EPOLLRDHUP
// Note: WTF cross tools? Y U NO DEFINE THIS, 
// EVEN THOUGH Linux version > 2.6.17???
#define EPOLLRDHUP 0x2000
#endif

pthread_mutex_t watchlock = PTHREAD_MUTEX_INITIALIZER;
ThreadData *head_thread = 0;

extern const char build[];
extern const char archstr[];
extern const int compiletime;
int startuptime;
int BaseBoardIdGet();
int DIOMapLength(int *max);
char *DIOMapGet(int i,int *value);
int DIOMapLookup(char *string);
char *DIOMapLookupPartial(char *string,int value);
int DIOMapAdd(char *string,int value);
int DIOMapDelete(char *string);
extern pthread_mutex_t diomaplock;


int SPIInterpretCommand(int socket,unsigned char API,
			unsigned char instance) {
  Packet reply;
  struct ThreadData *th = pthread_getspecific(tdata);
  SPI *spi;
  int len,ret,adrs;

  if (instance >= numDIO) {
    return -1;
  }
  if (API >= SPI_APICount) return -1;
  spi = spis[instance];
  PacketInit(&reply);
  switch (API) {
  case NetSPI_Lock: {
    int command,ret;
    unsigned num;
    struct ThreadData *th = pthread_getspecific(tdata);
    struct epoll_event event;
    event.events = EPOLLRDHUP;
    event.data.ptr = pthread_getspecific(tdata);

    if (PacketPull32(socket,&command) != 4) return -1;
    if (PacketPull32(socket,&num) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);
    th->state = 1;
    PacketPush32(&reply,ret=spi->API->Lock(spi,command,num));
    log9(LOG_SPI,"%d SPI: Lock ret=%d\n",th->tid,ret);
    th->state = 0;
    if (epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0) ==-1) 
      log9(LOG_SYSTEM,"epoll_ctl:%m\n");
    PacketSend(&reply,socket);
    return 1;
  }
  case NetSPI_Write:
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&len) != 4) return -1;
    if (len <= 500) {
      char buf[len];
      if (PacketPullBytes(socket,len,buf) != len) return -1;
      PacketPush32(&reply,spi->API->Write(spi,adrs,buf,len));
    } else return -1;
    PacketSend(&reply,socket);
    return 1;
  case NetSPI_Read:
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&len) != 4) return -1;
    if (len <= 500) {
      unsigned char buf[len];
      PacketPush32(&reply,ret = spi->API->Read(spi,adrs,buf,len));
      if (ret > 0) {
	PacketPushBytes(&reply,len,buf);
      }
      PacketSend(&reply,socket);
      return 1;
    } else return -1;
  case NetSPI_ReadWrite:
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&len) != 4) return -1;
    if (len <= 500) {
      char wbuf[len],rbuf[len];
      if (PacketPullBytes(socket,len,wbuf) != len) return -1;
      PacketPush32(&reply,ret=spi->API->ReadWrite(spi,adrs,wbuf,rbuf,len));
      if (ret > 0) {
	PacketPushBytes(&reply,len,rbuf);
      }
      PacketSend(&reply,socket);
      return 1;      
    } else return -1;
  case NetSPI_ClockSet: {
    unsigned hz;

    if (PacketPull32(socket,&hz) != 4) return -1;
    PacketPush32(&reply,spi->API->ClockSet(spi,hz));
    PacketSend(&reply,socket);
    return 1;      
  }
  case NetSPI_EdgeSet: {
    unsigned posedge;

    if (PacketPull32(socket,&posedge) != 4) return -1;
    PacketPush32(&reply,spi->API->EdgeSet(spi,posedge));
    PacketSend(&reply,socket);
    return 1;      
  }
  }
}

void *LocalSystemInit(System *sys) {
  return sys;
}

void LocalSystemFini(System *sys) {
}

int LocalSystemClassCount(System *sys) {
  return NetSPIClass+1;
}

int LocalSystemInstanceCount(System *sys,int class) {
  switch (class) {
  case NetSystemClass:
    return 1;
  case NetBusClass:
    return numBusses;
  case NetTimeClass:
    return numTimes;
  case NetPinClass:
    return 1;
  case NetDIORawClass:
    return numDIORaw;
  case NetDIOClass:
    return numDIO;
  case NetTWIClass:
    return numTWI;
  case NetCANClass:
    return numCAN;
  case NetSPIClass:
    return numSPI;
  default:
    return 0;
  }
  
}

int LocalSystemAPICount(System *sys,int class) {
  switch (class) {                             
  case NetSystemClass:                     
    return System_APICount;
  case NetBusClass:                        
    return Bus_APICount;
  case NetTimeClass:                       
    return Time_APICount;
  case NetPinClass:                        
    return Pin_APICount;
  case NetDIORawClass:                     
    return DIORaw_APICount;
  case NetDIOClass:                        
    return DIO_APICount;
  case NetTWIClass:                        
    return TWI_APICount;
  case NetCANClass:                        
    return CAN_APICount;
  case NetSPIClass:                        
    return SPI_APICount;
  default:                                 
    return 0;
  }
}

int LocalSystemLockCount(System *sys) {
#ifndef NO_SERVER
  return globallock.numLocks;
#else
  return 0;
#endif
}

int LocalSystemLockHolderInfo(System *sys,LockHolderInfo *lhi) {
}

int LocalSystemConnWaitInfo(System *sys,int max,ConnectionWaitInfo *info) {
#ifndef NO_SERVER
  ThreadData *cur;
  int conns=0,n=0,i=max;

  pthread_mutex_lock(&watchlock);
  cur = head_thread;
  while (cur) {
    conns++;
    cur = cur->next;
  }
  if (conns > i) conns = i;
  cur = head_thread;
  while (cur && i--) {
    info[n].pid = cur->pid;
    info[n++].wait = cur->wait;
    cur = cur->next;
  }
  pthread_mutex_unlock(&watchlock);
  return conns;
#else
  return 0;
#endif
    
}

int LocalSystemCANBusGet(System *sys,int CANInst) {
  if (CANInst > numCAN) {
    return -1;
  } else {
    return CANBusNum[CANInst];
  }
}

void LocalSystemBuildInfo(System *sys,struct BuildInfo *buildinfo) {
}

int LocalSystemModelId(System *sys) {
  return ModelId;
}

int LocalSystemBaseBoardId(System *sys) {
  int bbid=0,ret;
  struct ThreadData *th = pthread_getspecific(tdata);
  struct epoll_event event;
  event.events = EPOLLRDHUP;
  event.data.ptr = pthread_getspecific(tdata);
  
  ret = epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event);
  if (ret == -1) fprintf(stderr,"epoll:%d:%m\n",efd); else fprintf(stderr,"ok:%d\n",efd);
  assert(ret!=-1);
  th->state = 1;
  bbid = BaseBoardIdGet();
  th->state = 0;
  assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);
  return bbid;
}

int LocalSystemMapLength(System *sys) {
  int i;
  pthread_mutex_lock(&diomaplock);
  i = DIOMapLength(0);
  pthread_mutex_unlock(&diomaplock);
  return i;
}

int LocalSystemMapGet(System *sys,int num,char *dest,int *value) {
  char *s;

  pthread_mutex_lock(&diomaplock);
  s = DIOMapGet(num,value);
  memcpy(dest,s,256);
  pthread_mutex_unlock(&diomaplock);
  return strlen(dest);
}

int LocalSystemMapLookup(System *sys,char *name) {
  int i;
  pthread_mutex_lock(&diomaplock);
  i = DIOMapLookup(name);
  pthread_mutex_unlock(&diomaplock);
  return i;
}

int LocalSystemMapLookupPartial(System *sys,char *name,int value,char *dest) {
  char *s;
  pthread_mutex_lock(&diomaplock);
  s = DIOMapLookupPartial(name,value);
  memcpy(dest,s,256);
  pthread_mutex_unlock(&diomaplock);
  return strlen(dest);
}

int LocalSystemMapAdd(System *sys,char *name,int value) {
  int i=0;
  pthread_mutex_lock(&diomaplock);
  if (DIOMapLookup(name) != -1) {
    DIOMapDelete(name);
    i++;
  }
  DIOMapAdd(name,value);
  i++;
  pthread_mutex_unlock(&diomaplock);
  return i;
}

int LocalSystemMapDelete(System *sys,char *name) {
  int i;
  pthread_mutex_lock(&diomaplock);
  i = DIOMapDelete(name);
  pthread_mutex_unlock(&diomaplock);
  return i;
}

SystemAPI LocalSystemAPI = {
  .Init = (void *)LocalSystemInit,
  .Fini = (void *)LocalSystemFini,
  .ClassCount = (void *)LocalSystemClassCount,
  .InstanceCount = (void *)LocalSystemInstanceCount,
  .APICount = (void *)LocalSystemAPICount,
  .LockCount = (void *)LocalSystemLockCount,
  .LockHolderInfo = (void *)LocalSystemLockHolderInfo,
  .ConnWaitInfo = (void *)LocalSystemConnWaitInfo,
  .CANBusGet = (void *)LocalSystemCANBusGet,
  .BuildInfo = (void *)LocalSystemBuildInfo,
  .ModelId = (void *)LocalSystemModelId,
  .BaseBoardId = (void *)LocalSystemBaseBoardId,
  .MapLength = (void *)LocalSystemMapLength,
  .MapGet = (void *)LocalSystemMapGet,
  .MapLookup = (void *)LocalSystemMapLookup,
  .MapLookupPartial = (void *)LocalSystemMapLookupPartial,
  .MapAdd = (void *)LocalSystemMapAdd,
  .MapDelete = (void *)LocalSystemMapDelete
};
System LocalSystem = {
  .API = &LocalSystemAPI,
  .InitStatus = 1
};

int SystemInterpretCommand(int socket,unsigned char API,
			   unsigned char instance) {
  Packet reply;
  int i,j;
  unsigned char tmp;

  log9(LOG_SYSTEM,"SystemInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance > 0) return -1;
  if (API >= System_APICount) return -1;
  PacketInit(&reply);
  switch (API) {
  case NetSystem_MapLength:
    PacketPush32(&reply,LocalSystemMapLength(&LocalSystem));
    PacketSend(&reply,socket);
    break;
  case NetSystem_MapGet: {
    char name[256];
    int value,len;
    if (PacketPull32(socket,&i) != 4) return -1;
    len = LocalSystemMapGet(&LocalSystem,i,name,&value);
    PacketPush32(&reply,value);
    PacketPush8(&reply,len);
    PacketPushBytes(&reply,len,name);
    PacketSend(&reply,socket);    
  }
  case NetSystem_MapLookup:
    if (PacketPull8(socket,&tmp) != 1) return -1;
    i = tmp;
    {
      char name[256];
      if (PacketPullBytes(socket,i,name) != i) return -1;
      name[i] = 0;
      j=LocalSystemMapLookup(&LocalSystem,name);
      PacketPush32(&reply,j);
      PacketSend(&reply,socket);
    }
    break;
  case NetSystem_MapLookupPartial: {
    int value;
    char *str;
    if (PacketPull32(socket,&value) != 4) return -1;
    if (PacketPull8(socket,&tmp) != 1) return -1;
    i = tmp;
    {
      char *s,str[256];
      if (PacketPullBytes(socket,i,str) != i) return -1;
      str[i] = 0;
      tmp = LocalSystemMapLookupPartial(&LocalSystem,str,value,str); 
      PacketPush8(&reply,tmp);
      PacketPushBytes(&reply,tmp,str);
      PacketSend(&reply,socket);
    }
  }
  case NetSystem_MapAdd: {
    int value;
    if (PacketPull32(socket,&value) != 4) return -1;
    if (PacketPull8(socket,&tmp) != 1) return -1;
    i = tmp;
    {
      char name[256];
      if (PacketPullBytes(socket,i,name) != i) return -1;
      name[i] = 0;
      i = LocalSystemMapAdd(&LocalSystem,name,value);
      PacketPush32(&reply,i);
      PacketSend(&reply,socket);
    }
    break;
  }
  case NetSystem_MapDelete: {
    if (PacketPull8(socket,&tmp) != 1) return -1;
    i = tmp;
    {
      char name[256];
      if (PacketPullBytes(socket,i,name) != i) return -1;
      name[i] = 0;
      i = LocalSystemMapDelete(&LocalSystem,name);
      PacketPush32(&reply,i);
      PacketSend(&reply,socket);
    }
    break;
  }
  case NetSystem_ClassCount:
    PacketPush32(&reply,LocalSystemClassCount(&LocalSystem));
    PacketSend(&reply,socket);
    return 1;
  case NetSystem_InstanceCount:
    if (PacketPull32(socket,&i) != 4) return -1;
    PacketPush32(&reply,LocalSystemInstanceCount(&LocalSystem,i));
    PacketSend(&reply,socket);
    return 1;
  case NetSystem_APICount:
    if (PacketPull32(socket,&i) != 4) return -1;
    PacketPush32(&reply,LocalSystemAPICount(&LocalSystem,i));
    PacketSend(&reply,socket);
    return 1;
  case NetSystem_LockCount:
    PacketPush32(&reply,LocalSystemLockCount(&LocalSystem));
    PacketSend(&reply,socket);
    return 1;
  case NetSystem_LockHolderInfo: {
    int i,n=0;
#ifndef NO_SERVER
    PacketPush32(&reply,globallock.numLocks);
    for (i=0;i<globallock.numLocks;i++) {
      if (globallock.info[i].thread) {
	PacketPush32(&reply,globallock.info[i].thread->pid);
      } else {
	PacketPush32(&reply,-1);
      }
      PacketPush32(&reply,globallock.info[i].count);
      n+= 8;
      if (n > 500) {
	PacketSend(&reply,socket);
	PacketInit(&reply);
	n=0;
      }
    }
    PacketSend(&reply,socket);
#endif
    break;
  }
  case NetSystem_ConnWaitInfo: {
#ifndef NO_SERVER
    ThreadData *cur;
    int conns=0,n=0,i;

    if (PacketPull32(socket,&i) != 4) return -1;
    pthread_mutex_lock(&watchlock);
    cur = head_thread;
    while (cur) {
      conns++;
      cur = cur->next;
    }
    if (conns > i) conns = i;
    PacketPush32(&reply,conns);
    cur = head_thread;
    while (cur && i--) {
      PacketPush32(&reply,cur->pid);
      PacketPush32(&reply,cur->wait);
      n+= 8;
      if (n > 500) {
	PacketSend(&reply,socket);
	PacketInit(&reply);
	n=0;
      }
      cur = cur->next;
    }
    PacketSend(&reply,socket);
    pthread_mutex_unlock(&watchlock);
#endif
    break;
  }
  case NetSystem_CANBusGet:
    if (PacketPull32(socket,&i) != 4) return -1;
    PacketPush32(&reply,LocalSystemCANBusGet(&LocalSystem,i));
    PacketSend(&reply,socket);
    break;
  case NetSystem_BuildInfo: {
    char *str;
    unsigned buildNumber = strtoul(build+7,&str,10);

    PacketPush32(&reply,buildNumber);
    PacketPush32(&reply,compiletime);
    PacketPush32(&reply,time(0)-startuptime);
    PacketPush8(&reply,strlen(str+1));
    PacketPushBytes(&reply,strlen(str+1),str+1);
    PacketPush8(&reply,strlen(archstr+1));
    PacketPushBytes(&reply,strlen(archstr+1),(unsigned char *)archstr+1);
    PacketSend(&reply,socket);
    break;
  }
  case NetSystem_ModelId:
    PacketPush32(&reply,ModelId);
    PacketSend(&reply,socket);
    break;
  case NetSystem_BaseBoardId: {
    PacketPush32(&reply,LocalSystemBaseBoardId(&LocalSystem));
    PacketSend(&reply,socket);
    break;
  }
  }
  return 1;
}

int CANInterpretCommand(int socket,unsigned char API,unsigned char instance,
			int *can) {
  Packet reply;
  CANMessage msg;
  int ret;
  unsigned baud;
  struct ThreadData *th = pthread_getspecific(tdata);
  struct epoll_event event;
  event.events = EPOLLRDHUP;
  event.data.ptr = pthread_getspecific(tdata);

  log9(LOG_CAN,"CANInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance >= numCAN) {
    return -1;
  }
  if (API >= CAN_APICount) {
    return -1;
  }
  PacketInit(&reply);
  if (can[instance] == -1) {
    can[instance] = ClientSocketNew("127.0.0.1",7552+instance);
    log9(LOG_CAN,"local can%d=%d\n",instance,can[instance]);
    //fprintf(stderr,"CANInterpretCommand new socket %d\n",can[instance]);
    fcntl(can[instance],F_SETFL,fcntl(can[instance],F_GETFL,0)&~O_NONBLOCK);
  }
  switch (API) {
  case NetCAN_Rx:
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);
    th->state = 1;
    ret = CANMsgRx(can[instance],&msg);
    if (ret == 0) {
      close(can[instance]);
      can[instance] = ClientSocketNew("127.0.0.1",7552+instance);
      ret = CANMsgRx(can[instance],&msg);
    }
    log9(LOG_CAN,"CANMsgRx ret=%d\n",ret);
    if (ret == 0) {
      close(can[instance]);
      can[instance] = -1;
    }
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);
    if (ret < 0) can[instance] = -1;
    PacketPush32(&reply,(ret <= 0)?0:ret);
    if (ret > 0) {
      PacketPushBytes(&reply,sizeof(CANMessage),(char *)&msg);
    }
    PacketSend(&reply,socket);
    return 1;
  case NetCAN_Tx:
    if (CANMsgRx(socket,&msg) == 0) return -1;
      //if (PacketPullBytes(socket,sizeof(CANMessage),(char *)&msg) 
      //!= sizeof(CANMessage)) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);
    th->state = 1;
    ret = CANMsgTx(can[instance],&msg);
    if (ret == 0) {
      close(can[instance]);
      can[instance] = ClientSocketNew("127.0.0.1",7552+instance);
      ret = CANMsgTx(can[instance],&msg);
    }
    log9(LOG_CAN,"CANMsgTx ret=%d\n",ret);
    if (ret == 0) {
      close(can[instance]);
      can[instance] = -1;
    }
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);
    PacketPush32(&reply,ret);
    PacketSend(&reply,socket);
    //fprintf(stderr,"Tx done\n");
    return 1;
  case NetCAN_BaudSet:
    //fprintf(stderr,"baudset=");
    if (PacketPull32(socket,&baud) != 4) return -1;
    //fprintf(stderr,"%d\n",baud);
    msg.flags = FLAG_CONTROL;
    msg.id = 0;
    msg.length = 1;
    *((unsigned *)msg.data) = baud;

    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);
    //fprintf(stderr,"debug1\n");
    th->state = 1;
    ret = CANMsgTx(can[instance],&msg);
    //fprintf(stderr,"debug2:%d\n",ret);
    if (ret == 0) {
    } else { // drain off receiver until socket closes
      while ((ret=CANMsgRx(can[instance],&msg)) > 0);
      //fprintf(stderr,"%d",ret);
      //fprintf(stderr,"%d--\n",ret);
    }
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);
    close(can[instance]);
    can[instance] = -1;    
    PacketPush32(&reply,ret=cans[instance]->API->BaudGet(cans[instance]));
    //fprintf(stderr,"baud=%d\n",ret);
    PacketSend(&reply,socket);
    return 1;
  case NetCAN_BaudGet:
    PacketPush32(&reply,cans[instance]->API->BaudGet(cans[instance]));
    PacketSend(&reply,socket);
    return 1;
  }
}

int TWIInterpretCommand(int socket,unsigned char API,unsigned char instance) {
  Packet reply;
  TWI *twi;

  log9(LOG_TWI,"TWIInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance >= numTWI) {
    return -1;
  }
  if (API >= TWI_APICount) {
    return -1;
  }
  twi = twis[instance];
  PacketInit(&reply);
  //pthread_mutex_lock(&diomutex[instance]);
  switch (API) {
  case NetTWI_Lock: {
    int command,ret;
    unsigned num;
    struct ThreadData *th = pthread_getspecific(tdata);
    struct epoll_event event;
    event.events = EPOLLRDHUP;
    event.data.ptr = pthread_getspecific(tdata);

    if (PacketPull32(socket,&command) != 4) return -1;
    if (PacketPull32(socket,&num) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);
    th->state = 1;
    PacketPush32(&reply,ret=twi->API->Lock(twi,command,num));
    log9(LOG_TWI,"%d TWI: Lock ret=%d\n",th->tid,ret);
    th->state = 0;
    if (epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0) ==-1) 
      log9(LOG_SYSTEM,"epoll_ctl:%m\n");
    PacketSend(&reply,socket);
    return 1;
  }
  case NetTWI_Read: {
    int devadr,adrslen,adrs,len,ret;
    if (PacketPull32(socket,&devadr) != 4) return -1;
    if (PacketPull32(socket,&adrslen) != 4) return -1;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&len) != 4) return -1;
    if (len > 0) {
      unsigned char buf[len];
      ret = twi->API->Read(twi,devadr,adrslen,adrs,len,buf);
      log9(LOG_TWI,"TWI: Read ret=%d\n",ret);
      PacketPush32(&reply,ret);
      if (ret > 0) {
	PacketPushBytes(&reply,len,buf);
      }
      PacketSend(&reply,socket);
    }
    return 1;
  }
  case NetTWI_Write: {
    int devadr,adrslen,adrs,len;
    if (PacketPull32(socket,&devadr) != 4) return -1;
    if (PacketPull32(socket,&adrslen) != 4) return -1;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&len) != 4) return -1;
    if (len > 0) {
      unsigned char buf[len];
      if (PacketPullBytes(socket,len,buf) != len) return -1;
      PacketPush32(&reply,twi->API->Write(twi,devadr,adrslen,adrs,len,buf));
      PacketSend(&reply,socket);
      return 1;
    }
  }
  }
}

//==========================================================================
// dioctl support
//==========================================================================
#define DIOMapMax 1024
struct DIOMapInfo {
  char map[256];
} info[1024];
int DIOMapEntries = 0;
pthread_mutex_t diomaplock = PTHREAD_MUTEX_INITIALIZER;

int DIOMapLength(int *max) {
  int i,n=0;

  for (i=0;i<DIOMapEntries;i++) {
    if (info->map[256*i+1]) n++;
  }
  if (max) *max = DIOMapEntries;
  return n;
}

char *DIOMapGet(int i,int *value) {
  if (!info->map[256*i+1]) {
    return 0;
  }
  *value = (int)info->map[256*i];
  return &info->map[256*i+1];
}

int DIOMapLookup(char *string) {
  int i;

  for (i=0;i<DIOMapEntries;i++) {
    if (!strcmp(&info->map[256*i+1],string)) {
      return info->map[256*i];
    }
  }
  return -1;
}

char *DIOMapLookupPartial(char *string,int value) {
  int i;
  for (i=0;i<DIOMapEntries;i++) {
    if (!strncmp(&info->map[256*i+1],string,strlen(string)) && info->map[256*i]==(char)value) {
      return &info->map[256*i+1];
    }
  }
  return "";
}

static int compar(const void *a, const void *b) {
  const char *a1 = a, *b1 = b;
  return strncmp(a1+1,b1+1,255);
}

int DIOMapAdd(char *string,int value) {
  int i;
  if (DIOMapEntries == DIOMapMax) {
    return -1;
  }
  for (i=0;i<DIOMapEntries && info->map[256*i+1];i++);
  strncpy(&info->map[256*i+1],string,255);
  info->map[256*i] = value;
  if (i >= DIOMapEntries) {
    DIOMapEntries++;
  }
  qsort(&info,DIOMapEntries,256,compar);
  return DIOMapEntries;
}

int DIOMapDelete(char *string) {
  int i = DIOMapLookup(string);
  for (i=0;i<DIOMapEntries;i++) {
    if (!strcmp(&info->map[256*i+1],string)) {
      break;
    }
  }
  if (i == DIOMapEntries) {
    return -1;
  }
  info->map[256*i+1] = 0;
  return 1;
}

void DIOMapDump() {
  int i;
  for (i=0;i<DIOMapEntries;i++) {
    fprintf(stderr,"%s=%d\n",&info->map[256*i+1],info->map[256*i]);
  }
}

// parse str according to "name=value" and replace in map
// if value is a string, interpret as "name.value=1"
// returns 1 on success, or 0 on failure
static int ConfigLineAssign(char *str) {
  char *str0 = str;
  char *name,*cname;
  char tmp[256];
  unsigned value;
  int connectorNum,connectorPin;

  while (*str && *str != '=') str++;
  if (*str != '=') return 0;
  *str = 0;
  str++;
  value = strtoul(str,0,0);
  name = str0;
  if (value == 0 && str[0] != '0') {
    str[-1] = '.';
    value = 1;
  } else {
  }
  // automatic connector to connector name assignment
  if (2==sscanf(name,"attrib.Connector.%d.%d",
		&connectorNum,&connectorPin)) {
    cname = DIOMapLookupPartial("attrib.Connector.Name.",connectorNum);
    if (cname[0] != 0) {
      sprintf(tmp,"%s%d",cname+22,connectorPin);
      //fprintf(stderr,"%s=%d\n",tmp,value);
      DIOMapAdd(tmp,value);
    }
  }
  DIOMapAdd(name,value);
  return 1;
}

int ConfigLoadFile(char *filename) {
  char buf[300],*str,*inst_str,*wc;
  int lines=0,n;
  FILE *f;

  f = fopen(filename,"r");
  if (!f) return 0;
  while (!feof(f)) {
    fgets(buf,300,f);
    if (buf[0] == 0) goto ConfigFileProcessEnd;
    str = buf;
    while (*str) str++;
    str--;
    while(str >= buf && (*str == '\n' || *str == '\r' || *str == ' ' || *str == '\t')) {
      *str = 0;
      str--;
    }
    lines++;
    if (buf[0] != '#') {
      ConfigLineAssign(buf);
    }
  }
 ConfigFileProcessEnd:
  fclose(f);
  return 1;  // or length of map array?
}

enum {
  dioctlDIOCapabilitiesGet=0,
  dioctlDIOSetInput=1,
  dioctlDIOSetHigh=2,
  dioctlDIOSetLow=3,
  dioctlDIOGet=4,
  dioctlDIOGroupDefine=5,
  dioctlDIOGroupSetInput=6,
  dioctlDIOGroupSetHigh=7,
  dioctlDIOGroupSetLow=8,
  dioctlDIOGroupSet=9,
  dioctlDIOGroupGet=10,
  dioctlDIOGroupLockBlocking=11,
  dioctlDIOGroupLockRelease=12,
  dioctlDIOGroupLockNonBlocking=13,
  dioctlDIOGroupWaitOld=14,
  dioctlDIOGroupWait=114,
  dioctlDIOWaitLow=15,
  dioctlDIOWaitHigh=16,
  dioctlDIOLockBlocking=17,
  dioctlDIOLockRelease=18,
  dioctlDIOLockNonBlocking=19,
  dioctlDIOGroupWaitLow=20,
  dioctlDIOGroupWaitHigh=21,
  dioctlBoardModelGet=22,
  dioctlDIONameLookup=23,
  dioctlDIOLookup=27,
  dioctlDIONameAssign=24,
  dioctlDIONameRemove=25,
  dioctlDIONamesList=26,
  dioctlDIOTimeEventGet=28,
  dioctlDIOTimeCurrentGet=29,
  dioctlDIOTimeArm=30,
  dioctlDIOSetPin=32,
  dioctlDIOGetPin=33,
  dioctlDIOGroupCommit=34,
  dioctlDIOGroupRefresh=35,
  dioctlDIOSleep=200
};

static inline int DIOStateUnmake(DIOState state) {
  switch (state) {
  case INPUT_HIGH: return 0x40;
  case INPUT_LOW: return 0x00;
  case HIGH: return 0xC0;
  case LOW: return 0x80;
  }
}

static inline DIOState DIOStateMake(int snip) {
  switch ((snip >> 6)&3) {
  case 0: return INPUT;
  case 1: return INPUT;
  case 2: return LOW;
  case 3: return HIGH;
  }
}

typedef struct dioctlConn {
  int DIOGroup[16][16]; // -1 = no DIO
  unsigned char dio_group_count[256]; // number of groups each DIO is in
  int arm;
  struct timeval EventLast;
} dioctlConn;

static void watchlockremove(ThreadData *th);
int OptPollRate=10;

void zprintf(int socket,char *format, ...) {
  va_list ap;
  char dummy[1];
  int len;

  va_start(ap,format);
  len = vsnprintf(dummy,0,format,ap);
  {
    char str[len];
    vsprintf(str,format,ap);
    write(socket,str,len);
  }
}

void httpCommand(int socket,char *cmd);

void *httpServer(void *arg) {
  int socket = (int)arg;
  int last,next,cmdcount=0;
  FILE *sf;
  char *cstr,cmd[1024],tmp[256],*s;
  time_t t;
  THREAD_SPECIFIC_DATA_INIT;
  thread_data.socket = socket;

  fcntl(socket,F_SETFL,fcntl(socket,F_GETFL,0)&~O_NONBLOCK);

  thread_data.pid = getpid();
  pthread_setspecific(tdata,&thread_data);
  pthread_mutex_lock(&watchlock);
  thread_data.next = head_thread;
  head_thread = &thread_data;
  pthread_mutex_unlock(&watchlock);

  struct epoll_event event;                    
  event.events = EPOLLRDHUP;                   
  event.data.ptr = pthread_getspecific(tdata); 

  sf = fdopen(socket,"w+");
  if (sf == 0) goto httpServerDone;

  // Assume POST, as that is all we support
  // ignore headers, because the format of our data and return is fixed
  // look for POST data, which follows headers.
  do {
    last = fgetc(sf);
  } while (!feof(sf) && last != '\r');
  while (!feof(sf)) {
    next = fgetc(sf);
    if (next == '\r') continue;
    if (last == '\n' && next == '\n') break;
    last = next;
  }
  t = time(0);
  sprintf(cmd,"HTTP/1.1 200 OK\r\nExpires: Sun, 1 Apr 2012 00:00:00 GMT\r\nServer: tsctl %s%s\r\nAccess-Control-Allow-Origin: *\r\nContent-Type: text/plain; charset=UTF-8\r\nConnection: close\r\n\r\n",build,archstr);
  //sprintf(cmd,"HTTP/1.1 200 OK\r\nExpires: Sun, 1 Apr 2012 00:00:00 GMT\r\nServer: tsctl %s%s\r\nAccess-Control-Allow-Origin: *\r\nContent-Type: text/plain; charset=UTF-8\r\n",build,archstr);
  //sprintf(cmd,"HTTP/1.1 200 OK\r\nExpires: Sun, 1 Apr 2012 00:00:00 GMT\r\nServer: tsctl %s %s\r\nAccess-Control-Allow-Origin: *\r\nContent-Type: multipart/x-mixed-replace; boundary=\"1234567890\"\r\n\r\n",build,archstr);
  write(socket,cmd,strlen(cmd));
  write(socket,"[",1);
  while (!feof(sf)) {
    /*
    fprintf(stderr,"epoll adding %d\n",socket);
    if(epoll_ctl(efd,EPOLL_CTL_ADD,socket,&event)==-1) {
      fprintf(stderr,"epoll1: %m: %d\n",socket);
      //sleep(86400);
    }
    thread_data.state = 1;
    */
    fgets(cmd,1023,sf);
    /*
    thread_data.state = 0;
    fprintf(stderr,"epoll removing %d\n",socket);
    if (epoll_ctl(efd,EPOLL_CTL_DEL,socket,0)==-1) {
      fprintf(stderr,"epoll2: %m: %d\n",socket);
      //sleep(86400);
    }
    */
    cstr = cmd;
    while (*cstr == '\r') cstr++;
    if (*cstr == 0) continue;
    // now cstr points to the next command
    //sprintf(tmp,"Content-Length: %d\r\n\r\n",strlen("Got command:")+strlen(cstr));
    //sprintf(tmp,"--1234567890\r\nContent-Type: text/plain; charset=UTF-8\r\nContent-Length: %d\r\n\r\n",strlen("Got command:")+strlen(cstr));
    if (!strcmp(cstr,"end\n")) break;
    if (cmdcount++ > 0) write(socket,",",1);
    s = cstr;
    while (*s) s++; s--;
    while ((*s == '\n' || *s == '\r') && s > cstr) s--;
    s[1] = 0;
    httpCommand(socket,cstr);

    //write(socket,tmp,strlen(tmp));
    //write(socket,"Got command:",strlen("Got command:"));
    //write(socket,cstr,strlen(cstr));
  }
  write(socket,"]",1);
  close(socket);

  // now the next character will either be the first character of
  // the parameters, or a '\r' which precedes it
  // the format of our parameters is identical to the command-line format
  // with each command on a separate line.
  // so, parse out a single command
  // 
  // TO DO: factor out command line processing
  // we should be able to use the same code here.
  // the main difference is that here we will be operating in server mode
  // so we will be calling objects directly.
  // this seems to imply that we must support locking, which the command
  // line arguments do not.
  // in addition, our output must be JSON, not what the command line
  // client returns.
 httpServerDone:
  fclose(sf);
  pthread_exit(0);
  return 0;

}


void *dioctlServer(void *arg) {
  int socket = (int)arg,temp,temp2,GroupNum,DIOCount,i,j;
  DIO *dio = dios[0];
  dioctlConn conn;
  unsigned char cmd,DIONum,tmp;
  Packet reply;
  THREAD_SPECIFIC_DATA_INIT;

  thread_data.pid = getpid();
  thread_data.socket = socket;
  pthread_setspecific(tdata,&thread_data);
  pthread_mutex_lock(&watchlock);
  thread_data.next = head_thread;
  head_thread = &thread_data;
  pthread_mutex_unlock(&watchlock);

  for (i=0;i<16;i++) {
    for (temp=0;temp<16;temp++) {
      conn.DIOGroup[i][temp] = -1;
    }
  }
  for (i=0;i<256;i++) {
    conn.dio_group_count[i] = 0;
  }

  while (1) {
    if (PacketPull8(socket,&cmd) != 1) goto dioctlServerDone;
    log9(LOG_DIOCTL,"dioctl cmd %d\n",cmd);
    switch (cmd) {
    case dioctlDIOCapabilitiesGet:
      // don't have a way to expose this
      // it's already cached in the DIO objects though
      break;
    case dioctlDIOSetInput:
      if (PacketPull8(socket,&DIONum) != 1) goto dioctlServerDone;
      dio->API->Lock(dio,WaitLock,DIONum);
      dio->API->SetAsync(dio,DIONum,INPUT);
      dio->API->Lock(dio,DoRelease,DIONum);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOSetHigh:
      if (PacketPull8(socket,&DIONum) != 1) goto dioctlServerDone;
      dio->API->Lock(dio,WaitLock,DIONum);
      dio->API->SetAsync(dio,DIONum,HIGH);
      dio->API->Lock(dio,DoRelease,DIONum);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOSetLow:
      if (PacketPull8(socket,&DIONum) != 1) goto dioctlServerDone;
      dio->API->Lock(dio,WaitLock,DIONum);
      dio->API->SetAsync(dio,DIONum,LOW);
      dio->API->Lock(dio,DoRelease,DIONum);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOGet:
      if (PacketPull8(socket,&DIONum) != 1) goto dioctlServerDone;
      dio->API->Lock(dio,WaitLock,DIONum);
      temp = dio->API->GetAsync(dio,DIONum);
      dio->API->Lock(dio,DoRelease,DIONum);
      temp2 = DIOStateUnmake(temp);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      PacketInit(&reply);
      PacketPush8(&reply,temp2);
      PacketSend(&reply,socket);
      break;
    case dioctlDIOGroupDefine:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp = tmp;
      GroupNum = temp >> 4;
      DIOCount = 1+(temp & 0xF);
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0; i++) {
	conn.dio_group_count[conn.DIOGroup[GroupNum][i]]--;
      }
      for (i=0;i<DIOCount;i++) {
	if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
	conn.DIOGroup[GroupNum][i] = tmp;
	conn.dio_group_count[tmp]++;
      }
      for (;i<16;i++) {
	conn.DIOGroup[GroupNum][i] = -1;
      }
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOGroupSetInput:
      temp2 = INPUT;
      goto dioctlDIOGroupSetAll;
    case dioctlDIOGroupSetHigh:
      temp2 = HIGH;
      goto dioctlDIOGroupSetAll;
    case dioctlDIOGroupSetLow:
      temp2 = LOW;
    dioctlDIOGroupSetAll:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp = tmp;
      GroupNum = temp >> 4;
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,WaitLock,conn.DIOGroup[GroupNum][i]);
      }
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Set(dio,conn.DIOGroup[GroupNum][i],temp2);
      }
      dio->API->Commit(dio,0);
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,DoRelease,conn.DIOGroup[GroupNum][i]);
      }
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;

    case dioctlDIOSetPin:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp2 = (char)tmp;
      dio->API->Lock(dio,WaitLock,DIONum);
      if (!conn.dio_group_count[DIONum]) {
	//fprintf(stderr,"[%d]:=%d\n",DIONum,temp2);
	dio->API->SetAsync(dio,DIONum,temp2);
      } else {
	//fprintf(stderr,"[%d]:=%d.\n",DIONum,temp2);
	dio->API->Set(dio,DIONum,temp2);
      }
      dio->API->Lock(dio,DoRelease,DIONum);
      break;
    case dioctlDIOGetPin:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      PacketInit(&reply);
      dio->API->Lock(dio,WaitLock,DIONum);
      if (!conn.dio_group_count[DIONum]) {
	PacketPush8(&reply,tmp=dio->API->GetAsync(dio,DIONum));
	//fprintf(stderr,"[%d]=%d\n",DIONum,tmp);
      } else {
	PacketPush8(&reply,tmp=dio->API->Get(dio,DIONum));
	//fprintf(stderr,"[%d]=%d.\n",DIONum,tmp);
      }
      dio->API->Lock(dio,DoRelease,DIONum);
      PacketSend(&reply,socket);
      break;
    case dioctlDIOGroupCommit:
      dio->API->Commit(dio,0);      
      break;
    case dioctlDIOGroupRefresh:
      dio->API->Refresh(dio);      
      break;
    case dioctlDIOGroupSet:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,WaitLock,conn.DIOGroup[GroupNum][i]);
      }
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i+=4) {
	if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
        dio->API->Set(dio,conn.DIOGroup[GroupNum][i],DIOStateMake(tmp));
	dio->API->Set(dio,conn.DIOGroup[GroupNum][i+1],DIOStateMake(tmp<<2));
	dio->API->Set(dio,conn.DIOGroup[GroupNum][i+2],DIOStateMake(tmp<<4));
	dio->API->Set(dio,conn.DIOGroup[GroupNum][i+3],DIOStateMake(tmp<<6));
      }
      dio->API->Commit(dio,0);
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,DoRelease,conn.DIOGroup[GroupNum][i]);
      }
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOGroupGet:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      PacketInit(&reply);
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++);
      PacketPush8(&reply,i);
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,WaitLock,conn.DIOGroup[GroupNum][i]);
      }
      dio->API->Refresh(dio);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i+=4) {
	temp = dio->API->Get(dio,conn.DIOGroup[GroupNum][i]);
	temp2 = DIOStateUnmake(temp);
	temp = dio->API->Get(dio,conn.DIOGroup[GroupNum][i+1]);
	temp2 += (DIOStateUnmake(temp) >> 2) & 0x30;
	temp = dio->API->Get(dio,conn.DIOGroup[GroupNum][i+2]);
	temp2 += (DIOStateUnmake(temp) >> 4) & 0x0C;
	temp = dio->API->Get(dio,conn.DIOGroup[GroupNum][i+3]);
	temp2 += (DIOStateUnmake(temp) >> 6) & 0x03;
	PacketPush8(&reply,temp2);
      }
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,DoRelease,conn.DIOGroup[GroupNum][i]);
      }
      PacketSend(&reply,socket);
      break;
    case dioctlDIOGroupLockBlocking:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,WaitLock,conn.DIOGroup[GroupNum][i]);
      }
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOGroupLockRelease:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	dio->API->Lock(dio,DoRelease,conn.DIOGroup[GroupNum][i]);
      }
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOGroupLockNonBlocking:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      temp = 1;
      for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i++) {
	if (dio->API->Lock(dio,TryLock,conn.DIOGroup[GroupNum][i]) < 0) {
	  temp = 0;
	  while (i > 0) {
	    dio->API->Lock(dio,DoRelease,conn.DIOGroup[GroupNum][i--]);
	  }
	}
      }
      PacketInit(&reply);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      //
      if (temp) {
	PacketPush8(&reply,0x77);
      } else {
	PacketPush8(&reply,0x5A);
      }
      PacketSend(&reply,socket);
      break;
    case dioctlDIOGroupWaitOld:
      // not used by dioctl API, but it *is* used by dioctl -w !
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      {
	int match,value[16],actual[16];
	for (i=0;i<16 && conn.DIOGroup[GroupNum][i] >= 0;i+=8) {
	  if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
	  for (j=0;j<8;j++) {
	    value[i+j] = ((tmp >> (8-j-1)) & 1) ? HIGH : LOW;
	  }
	}
	PacketInit(&reply);
	do {
	  match = 0;
	  dio->API->Refresh(dio);
	  for (i=0;i<16;i++) {
	    if (value[i] 
		== DIOValue(dio->API->Get(dio,conn.DIOGroup[GroupNum][i]))) {
	      actual[match++] = i;
	    }
	  }
	  if (match == 0 && OptPollRate > 0) {
	    times[0]->API->Delay(times[0],1000*OptPollRate);
	  }
	} while (match == 0 && sched_yield() != -1);

	PacketPush8(&reply,match);
	for (i=0;i<match;i++) {
	  PacketPush8(&reply,actual[i]);
	}
	PacketSend(&reply,socket);
      }
      break;
    case dioctlDIOGroupWait:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      GroupNum = tmp >> 4;
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp=tmp;
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp2=tmp;
      {
	int actual[16],value[16],mask[16],min,max,match=-1;
	
	for (i=0;i<8;i++) {
	  value[i] = (temp & 0x80) ? HIGH : LOW;
	  temp <<= 1;
	}
	for (;i<16;i++) {
	  value[i] = (temp2 & 0x80) ? HIGH : LOW;
	  temp2 <<= 1;
	}
	if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
	temp = tmp;
	if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
	temp2 = tmp;
	
	for (i=0;i<8;i++) {
	  mask[i] = (temp & 0x80) ? HIGH : LOW;
	  temp <<= 1;
	}
	for (;i<16;i++) {
	  mask[i] = (temp2 & 0x80) ? HIGH : LOW;
	  temp2 <<= 1;
	}
	if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
	temp = tmp;
	min = (temp >> 4) & 0xF;
	max = (temp & 0xF);
	while (match < min || match > max) {
	  match=0;
	  dio->API->Refresh(dio);
	  for (i=0;i<16;i++) {
	    actual[i]=DIOValue(dio->API->Get(dio,conn.DIOGroup[GroupNum][i]));
	    if (mask[i] && actual[i] == value[i]) {
	      match++;
	    }
	  }
	  if (match < min || match > max) sched_yield();
	}
	if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
	temp = temp2 = 0;
	for (i=0;i<8;i++) {
	  temp <<= 1;
	  temp |= actual[i];
	}
	for (;i<16;i++) {
	  temp2 <<= 1;
	  temp2 |= actual[i];
	}
	PacketInit(&reply);
	PacketPush8(&reply,temp2);
	PacketPush8(&reply,temp);
	PacketSend(&reply,socket);
      }
      break;
    case dioctlDIOWaitLow: {
      unsigned loops = 0;
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      while (1) {
	loops++;
	temp = dio->API->GetAsync(dio,DIONum);
	if (temp == INPUT_LOW || temp == LOW) {
	  break;
	} else {
	  if (OptPollRate > 0) {
	    times[0]->API->Delay(times[0],1000*OptPollRate);
	  }
	  sched_yield();
	}
      }
      //if (loops > 1) fprintf(stderr,"loops0=%d\n",loops);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      PacketInit(&reply);
      PacketPush8(&reply,DIONum);
      PacketSend(&reply,socket);
      break;
    }
    case dioctlDIOWaitHigh: {
      unsigned loops = 0;
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      while (1) {
	loops++;
	temp = dio->API->GetAsync(dio,DIONum);
	if (temp == INPUT_HIGH || temp == HIGH) {
	  break;
	} else {
	  if (OptPollRate > 0) {
	    times[0]->API->Delay(times[0],1000*OptPollRate);
	  }
	  sched_yield();
	}
      }
      //if (loops > 1) fprintf(stderr,"loops0=%d\n",loops);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      PacketInit(&reply);
      PacketPush8(&reply,DIONum);
      PacketSend(&reply,socket);
      break;
    }
    case dioctlDIOLockBlocking:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      dio->API->Lock(dio,WaitLock,DIONum);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOLockRelease:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      dio->API->Lock(dio,DoRelease,DIONum);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      break;
    case dioctlDIOLockNonBlocking:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      PacketInit(&reply);
      if (dio->API->Lock(dio,TryLock,DIONum)) {
	PacketPush8(&reply,0x5A);
      } else {
	PacketPush8(&reply,0x77);
      }
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      PacketSend(&reply,socket);
      break;
    case dioctlBoardModelGet:
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm = 0; }
      PacketInit(&reply);
      PacketPush8(&reply,0);
      PacketPush8(&reply,0);
      PacketPush8(&reply,ModelId>>8);
      PacketPush8(&reply,ModelId&0xFF);
      PacketSend(&reply,socket);
      break;
    case dioctlDIOTimeEventGet:
      {
	struct timeval tv;
	gettimeofday(&tv,0);
	PacketInit(&reply);
	PacketPush32(&reply,conn.EventLast.tv_sec);
	PacketPush32(&reply,conn.EventLast.tv_usec);
	PacketSend(&reply,socket);
      }
      break;
    case dioctlDIOTimeCurrentGet:
      {
	struct timeval tv;
	gettimeofday(&tv,0);
	PacketInit(&reply);
	PacketPush32(&reply,tv.tv_sec);
	PacketPush32(&reply,tv.tv_usec);
	PacketSend(&reply,socket);
      }
      break;
    case dioctlDIOTimeArm:
      conn.arm = 1;
      break;
    case dioctlDIOSleep:
      if (PacketPull24(socket,&temp) != 3) goto dioctlServerDone;
      times[0]->API->Delay(times[0],temp);
      break;
    case dioctlDIOLookup:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp = tmp;
      {
	char str1[256];
	char *str = str1;
	if (PacketPullBytes(socket,temp,str) != temp) goto dioctlServerDone;
	str[temp] = 0;
	if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
	DIONum = tmp;
	pthread_mutex_lock(&diomaplock);
	str = DIOMapLookupPartial(str,DIONum);
	pthread_mutex_unlock(&diomaplock);
	if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm=0; }
	temp = strlen(str);
	PacketInit(&reply);
	PacketPush8(&reply,temp);
	PacketPushBytes(&reply,temp,str);
	PacketSend(&reply,socket);
      }
      break;
    case dioctlDIONameLookup:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp = tmp;
      {
	char str[256];
	if (PacketPullBytes(socket,temp,str) != temp) goto dioctlServerDone;
	str[temp] = 0;
	pthread_mutex_lock(&diomaplock);
	temp2 = DIOMapLookup(str);
	pthread_mutex_unlock(&diomaplock);
	if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm=0; }
	PacketInit(&reply);
	if (temp2 == -1) {
	  PacketPush8(&reply,0);
	} else {
	  PacketPush8(&reply,1);
	}
	PacketPush8(&reply,temp2 & 0xFF);
	PacketSend(&reply,socket);
	log9(LOG_DIOCTL,"lookup %s=%d\n",str,temp2);
      }
      break;
    case dioctlDIONameAssign:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      DIONum = tmp;
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp = tmp;
      {
	char str[256];
	if (PacketPullBytes(socket,temp,str) != temp) goto dioctlServerDone;
	str[temp] = 0;
	pthread_mutex_lock(&diomaplock);
	if (DIOMapLookup(str) == -1) {
	  i=DIOMapAdd(str,DIONum);
	}
	pthread_mutex_unlock(&diomaplock);
	if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm=0; }
      }
      break;
    case dioctlDIONameRemove:
      if (PacketPull8(socket,&tmp) != 1) goto dioctlServerDone;
      temp = tmp;
      {
	char str[256];
	if (PacketPullBytes(socket,temp,str) != temp) goto dioctlServerDone;
	str[temp] = 0;
	pthread_mutex_lock(&diomaplock);
	DIOMapDelete(str);
	pthread_mutex_unlock(&diomaplock);
	if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm=0; }
      }
      break;
    case dioctlDIONamesList:
      PacketInit(&reply);
      pthread_mutex_lock(&diomaplock);
      i = DIOMapLength(&temp);
      PacketPush16(&reply,i);
      for (i=0;i<temp;i++) {
	char *str;

	str = DIOMapGet(i,&temp2);
	if (!str) continue;
	PacketPush8(&reply,temp2 & 0xFF);
	for (j=0;j<256 && str[j];j++);
	PacketPush8(&reply,j);
	PacketPushBytes(&reply,j,str);
	PacketSend(&reply,socket);
	PacketInit(&reply);
      }
      pthread_mutex_unlock(&diomaplock);
      if (conn.arm) { gettimeofday(&conn.EventLast,0); conn.arm=0; }
      break;
    }
  }
 dioctlServerDone:
  _PThreadLockUnlockAll((PThreadLock *)lock,&thread_data);
  pthread_mutex_lock(&watchlock);
  watchlockremove(&thread_data);
  pthread_mutex_unlock(&watchlock);

  close(socket);
  pthread_exit(0);
  return 0;
}

int DIOInterpretCommand(int socket,unsigned char API,unsigned char instance) {
  Packet reply;
  DIO *dio;

  log9(LOG_DIO,"DIOInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance >= numDIO) {
    return -1;
  }
  if (API >= DIO_APICount) {
    return -1;
  }
  dio = dios[instance];
  PacketInit(&reply);
  //pthread_mutex_lock(&diomutex[instance]);
  switch (API) {
  case NetDIO_Refresh: {
    dio->API->Refresh(dio);
    //pthread_mutex_unlock(&diomutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_Commit: {
    int forceall;
    if (PacketPull32(socket,&forceall) != 4) return -1;
    dio->API->Commit(dio,forceall);
    //pthread_mutex_unlock(&diomutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_Set: {
    int DIONum;
    DIOState State;
    if (PacketPull32(socket,&DIONum) != 4) return -1;
    if (PacketPull32(socket,(int *)&State) != 4) return -1;
    dio->API->Set(dio,DIONum,State);
    //pthread_mutex_unlock(&diomutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_Get: {
    int DIONum;
    if (PacketPull32(socket,&DIONum) != 4) return -1;
    PacketPush32(&reply,dio->API->Get(dio,DIONum));
    //pthread_mutex_unlock(&diomutex[instance]);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_SetAsync: {
    int DIONum;
    DIOState State;
    if (PacketPull32(socket,&DIONum) != 4) return -1;
    if (PacketPull32(socket,(int *)&State) != 4) return -1;
    dio->API->SetAsync(dio,DIONum,State);
    //pthread_mutex_unlock(&diomutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_GetAsync: {
    int DIONum;
    if (PacketPull32(socket,&DIONum) != 4) return -1;
    PacketPush32(&reply,dio->API->GetAsync(dio,DIONum));
    //pthread_mutex_unlock(&diomutex[instance]);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_Lock: {
    int command,ret;
    unsigned num;
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 

    struct ThreadData *th = pthread_getspecific(tdata);
    if (PacketPull32(socket,&command) != 4) return -1;
    if (PacketPull32(socket,&num) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
    th->state = 1;
    PacketPush32(&reply,dio->API->Lock(dio,command,num));
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);          
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetDIO_Wait: {
    int i,min,max,nh,nl,n;
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 
    struct ThreadData *th = pthread_getspecific(tdata);

    if (PacketPull32(socket,&min) != 4) return -1;
    if (PacketPull32(socket,&max) != 4) return -1;
    if (PacketPull32(socket,&nh) != 4) return -1;
    if (PacketPull32(socket,&nl) != 4) return -1;
    {
      int l[nl],h[nh],matches[nl+nh];
      for (i=0;i<nh;i++) {
	if (PacketPull32(socket,h+i) != 4) return -1;
      }
      for (i=0;i<nl;i++) {
	if (PacketPull32(socket,l+i) != 4) return -1;
      }
      assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
      th->state = 1;
      n = dio->API->Wait(dio,matches,min,max,nh,h,nl,l);
      th->state = 0;
      assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1); 
      PacketPush32(&reply,n);
      for (i=0;i<n;i++) {
	PacketPush32(&reply,matches[i]);
      }
      PacketSend(&reply,socket);      
    }
  }
  case NetDIO_Count: {
    PacketPush32(&reply,dio->API->Count(dio));
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIO_Capabilities: {
    int num;
    if (PacketPull32(socket,&num) != 4) return -1;
    PacketPush8(&reply,dio->API->Capabilities(dio,num));
    PacketSend(&reply,socket);      
    return 1;
  }
  }
}

int DIORawInterpretCommand(int socket,unsigned char API,unsigned char instance) {
  Packet reply;
  DIORaw *dio;

  log9(LOG_DIORAW,"DIORawInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance >= numDIORaw) {
    return -1;
  }
  if (API >= DIORaw_APICount) {
    return -1;
  }
  dio = dioraws[instance];
  PacketInit(&reply);
  //pthread_mutex_lock(&diorawmutex[instance]);
  switch (API) {
  case NetDIORaw_Lock: {
    int command,ret;
    unsigned num;
    struct ThreadData *th = pthread_getspecific(tdata);
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 

    if (PacketPull32(socket,&command) != 4) return -1;
    if (PacketPull32(socket,&num) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
    th->state = 1;
    PacketPush32(&reply,dio->API->Lock(dio,command,num));
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);          
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetDIORaw_DirSet: {
    int num,asOutput;
    if (PacketPull32(socket,&num) != 4) return -1;
    if (PacketPull32(socket,&asOutput) != 4) return -1;
    dio->API->DirSet(dio,num,asOutput);
    //pthread_mutex_unlock(&diorawmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIORaw_DataSet: {
    int num,asHigh;
    if (PacketPull32(socket,&num) != 4) return -1;
    if (PacketPull32(socket,&asHigh) != 4) return -1;
    dio->API->DataSet(dio,num,asHigh);
    //pthread_mutex_unlock(&diorawmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIORaw_DirGet: {
    int num;
    if (PacketPull32(socket,&num) != 4) return -1;
    PacketPush32(&reply,dio->API->DirGet(dio,num));
    //pthread_mutex_unlock(&diorawmutex[instance]);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIORaw_DataGet: {
    int num;
    if (PacketPull32(socket,&num) != 4) return -1;
    PacketPush32(&reply,dio->API->DataGet(dio,num));
    //pthread_mutex_unlock(&diorawmutex[instance]);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetDIORaw_Count: {
    int num;
    PacketPush32(&reply,dio->API->Count(dio));
    PacketSend(&reply,socket);
    return 1;    
  }
  }
}

int PinInterpretCommand(int socket,unsigned char API,unsigned char instance) {
  Packet reply;
  
  log9(LOG_PIN,"PinInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance > 0) {
    return -1;
  }
  if (API >= Pin_APICount) {
    return -1;
  }
  PacketInit(&reply);
  //pthread_mutex_lock(&pinmutex);
  switch (API) {
  case NetPin_Lock: {
    int command,ret;
    unsigned num;
    struct ThreadData *th = pthread_getspecific(tdata);
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 

    if (PacketPull32(socket,&command) != 4) return -1;
    if (PacketPull32(socket,&num) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
    th->state = 1;
    PacketPush32(&reply,pin->API->Lock(pin,command,num));
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);          
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetPin_ModeGet: {
    int pinnum;
    if (PacketPull32(socket,&pinnum) != 4) return -1;
    PacketPush32(&reply,pin->API->ModeGet(pin,pinnum));
    //pthread_mutex_unlock(&pinmutex);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetPin_ModeSet: {
    int pinnum;
    PinMode mode;
    if (PacketPull32(socket,&pinnum) != 4) return -1;
    if (PacketPull32(socket,(int *)&mode) != 4) return -1;
    PacketPush32(&reply,pin->API->ModeSet(pin,pinnum,mode));
    //pthread_mutex_unlock(&pinmutex);
    PacketSend(&reply,socket);
    return 1;    
  }
  }
}

int TimeInterpretCommand(int socket,unsigned char API,unsigned char instance) {
  Packet reply;
  Time *t;

  log9(LOG_TIME,"TimeInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance >= numTimes) {
    return -1;
  }
  t = times[instance];
  if (API >= Time_APICount) {
    return -1;
  }
  // no mutexes needed to protect Time class? not for SystemTime anyway
  PacketInit(&reply);
  switch (API) {
  case NetTime_Wait: {
    unsigned us;
    struct ThreadData *th = pthread_getspecific(tdata);
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 

    if (PacketPull32(socket,&us) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
    th->state = 1;
    PacketPush32(&reply,t->API->Wait(t,us));
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);          
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetTime_Delay: {
    unsigned us;
    struct ThreadData *th = pthread_getspecific(tdata);
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 

    if (PacketPull32(socket,&us) != 4) return -1;
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
    th->state = 1;
    t->API->Delay(t,us);
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);          
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetTime_Tick: {
    PacketPush32(&reply,t->API->Tick(t));
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetTime_TPS: {
    PacketPush32(&reply,t->API->TPS(t));
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetTime_usElapsed: {
    unsigned start;
    if (PacketPull32(socket,&start) != 4) return -1;
    PacketPush32(&reply,t->API->usElapsed(t,start));
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetTime_usFuture: {
    unsigned start,us;
    if (PacketPull32(socket,&start) != 4) return -1;
    if (PacketPull32(socket,&us) != 4) return -1;
    PacketPush32(&reply,t->API->usFuture(t,start,us));
    PacketSend(&reply,socket);
    return 1;    
  }
  case NetTime_TimeoutQ: {
    unsigned start,end;
    if (PacketPull32(socket,&start) != 4) return -1;
    if (PacketPull32(socket,&end) != 4) return -1;
    PacketPush32(&reply,t->API->TimeoutQ(t,start,end));
    PacketSend(&reply,socket);
    return 1;    
  }
    /*
  case NetTime_Now: {
    unsigned long long tmp = t->API->Now(t);
    //fprintf(logg,"now=%llu\n",tmp);
    PacketPush64(&reply,tmp); // t->API->Now(t));
    PacketSend(&reply,socket);
    return 1;    
  }
    */
  }

}

// note: for NetBus*Stream, we may assume some maximum length for the buffer
// such as 256.  we probably need some better way of handling this, such as
// making the length parameter an unsigned character or something...
int BusInterpretCommand(int socket,unsigned char API,unsigned char instance) {
  Packet reply;
  Bus *bus;

  log9(LOG_BUS,"BusInterpretCommand(%d,%d,%d)\n",socket,API,instance);
  if (instance >= numBusses) {
    return -1;
  }
  if (API >= Bus_APICount) {
    return -1;
  }
  bus = busses[instance];
  //fprintf(stderr,"Bus command %d\n",API);
  //pthread_mutex_lock(&busmutex[instance]);
  //fprintf(logg,"Bus %d command %d begins\n",instance,API); fflush(logg);
  PacketInit(&reply);
  switch (API) {
  case NetBus_Lock: {
    int command,ret;
    unsigned num;
    struct ThreadData *th = pthread_getspecific(tdata);
    if (PacketPull32(socket,&command) != 4) return -1;
    if (PacketPull32(socket,&num) != 4) return -1;
    struct epoll_event event;                    
    event.events = EPOLLRDHUP;                   
    event.data.ptr = pthread_getspecific(tdata); 
    assert(epoll_ctl(efd,EPOLL_CTL_ADD,th->socket,&event)!=-1);     
    th->state = 1;
    PacketPush32(&reply,bus->API->Lock(bus,command,num));
    th->state = 0;
    assert(epoll_ctl(efd,EPOLL_CTL_DEL,th->socket,0)!=-1);          
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Peek8: {
    int adrs;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    PacketPush8(&reply, bus->API->Peek8(bus,adrs));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Poke8: {
    int adrs;
    unsigned char value;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull8(socket,&value) != 1) return -1;
    bus->API->Poke8(bus,adrs,value);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Peek16: {
    int adrs;
    unsigned short tmp;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    PacketPush16(&reply,tmp= bus->API->Peek16(bus,adrs));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    //fprintf(stderr,"%d sent %X\n",instance,tmp);
    return 1;
  }
  case NetBus_Poke16: {
    int adrs;
    unsigned short value;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull16(socket,&value) != 2) return -1;
    bus->API->Poke16(bus,adrs,value);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Peek32: {
    int adrs;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    PacketPush32(&reply, bus->API->Peek32(bus,adrs));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Poke32: {
    int adrs;
    unsigned value;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&value) != 4) return -1;
    bus->API->Poke32(bus,adrs,value);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitGet8: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    PacketPush32(&reply,bus->API->BitGet8(bus,adrs,bit));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitAssign8: {
    int adrs,bit,val;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    if (PacketPull32(socket,&val) != 4) return -1;
    bus->API->BitAssign8(bus,adrs,bit,val);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitSet8: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    bus->API->BitSet8(bus,adrs,bit);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitClear8: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    bus->API->BitClear8(bus,adrs,bit);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitGet16: {
    int adrs,bit,tmp;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    PacketPush32(&reply,tmp=bus->API->BitGet16(bus,adrs,bit));
    log9(LOG_BUS,"BitGet16(%d,%d)=%d\n",adrs,bit,tmp);
    //fprintf(stderr,"BitGet16<%d>(%d,%d)=%d\n",instance,adrs,bit,tmp);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitAssign16: {
    int adrs,bit,val;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    if (PacketPull32(socket,&val) != 4) return -1;
    bus->API->BitAssign16(bus,adrs,bit,val);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitSet16: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    bus->API->BitSet16(bus,adrs,bit);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitClear16: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    bus->API->BitClear16(bus,adrs,bit);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitGet32: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    PacketPush32(&reply,bus->API->BitGet32(bus,adrs,bit));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitAssign32: {
    int adrs,bit,val;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    if (PacketPull32(socket,&val) != 4) return -1;
    bus->API->BitAssign32(bus,adrs,bit,val);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitSet32: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    bus->API->BitSet32(bus,adrs,bit);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitClear32: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    bus->API->BitClear32(bus,adrs,bit);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_PeekStream: {
    int adrs,dir;
    unsigned char len;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&dir) != 4) return -1;
    if (PacketPull8(socket,&len) != 1) return -1;
    bus->API->PeekStream(bus,adrs,dir,reply.data,len);
    //pthread_mutex_unlock(&busmutex[instance]);
    reply.len = len;
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_PokeStream: {
    int adrs,dir;
    unsigned char len;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&dir) != 4) return -1;
    if (PacketPull8(socket,&len) != 1) return -1;
    if (PacketPullBytes(socket,len,reply.data) != len) return -1;
    bus->API->PokeStream(bus,adrs,dir,reply.data,len);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Refresh: {
    bus->API->Refresh(bus);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Commit: {
    int forceall;
    if (PacketPull32(socket,&forceall) != 4) return -1;
    bus->API->Commit(bus,forceall);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketPush32(&reply,1);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitToggle8: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    PacketPush32(&reply,bus->API->BitToggle8(bus,adrs,bit));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitToggle16: {
    int adrs,bit,tmp;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    PacketPush32(&reply,tmp=bus->API->BitToggle16(bus,adrs,bit));
    log9(LOG_BUS,"BitToggle16(%d,%d)=%d\n",adrs,bit,tmp);
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitToggle32: {
    int adrs,bit;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit) != 4) return -1;
    PacketPush32(&reply,bus->API->BitToggle32(bus,adrs,bit));
    //pthread_mutex_unlock(&busmutex[instance]);
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Assign8X: {
    int adrs,bit1,bit0,value;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit1) != 4) return -1;
    if (PacketPull32(socket,&bit0) != 4) return -1;
    if (PacketPull32(socket,&value) != 4) return -1;
    PacketPush8(&reply,bus->API->Assign8X(bus,adrs,bit1,bit0,value));
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Assign16X: {
    int adrs,bit1,bit0,value;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit1) != 4) return -1;
    if (PacketPull32(socket,&bit0) != 4) return -1;
    if (PacketPull32(socket,&value) != 4) return -1;
    PacketPush16(&reply,bus->API->Assign16X(bus,adrs,bit1,bit0,value));
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_Assign32X: {
    int adrs,bit1,bit0,value;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit1) != 4) return -1;
    if (PacketPull32(socket,&bit0) != 4) return -1;
    if (PacketPull32(socket,&value) != 4) return -1;
    PacketPush32(&reply,bus->API->Assign32X(bus,adrs,bit1,bit0,value));
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitsGet8: {
    int adrs,bit1,bit0;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit1) != 4) return -1;
    if (PacketPull32(socket,&bit0) != 4) return -1;
    PacketPush8(&reply,bus->API->BitsGet8(bus,adrs,bit1,bit0));
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitsGet16: {
    int adrs,bit1,bit0;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit1) != 4) return -1;
    if (PacketPull32(socket,&bit0) != 4) return -1;
    //tmp = bus->API->BitsGet16(bus,adrs,bit1,bit0);
    //log9(LOG_BUS,"tmp=%X\n",tmp);
    //PacketPush16(&reply,tmp);
    PacketPush16(&reply,bus->API->BitsGet16(bus,adrs,bit1,bit0));
    PacketSend(&reply,socket);
    return 1;
  }
  case NetBus_BitsGet32: {
    int adrs,bit1,bit0;
    if (PacketPull32(socket,&adrs) != 4) return -1;
    if (PacketPull32(socket,&bit1) != 4) return -1;
    if (PacketPull32(socket,&bit0) != 4) return -1;
    PacketPush32(&reply,bus->API->BitsGet32(bus,adrs,bit1,bit0));
    PacketSend(&reply,socket);
    return 1;
  }

  }
}

struct listener_info {
  int port,socket,inst;
  void *(*server_function)(void *); // Tx function or Tx/Rx
  void *(*server_function2)(void *); // Rx function or unused 
};

/*
  DIOServer actually has separate processes for each client.
  We want to have a single process with multiple threads, instead, since we
  need to share the same objects among all clients.  Since we use pthreads,
  we use a pthread mutex to protect each object so that only one thread
  at a time can be calling into an object.

  DIOServer uses a mechanism whereby we only have to receive part of
  a request before we start processing it.  We must do a similar thing.
  All requests start with NetRequest, therefore we must read a NetRequest,
  and then let the individual object determine how many additional bytes
  must be read before the command can be performed.

  Our main loop waits to accept new connections, and creates a new
  thread to handle each one.
 */

static void watchlockremove(ThreadData *th) {
  ThreadData *cur = head_thread;
  if (cur == th) { // we are the only thread
    head_thread = 0;
  } else while (cur) {
      if (cur->next == th) { // next thread is us
	cur->next = cur->next->next; // remove us from the chain
      } else {
	cur = cur->next; // keep looking
      }
    }
}

static void SignalHandle(int signal) {
  ThreadData *cur;
  log9(LOG_SYSTEM,"debug: Caught signal %d\n",signal);
  pthread_cancel(pthread_self());
  sleep(1);
}

void *WatchLoop(void *arg) {
  ThreadData *cur;
  int dummy;
  struct epoll_event events;

  while (1) {
    if (epoll_wait(efd,&events,1,-1) > 0) {
      cur = events.data.ptr;
      log9(LOG_SYSTEM,"shutdown %d\n",cur->tid);
      pthread_mutex_lock(&watchlock);
      close(cur->socket); 
      //_PThreadLockUnlockAll((PThreadLock *)lock,cur);
      watchlockremove(cur);
      dummy = pthread_kill(cur->tid,15);
      //sleep(2);
      //dummy = pthread_cancel(cur->tid);
      pthread_mutex_unlock(&watchlock);
    }

#if ZERO_BYTE_WRITE_RETURNS_MINUS_1_ON_CONNECTION_CLOSE
    usleep(1000000);
    pthread_mutex_lock(&watchlock);
    cur = head_thread;
    while (cur) {
      if (cur->state == 1 && write(cur->socket,&dummy,0) == -1
	  && errno == -ECONNRESET) {
	close(cur->socket);
	_PThreadLockUnlockAll(&globallock,cur);
	watchlockremove(cur);
	pthread_cancel(cur->tid);
      }
      cur = cur->next;
    }
    pthread_mutex_unlock(&watchlock);
#endif
  }
}

void thread_destructor(void *arg) {
  ThreadData *th = arg;
  int i,n;

  int *can = th->conndata;
  //fprintf(stderr,"debug: destructor\n");
  _PThreadLockUnlockAll((PThreadLock *)lock,th);
  //fprintf(stderr,"debug: destructor unlocked all\n");
  if (can) {
    for (i=0;i<numCAN;i++) {
      if (can[i] >= 0) {
	log9(LOG_SYSTEM,"*closing %d\n",can[i]);
	n = can[i];
	can[i] = -1;
	close(n);
      }
    }
  }
}

typedef struct modbus_frame {
  short transactionId;
  short protocolId;
  short length;
  unsigned char unitid;
  unsigned char functioncode;
  unsigned char data[256]; // probably can be less than this!
} __attribute__((packed)) modbus_frame;

short ModbusRead(int madrs) {
  DIO *dio = dios[0];
  int i,dionum = madrs * 8,val;
  short ret=0;

  for (i=0;i<8;i++) {
    dio->API->Lock(dio,WaitLock,dionum+i);
  }
  dio->API->Refresh(dio);
  for (i=0;i<8;i++) {
    val = dio->API->Get(dio,dionum+i);
    ret <<= 2;
    switch (val) {
    case INPUT_LOW:
      ret += 2;
      break;
    case INPUT_HIGH:
      ret += 3;
      break;
    case LOW:
      ret += 0;
      break;
    case HIGH:
      ret += 1;
      break;
    }
  }
  for (i=0;i<8;i++) {
    dio->API->Lock(dio,DoRelease,dionum+i);
  }
  return ret;
}

void ModbusWrite(int madrs,int val) {
  DIO *dio = dios[0];
  int i,dionum = madrs * 8,v;
  short ret=0;

  //printf("[%d]=%X\n",madrs,val);
  for (i=0;i<8;i++) {
    dio->API->Lock(dio,WaitLock,dionum+i);
  }
  dio->API->Refresh(dio);
  for (i=0;i<8;i++) {
    v = (val >> 14) & 3;
    val <<= 2;
    switch (v) {
    case 0:
      dio->API->Set(dio,dionum+i,LOW);
      //printf("%d=LOW\n",dionum+i);
      break;
    case 1:
      dio->API->Set(dio,dionum+i,HIGH);
      //printf("%d=HIGH\n",dionum+i);
      break;
    case 2:
    case 3:
      dio->API->Set(dio,dionum+i,INPUT);
      //printf("%d=INPUT\n",dionum+i);
      break;
    }
  }
  dio->API->Commit(dio,0);
  for (i=0;i<8;i++) {
    dio->API->Lock(dio,DoRelease,dionum+i);
  }
}

void *ModbusLoop(void *arg) {
  int socket = (int)arg;
  THREAD_SPECIFIC_DATA_INIT;
  modbus_frame request,reply;
  short regs[256];
  int i,len;

  for (i=0;i<256;i++) regs[i] = i;
  thread_data.socket = socket;
  fcntl(socket,F_SETFL,fcntl(socket,F_GETFL,0)&~O_NONBLOCK);
  pthread_setspecific(tdata,&thread_data);
  while (1) {
    if (PacketPullBytes(socket,8,(unsigned char *)&request) != 8) break;
    reply.transactionId = request.transactionId;
    reply.protocolId = request.protocolId;
    reply.unitid = request.unitid;
    reply.functioncode = request.functioncode;
    request.transactionId = ntohs(request.transactionId);
    request.protocolId = ntohs(request.protocolId);
    request.length = ntohs(request.length);
    if (request.length > 256) break;
    if (PacketPullBytes(socket,
			request.length-2,
			(unsigned char *)&request.data)
	!= request.length-2) break;
    //printf("%d:%d:%d:%d:%d\n",request.transactionId,request.protocolId,
    //	   request.length,request.unitid,request.functioncode);
    switch (request.functioncode) {
    case 3: { // read multiple
      short regnum = ntohs(*((short *)&request.data[0]));
      short wordcount = ntohs(*((short *)&request.data[2]));
      //printf("regnum=%d, wordcount=%d\n",regnum,wordcount);
      reply.data[0] = 2*wordcount;
      for (i=0;i<wordcount;i++) {
	short val = ModbusRead(regnum+i);
	//printf("%d:%X\n",regnum+i,val);
	reply.data[1+i*2] = val>>8;   // regs[(i+regnum)%256] >> 8;
	reply.data[2+i*2] = val&0xFF; //regs[(i+regnum)%256] & 0xFF;
      }
      len = 2+1+wordcount*2;
      reply.length = htons(len);
      PacketSend0(socket,(char *)&reply,6+len);
      break;
    }
    case 16: { // write multiple
      short regnum = ntohs(*((short *)&request.data[0]));
      short wordcount = ntohs(*((short *)&request.data[2]));
      unsigned char bytecount = request.data[4];
      printf("regnum=%d, wordcount=%d, bytecount=%d\n",regnum,wordcount,bytecount);
      for (i=0;i<wordcount;i++) {
	ModbusWrite(regnum+i,256*request.data[5+i*2] + request.data[6+i*2]);
	//printf("[%d]=%X (%X,%X)\n",(i+regnum)%256,256*request.data[5+i*2] + request.data[6+i*2],256*request.data[5+i*2],request.data[6+i*2]);
	//regs[(i+regnum)%256] = 256*request.data[5+i*2] + request.data[6+i*2];
      }
      reply.length = htons(6);
      reply.data[0] = request.data[0];
      reply.data[1] = request.data[1];
      reply.data[2] = request.data[2];
      reply.data[3] = request.data[3];
      PacketSend0(socket,(char *)&reply,6+6);
      break;
    }
    default: {
      reply.length = htons(1);
      reply.functioncode += 0x80;
      reply.data[0] = 1;
      PacketSend0(socket,(char *)&reply,6+1);
      // close connection?  we are out of sync now...
      break;
    }
    }

    
  }
  THREAD_SPECIFIC_DATA_FINI;
  close(socket);
  pthread_exit(0);
  return 0;
}

void *NetLoop(void *arg) {
  int socket = (int)arg;
  unsigned char API, instance;
  unsigned short class;
  THREAD_SPECIFIC_DATA_INIT;
  int i,n,can[numCAN];
  int tid = pthread_self();
  thread_data.socket = socket;
  thread_data.conndata = can;
  thread_data.pid = getpid();

  for (i=0;i<numCAN;i++) {
    can[i]=-1;
  }

  fcntl(socket,F_SETFL,fcntl(socket,F_GETFL,0)&~O_NONBLOCK);
  //fprintf(stderr,"debug: 1: th=%p\n",&thread_data);
  pthread_setspecific(tdata,&thread_data);

  // insert new thread at head of list
  pthread_mutex_lock(&watchlock);
  thread_data.next = head_thread;
  head_thread = &thread_data;
  pthread_mutex_unlock(&watchlock);

  log9(LOG_SYSTEM,"netloop: socket=%d tid=%d, pid=%d\n",socket,tid,
       thread_data.pid);
  while (1) {
    if (PacketPull8(socket,&API) != 1) goto NetLoopDone;
    if (PacketPull8(socket,&instance) != 1) goto NetLoopDone;
    if (PacketPull16(socket,&class) != 2) goto NetLoopDone;
    log9(LOG_SYSTEM,"%d netloop: class=%d inst=%d API=%d\n",tid,class,instance,API);
    switch (class) {
    case NetSystemClass:
      if (SystemInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetBusClass: 
      if (BusInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetTimeClass:
      if (TimeInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetPinClass:
      if (!pin) break;
      if (PinInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetDIORawClass:
      if (DIORawInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetDIOClass:
      if (DIOInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetTWIClass:
      if (TWIInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    case NetCANClass:
      if (CANInterpretCommand(socket,API,instance,can) < 0) goto NetLoopDone;
      break;
    case NetSPIClass:
      if (SPIInterpretCommand(socket,API,instance) < 0) goto NetLoopDone;
      break;
    default:
      goto NetLoopDone;
      break;
    }
  }
 NetLoopDone:
  log9(LOG_SYSTEM,"shutting down %d\n",tid);
  THREAD_SPECIFIC_DATA_FINI;
  close(socket);
  for (i=0;i<numCAN;i++) {
    if (can[i] >= 0) {
      log9(LOG_SYSTEM,"xclosing %d\n",can[i]);
      n = can[i];
      can[i] = -1;
      close(n);
    }
  }
  // remove thread from the list
  _PThreadLockUnlockAll((PThreadLock *)lock,&thread_data);
  pthread_mutex_lock(&watchlock);
  watchlockremove(&thread_data);
  pthread_mutex_unlock(&watchlock);

  pthread_exit(0);
  return 0;
}

// This is to work around a nasty bug in uClibc whereby
// pthread_create() never returns if daemon has been called.
void daemonize() {
  int i;

  for (i = 0; i < 3; ++i) {
    close(i);
  }
  open("/", O_RDONLY);
  dup2(0, 1);
  dup2(0, 2);

  i = open("/dev/tty", O_RDWR);
  if (i >= 0) {
    ioctl(i, TIOCNOTTY, 0);
    close(i);
  }
  setsid();
}

#include <unistd.h> // access
#define SYNC_FILE "/mnt/host/x/home/michael/tsctl/syncfile"
#include "SJA1000CAN.h"

volatile int gotSigInt = 0;
static void SigInt(int signal) {
  gotSigInt = 1;
}

int spictl_interpret(QueueBytes *request,QueueBytes *reply,void *data) {
  SPI *spi = data;
  int len,adrs=1,cs;

  // returns 0 if it can't get a CS command
  // otherwise sets returns the new adrs
  int spictl_interpret_CS() {
    char buf[3],*cmd;
    int hz,len;

    if (QueueBytesReady(request) < 3) return -1; // not enough data
    cmd = QueueBytesDequeuePrep(request,0,&len);
    if ((*cmd >> 6) != 0) return -1; // not CS command
    //fprintf(stderr,"SPI CS lookahead\n");
    QueueBytesDequeue(request,buf,3);
    hz = 2048*(256*buf[1]+buf[2]);
    if (hz != 0) {
      spi->API->ClockSet(spi,hz);
    }
    if (buf[0] & (1 << 3)) {
      spi->API->EdgeSet(spi,(buf[0] & (1 << 2))?1:0);
    }
    if (buf[0] & (1 << 4)) {
      adrs = 1 + (buf[0] & 3);
      if ((buf[0] & (1 << 5)) == 0) {
	adrs = -adrs;
      }
    }
    return 1;
  }

  // we will always have at least 1 byte in the request buffer when called
  // thus we request 0 bytes just to peek at the first byte in queue
  // a peek function would work better
  while (QueueBytesReady(request)>0) {
    char *cmd = QueueBytesDequeuePrep(request,0,&len);
    switch (*cmd >> 6) {
    case 0: // CS
      //fprintf(stderr,"SPI CS\n");
      if (spictl_interpret_CS() == -1) return 1; // not enough data
      break;
    case 1: // READ
      //fprintf(stderr,"SPI R\n");
      QueueBytesDequeuePrep(request,1,&len); // pop would work better!
      spictl_interpret_CS();
      len = 1 << (*cmd & 0x3F);
      {
	char buf[len];
	
	spi->API->Lock(spi,WaitLock,0); // won't wait since we are serialized
	spi->API->Read(spi,adrs,buf,len);
	spi->API->Lock(spi,DoRelease,0);
	QueueBytesEnqueue(reply,buf,len);
	// would be faster to use QueueBytesEnqueuePrep. this should not break
	// this SPI transaction apart into two pieces as our reply buffer
	// will always be completely sent before we process our next request
      }
      break;
    case 2: // WRITE
      //fprintf(stderr,"SPI W\n");
      QueueBytesDequeuePrep(request,1,&len); // pop would work better!
      if (QueueBytesSize(request) < len + 1) {
	QueueBytesResize(request,len+1);
      }
      if (QueueBytesReady(request) < len+1) return 1; // not enough data
      len = 1 << (*cmd & 0x3F);
      {
	char buf[len];
	QueueBytesDequeue(request,buf,len);
	spictl_interpret_CS();
	spi->API->Lock(spi,WaitLock,0); // won't wait since we are serialized
	spi->API->Write(spi,adrs,buf,len);
	spi->API->Lock(spi,DoRelease,0);
	// would be faster to use QueueBytesDequeuePrep. this should not break
	// this SPI transaction apart into two pieces as our reply buffer
	// will always be completely sent before we process our next request
      }
      break;
    case 3: // READWRITE
      //fprintf(stderr,"SPI RW\n");
      QueueBytesDequeuePrep(request,1,&len); // pop would work better!
      if (QueueBytesSize(request) < len + 1) {
	QueueBytesResize(request,len+1);
      }
      if (QueueBytesReady(request) < len+1) return 1; // not enough data
      len = 1 << (*cmd & 0x3F);
      {
	char wbuf[len],rbuf[len];
	QueueBytesDequeue(request,wbuf,len);
	spictl_interpret_CS();
	spi->API->Lock(spi,WaitLock,0); // won't wait since we are serialized
	spi->API->ReadWrite(spi,adrs,wbuf,rbuf,len);
	spi->API->Lock(spi,DoRelease,0);
	QueueBytesEnqueue(reply,rbuf,len);
	// use the prep functions instead to improve perforamnce
      }
      break;
    }
  }
  return 1; // unreachable
}

void *spi_listener(void *arg) {
  int inst = (int)arg,listen;

  THREAD_SPECIFIC_DATA_INIT;
  thread_data.pid = getpid();

  pthread_mutex_lock(&watchlock);
  thread_data.next = head_thread;
  head_thread = &thread_data;
  pthread_mutex_unlock(&watchlock);

  pthread_setspecific(tdata,&thread_data);
  ServerTCPRun(5501+inst,spictl_interpret,spis[inst]);
}

void *can_listener(void *arg) {
  int inst = (int)arg,i;
  int listen,irqfd,s,tmp;
  struct sockaddr_in csa;
  unsigned size_csa = sizeof(csa);
  THREAD_SPECIFIC_DATA_INIT;
  thread_data.pid = getpid();

  pthread_mutex_lock(&watchlock);
  thread_data.next = head_thread;
  head_thread = &thread_data;
  pthread_mutex_unlock(&watchlock);

  pthread_setspecific(tdata,&thread_data);

  {
    char tmp[80];
    sprintf(tmp,"/proc/irq/%d/irq",cans[inst]->irq);
    irqfd = open(tmp,O_RDONLY|O_SYNC);
  }
  if (irqfd < 0) return 0;
  if (inst == 1) {
    fcntl(irqfd,F_SETFL,fcntl(irqfd,F_GETFL,0)|O_NONBLOCK); // for debug
  }
  listen = ServerSocketNew((int)(7552+inst));
  fcntl(listen,F_SETFL,fcntl(listen,F_GETFL,0)&~O_NONBLOCK);
  // sync on file exists
  //  while (access(SYNC_FILE,O_RDONLY) != F_OK) sleep(1);
  // usleep(100000); // hopefully allows client to create before running canctl

  log9(LOG_CAN,"CAN%d listen=%d\n",inst,listen);
  while (!gotSigInt) {
    if ((s=accept(listen, (struct sockaddr *)&csa, &size_csa)) >= 0) {
      log9(LOG_CAN,"Entering %d CANMain (%d)\n",inst,s);
      log9(LOG_CAN,"cans[%d]->D.nconn=%d ",inst,((SJA1000CAN *)cans[inst])->D.nconn);
      for (i=0;i<((SJA1000CAN *)cans[inst])->D.nconn;i++) {
	log9(LOG_CAN,"[%d]",((SJA1000CAN *)cans[inst])->D.conn[i].socket);
      }
      log9(LOG_CAN,"\n");
      tmp = CANMain(cans[inst],listen,irqfd,s);
      log9(LOG_CAN,"CANMain returned\n");
      if (tmp == 6) break;
    }
  }
}

void *server_listener(void *arg) {
  struct listener_info *info = arg;
  pthread_t th;
  int listen,s;
  struct sockaddr_in csa;
  unsigned size_csa = sizeof(csa);
  fd_set rfd, c_rfd;
  int rc,total=0;

  listen = ServerSocketNew(info->port);
  if (listen < 0) {
    log9(LOG_SYSTEM,"ServerSocketNew(%d): %d: %m\n",info->port,listen);
    return 0;
  }

  FD_ZERO(&rfd);
  FD_SET(listen,&rfd);

  while (1) {
    do {
      c_rfd = rfd;
      //fprintf(stderr,"waiting for connection on port %d\n",info->port); fflush(logg);
      rc = select(listen+1, &c_rfd, 0, 0, 0);
    } while (rc == -1 && errno == EINTR);
    info->socket = s = accept(listen, (struct sockaddr *)&csa, &size_csa);
    //fprintf(stderr,"port %d socket %d/%d\n",info->port,info->socket,s); //fflush(logg);
    if (info->socket < 0) continue;
    pthread_create(&th, NULL, info->server_function, (void *)info->socket);
    pthread_detach(th); // MANDATORY
    total++;
    if (info->server_function2) {
      pthread_create(&th, NULL, info->server_function2, (void *)info->socket);
      pthread_detach(th); // MANDATORY
    }
  }
}

void *log_listener(void *arg) {
  int listen;
  struct sockaddr_in csa;
  unsigned size_csa = sizeof(csa);
  char tmp[16];

  listen = ServerSocketNew(1099);
  if (listen < 0) {
    log9(LOG_SYSTEM,"ServerSocketNew(%d): %d: %m\n",1099,listen);
    return 0;
  }
  fcntl(listen,F_SETFL,fcntl(listen,F_GETFL,0)&~O_NONBLOCK);
  while (1) {
    if ((s=accept(listen, (struct sockaddr *)&csa, &size_csa)) >= 0) {
      fcntl(s,F_SETFL,fcntl(s,F_GETFL,0)&~O_NONBLOCK);
      pthread_mutex_lock(&loglock);
      if (logconns < MAX_LOG_CONN) {
	LC[logconns].socket = s;
	if (read(s,tmp,16) > 0) {
	  LC[logconns].flags =  strtoul(tmp, NULL, 0);
	  logflags |= LC[logconns].flags;
	  logconns++;
	} else {
	  close(s);
	}
      } else {
	close(s);
      }
      pthread_mutex_unlock(&loglock);
    }
  }

}

int server() {
  pthread_t th;
  void *arg;
  int socket;
  int listen,i,j;
  THREAD_SPECIFIC_DATA_INIT;

  pthread_key_create(&tdata, thread_destructor);
  pthread_setspecific(tdata,&thread_data);
  ConfigLoadFile("/etc/dioctl.config");
  ArchInit0();
  
  //daemonize(); //daemon(1,0); avoided due to uClibc bug
  SignalIgnore(SIGCHLD);
  SignalIgnore(SIGPIPE);
  
  struct sockaddr_in csa;
  unsigned size_csa = sizeof(csa);
  fd_set rfd, c_rfd;
  int rc;
  struct listener_info main_listener, dioctl_listener, modbus_listener,
    http_listener;

  pthread_create(&th, NULL, log_listener, 0);
  pthread_detach(th); // MANDATORY

  for (i=0;i<numCAN;i++) {
    pthread_create(&th, NULL, can_listener, (void *)i);
    pthread_detach(th); // MANDATORY
  }

  for (i=0;i<numSPI;i++) {
    pthread_create(&th, NULL, spi_listener, (void *)i);
    pthread_detach(th); // MANDATORY
  }

  efd = epoll_create(8);
  assert(efd != -1);
  pthread_create(&th, NULL, WatchLoop, 0);
  pthread_detach(th); // MANDATORY

  {
    DIO *dio = dios[0]; // DIOInit1(0);
    char str[256];
    int cap;

    j = dio->API->Count(dio);
    for (i=0;i<j;i++) {
      cap = dio->API->Capabilities(dio,i);
      switch(cap) {
      case DIO_INPUT_ONLY: 
	sprintf(str,"attrib.%d.inputOnly",i);
	DIOMapAdd(str,1);
	break;
      case DIO_CAN_DRIVE_HIGH: 
	sprintf(str,"attrib.%d.highOnly",i);
	DIOMapAdd(str,1);
	break;
      case DIO_CAN_DRIVE_LOW: 
	sprintf(str,"attrib.%d.lowOnly",i);
	DIOMapAdd(str,1);
	break;
      case DIO_OUTPUT_ONLY: 
	sprintf(str,"attrib.%d.outputOnly",i);
	DIOMapAdd(str,1);
	break;
      case DIO_NORMAL: 
	sprintf(str,"attrib.%d.normalIO",i);
	DIOMapAdd(str,1);
	break;
      }
    }
  }
  dioctl_listener.port = 4500;
  dioctl_listener.server_function = dioctlServer;
  dioctl_listener.server_function2 = 0;
  pthread_create(&th, NULL, server_listener, &dioctl_listener);
  pthread_detach(th); // MANDATORY

  modbus_listener.port = 502;
  modbus_listener.server_function = ModbusLoop;
  modbus_listener.server_function2 = 0;
  pthread_create(&th, NULL, server_listener, &modbus_listener);
  pthread_detach(th); // MANDATORY

  http_listener.port = 8000;
  http_listener.server_function = httpServer;
  http_listener.server_function2 = 0;
  pthread_create(&th, NULL, server_listener, &http_listener);
  pthread_detach(th); // MANDATORY

  main_listener.port = 5001;
  main_listener.server_function = NetLoop;
  main_listener.server_function2 = 0;
  server_listener(&main_listener);

}

char *Class[] = {
  "System", "Bus", "Time", "Pin", "DIORaw", "DIO", "TWI", "CAN", "SPI", 0
};

char *SystemCommandNames[] = {
  "ClassCount", "InstanceCount", "APICount", 
  "LockCount", "LockHolderInfo", "ConnWaitInfo", "CANBusGet", "BuildInfo",
  "ModelId", "BaseBoardId", "MapLength", "MapGet", "MapLookup", 
  "MapLookupPartial", "MapAdd", "MapDelete",
  0
};

char *BusCommandNames[] = {
  "Peek8", "Poke8", "Peek16", "Poke16", "Peek32",
  "Poke32", "BitGet8", "BitAssign8", "BitSet8", "BitClear8", "BitGet16",
  "BitAssign16", "BitSet16", "BitClear16", "BitGet32", "BitAssign32",
  "BitSet32", "BitClear32", "PeekStream", "PokeStream", "Refresh", "Commit",
  "BitToggle8", "BitToggle16", "BitToggle32", "Assign8X", "Assign16X",
  "NetBus_Assign32X", "BitsGet8", "BitsGet16", "BitsGet32", 0
};
char *TimeCommandNames[] = {
  "Wait", "Delay", "Tick", "usElapsed", "usFuture", "TimeoutQ", "TPS", 0
};
char *PinCommandNames[] = {
  "ModeGet", "ModeSet", 0
};
char *DIORawCommandNames[] = {
  "DirSet", "DataSet", "DirGet", "DataGet", "Count", 0
};
char *DIOCommandNames[] = {
  "Refresh", "Commit", "Set", "Get", 
  "SetAsync", "GetAsync", "Wait", "Count", "Capabilities", 0
};

char *TWICommandNames[] = {
  "Write","Read", 0
};

char *CANCommandNames[] = {
  "Rx", "Tx", "BaudSet", "BaudGet", 0
};

char *SPICommandNames[] = {
  "Write","Read","ReadWrite","ClockSet","EdgeSet", 0
};

int SystemCommandOpcode[] = {
  NetSystem_ClassCount, NetSystem_InstanceCount, NetSystem_APICount,
  NetSystem_LockCount, NetSystem_LockHolderInfo, NetSystem_ConnWaitInfo,
  NetSystem_CANBusGet, NetSystem_BuildInfo, NetSystem_ModelId, 
  NetSystem_BaseBoardId, NetSystem_MapLength, NetSystem_MapGet,
  NetSystem_MapLookup, NetSystem_MapLookupPartial, NetSystem_MapAdd,
  NetSystem_MapDelete
};

int BusCommandOpcode[] = {
  NetBus_Peek8, NetBus_Poke8, 
  NetBus_Peek16, NetBus_Poke16, NetBus_Peek32, NetBus_Poke32, 
  NetBus_BitGet8, NetBus_BitAssign8, NetBus_BitSet8, NetBus_BitClear8, 
  NetBus_BitGet16, NetBus_BitAssign16, NetBus_BitSet16, NetBus_BitClear16, 
  NetBus_BitGet32, NetBus_BitAssign32, NetBus_BitSet32, NetBus_BitClear32, 
  NetBus_PeekStream, NetBus_PokeStream, NetBus_Refresh, NetBus_Commit, 
  NetBus_BitToggle8, NetBus_BitToggle16, NetBus_BitToggle32,
  NetBus_Assign8X, NetBus_Assign16X, NetBus_Assign32X,
  NetBus_BitsGet8, NetBus_BitsGet16, NetBus_BitsGet32
};
int TimeCommandOpcode[] = {
  NetTime_Wait, NetTime_Delay, NetTime_Tick, NetTime_usElapsed,
  NetTime_usFuture, NetTime_TimeoutQ, NetTime_TPS
};
int PinCommandOpcode[] = {
  NetPin_ModeGet, NetPin_ModeSet
};
int DIORawCommandOpcode[] = {
  NetDIORaw_DirSet, NetDIORaw_DataSet, NetDIORaw_DirGet, 
  NetDIORaw_DataGet, NetDIORaw_Count
};
int DIOCommandOpcode[] = {
  NetDIO_Refresh, NetDIO_Commit, NetDIO_Set, NetDIO_Get, 
  NetDIO_SetAsync, NetDIO_GetAsync, NetDIO_Wait, NetDIO_Count,
  NetDIO_Capabilities
};
int TWICommandOpcode[] = {
  NetTWI_Write, NetTWI_Read
};

int CANCommandOpcode[] = {
  NetCAN_Rx, NetCAN_Tx, NetCAN_BaudSet, NetCAN_BaudGet
};

int SPICommandOpcode[] = {
  NetSPI_Write, NetSPI_Read, NetSPI_ReadWrite,
  NetSPI_ClockSet, NetSPI_EdgeSet
};

int SystemCommandOpts[] = {
  0,1,1,0,0,1,1,0,0,0,0,1,1,2,2,1
};

int BusCommandOpts[] = {
  1,2,1,2,1,2,2,3,2,2,2,3,2,2,2,3,2,2,3,3,0,1,2,2,2,4,4,4,3,3,3
};
int TimeCommandOpts[] = {
  1,1,0,1,2,2,0
};
int PinCommandOpts[] = {
  1,2
};
int DIORawCommandOpts[] = {
  2,2,1,1,0
};
int DIOCommandOpts[] = {
  0,1,2,1,2,1,3,0,1
};
int TWICommandOpts[] = {
  4,4
};
int CANCommandOpts[] = {
  0,1,1,0
};
int SPICommandOpts[] = {
  2,2,2,1,1
};

char *SystemCommandDesc[] = {
  "", // ClassCount
  "<class#>", // InstanceCount
  "<class#>", // APICount
  "", //LockCount
  "", //LockHolderInfo
  "<Max>", //ConnWaitInfo
  "<CANInst>", // CANBusGet
  "", // BuildInfo
  "", // ModelId
  "" // BaseBoardId
  "", // MapLength
  "<index>", // MapGet
  "<name>", // MapLookup
  "<name> <value>", // MapLookupPartial
  "<name> <value>", //MapAdd
  "<name>" // MapDelete
};

char *BusCommandDesc[] = {
  "<address>", // Peek8
  "<address> <8-bit value>", // Poke8
  "<address>", // Peek16
  "<address> <16-bit value>", // Poke16
  "<address>", // Peek32
  "<address> <32-bit value>", // Poke32
  "<address> <bit>", // BitGet8
  "<address> <bit> <0|1>", // BitAssign8
  "<address> <bit>", // BitSet8
  "<address> <bit>", // BitClear8
  "<address> <bit>", // BitGet16
  "<address> <bit> <0|1>", // BitAssign16
  "<address> <bit>", // BitSet16
  "<address> <bit>", // BitClear16
  "<address> <bit>", // BitGet32
  "<address> <bit> <0|1>", // BitAssign32
  "<address> <bit>", // BitSet32
  "<address> <bit>", // BitClear32
  "<address> <direction> <length>", // PeekStream
  "<address> <direction> <byte array>", // PokeStream
  "", // Refresh
  "<forceall flag>", // Commit
  "<address> <bit>", // BitToggle8
  "<address> <bit>", // BitToggle16
  "<address> <bit>", // BitToggle32
  "<address> <bit1> <bit0> <value>", // Assign8X
  "<address> <bit1> <bit0> <value>", // Assign16X
  "<address> <bit1> <bit0> <value>", // Assign32X
  "<address> <bit1> <bit0>", // BitsGet8
  "<address> <bit1> <bit0>", // BitsGet16
  "<address> <bit1> <bit0>" // BitsGet32
};
char *TimeCommandDesc[] = {
  "<microseconds>", // Wait
  "<microseconds>", // Delay
  "", // Tick
  "<start>", // usElapsed
  "<start> <us>", // usFuture
  "<start> <end>", // TimeoutQ
  "" // TPS
};
char *PinCommandDesc[] = {
  "<pin number>", // ModeGet
  "<pin number> <mode>", // ModeSet
};
char *DIORawCommandDesc[] = {
  "<num> <asOutput>", // DirSet
  "<num> <asHigh>", // DataSet
  "<num>", // DirGet
  "<num>", // DataGet
  "" // Count
};
char *DIOCommandDesc[] = {
  "", // Refresh
  "<forceall flag>", // Commit
  "<DIO number> <state>", // Set
  "<DIO number>", // Get
  "<DIO number> <state>", // SetAsync
  "<DIO number>", // GetAsync
  "<min> <max> <DIONum:...> <DIONum:...>", // Wait
  "", // Count
  "<DIO number>" // Capabilities
};
char *TWICommandDesc[] = {
  "<devadr> <adrslen> <adrs> <byte array>", // Write
  "<devadr> <adrslen> <adrs> <length>" // Read
};
char *CANCommandDesc[] = {
  "", // Rx
  "<byte array>", // Tx
  "<baudrate>", // BaudSet
  "", // BaudGet
};
char *SPICommandDesc[] = {
  "<adrs> <byte array>", // Write
  "<adrs> <length>", // Read
  "<adrs> <byte array>", // ReadWrite
  "<clock freq>", // ClockSet
  "<edge>" // EdgeSet
};

char **CommandNames[] = {
  SystemCommandNames, BusCommandNames, TimeCommandNames, PinCommandNames, 
  DIORawCommandNames, DIOCommandNames, TWICommandNames, CANCommandNames, 
  SPICommandNames, 0
};
int *CommandOpts[] = {
  SystemCommandOpts, BusCommandOpts, TimeCommandOpts, PinCommandOpts, 
  DIORawCommandOpts, DIOCommandOpts, TWICommandOpts, CANCommandOpts,
  SPICommandOpts
};
char **CommandDesc[] = {
  SystemCommandDesc, BusCommandDesc, TimeCommandDesc, PinCommandDesc,
  DIORawCommandDesc, DIOCommandDesc, TWICommandDesc, CANCommandDesc,
  SPICommandDesc
};
int *CommandOpcode[] = {
  SystemCommandOpcode, BusCommandOpcode, TimeCommandOpcode, PinCommandOpcode,
  DIORawCommandOpcode, DIOCommandOpcode, TWICommandOpcode, CANCommandOpcode,
  SPICommandOpcode
};

struct opt_numlist {
  int opt_numlist;
  int len;
  unsigned char dat[256],*next;
};

static int parse_numlist(char *s, unsigned char *dat, unsigned char **next) {
	unsigned int val, len;
	char *e;

	len = 0;
	while (*s && *s != ' ') {
		val = strtoul(s, &e, 16);
		if (s == e) return -1;
		else if (ispunct(*e)) s = e + 1; 
		else s = e;
		*dat++ = val;
		len++;
		if (len == 256) break;
	}
	*next = s;
	return len;	
}

int OptionNumList(char *arg,void *target0,int opt) {
  struct opt_numlist *target = target0;

  target->opt_numlist = 1;
  target->len = parse_numlist(arg, target->dat, &target->next);
  if (target->len == -1) {
    fprintf(stderr, "error parsing numeric list\n");
    return 0;
  }
  return target->len;
}

int OptionCAN(char *arg,void *target0,int opt) {
  CANMessage *msg = target0;
  char *s = arg,*lasts;
  struct opt_numlist nl;

  msg->flags = FLAG_EXT_ID;
  msg->id = 0;
  msg->timestamp.tv_sec = 0;
  msg->timestamp.tv_usec = 0;
  msg->length = 0;
  while (*s) {
    lasts = s;
    while (*arg && *arg != ' ') arg++;
    if (s[0] == 'E' && (s[1] == 0||s[1] == ' ')) {
      msg->flags |= FLAG_EXT_ID;
      s++;
    } else if (s[0] == 'S' && (s[1] == 0||s[1] == ' ')) {
      msg->flags &= ~FLAG_EXT_ID;
      s++;
    } else if (s[0] == 'R' && s[1] == 'T' && s[2] == 'R' && (s[3] == 0||s[1] == ' ')) {
      msg->flags |= FLAG_RTR;
      s += 3;
    } else if (strchr(s,' ') > strchr(s,':')
	       || strchr(s,' ') == 0 && strchr(s,':') != 0
	       || msg->id != 0) {
      int i;
      OptionNumList(s,&nl,0);
      arg = s = nl.next;
      if (nl.len > 8) {
	nl.len = 8;
      }
      for (i=0;i<nl.len;i++) msg->data[i] = nl.dat[i];
      msg->length = nl.len;
    } else {
      msg->id = strtoul(s, &arg, 0);
      s=arg;
    }
    if (*s == ' ') s++;
    if (s == lasts) {
      fprintf(stderr,"Invalid CAN Tx argument:\n");
      fprintf(stderr,"\
Tx expect 1-4 arguments in any order:\n\
D1:D2:D3:D4:D5:D6:D7 one to seven data bytes, separated by colons\n\
E|S        extended or standard ID (default is extended)\n\
RTR        RTR flag\n\
ID         integer, default is 0\n\
D1[:...]   numlist containing data bytes\n\
\n\
All these arguments must appear in a single shell argument,\n\
so it is usually necessary to quote them, e.g.:\n\
    tsctl CAN 0 Tx \"0x1234 1:2:3\"\n");
      exit(1);
    }
  }
}

struct diowaitlist {
  int n;
  int *dio;
};

int OptionDIOWaitList(char *arg,void *target,int opt) {
  struct diowaitlist *wl = target;
  char *s0=arg,*s = arg;
  int i = 0,last=0;

  wl->n = 0;
  // case 0:
  // empty (no DIO number, no colon)
  // case 1:
  // single DIO number (no colon)
  // case 2:
  // two DIO numbers (1 colon)
  // ...
  while (*s) {
    if (*s == ':') i++;
    s++;
  }
  s = s0;
  wl->dio = malloc(sizeof(int)*(1+i));
  i = 0;
  while (*s) {
    while (*s && *s != ':') {
      s++;
    }
    if (!*s) last=1;
    *s=0;
    if (isdigit(*s0)) {
      wl->dio[i++] = strtol(s0,0,0);
      wl->n++;
    }
    if (last) return 1;
    s0 = s = s+1;
  }
}

// -------- CLIENT data
static NetSystem Net_system[16];
int NetSystemSocket[16];
int NetSystemInst[16];

static NetBus Net_bus[16];
int NetBusSocket[16];
int NetBusInst[16];

static NetTime Net_t[16];
int NetTimeSocket[16];
int NetTimeInst[16];

static NetPin Net_pin[16];
int NetPinSocket[16];
int NetPinInst[16];

static NetDIORaw Net_dioraw[16];
int NetDIORawSocket[16];
int NetDIORawInst[16];

static NetDIO Net_dio[16];
int NetDIOSocket[16];
int NetDIOInst[16];

static NetTWI Net_twi[16];
int NetTWISocket[16];
int NetTWIInst[16];

static NetSPI Net_spi[16];
int NetSPISocket[16];
int NetSPIInst[16];

static NetCAN Net_can[16];
int NetCANSocket[16];
int NetCANInst[16];

int NetMode = NetModeBlocking;
Packet q;
int cmds;

Lock *ClientLockInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return 0;
}

DIORaw *ClientDIORawInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetDIORawInit(&Net_dioraw[inst],NetDIORawSocket[inst],NetDIORawInst[inst],NetMode,&q,&cmds);
}

DIO *ClientDIOInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetDIOInit(&Net_dio[inst],NetDIOSocket[inst],NetDIOInst[inst],NetMode,&q,&cmds);
}

System *ClientSystemInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetSystemInit(&Net_system[inst],NetSystemSocket[inst],NetSystemInst[inst],NetMode,&q,&cmds);
}

Bus *ClientBusInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetBusInit(&Net_bus[inst],NetBusSocket[inst],NetBusInst[inst],NetMode,&q,&cmds);
}

Time *ClientTimeInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetTimeInit(&Net_t[inst],NetTimeSocket[inst],NetTimeInst[inst],NetMode,&q,&cmds);
}

Pin *ClientPinInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetPinInit(&Net_pin[inst],NetPinSocket[inst],NetPinInst[inst],NetMode,&q,&cmds);
}

TWI *ClientTWIInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetTWIInit(&Net_twi[inst],NetTWISocket[inst],NetTWIInst[inst],NetMode,&q,&cmds);
}

SPI *ClientSPIInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetSPIInit(&Net_spi[inst],NetSPISocket[inst],NetSPIInst[inst],NetMode,&q,&cmds);
}

CAN *ClientCANInit(int inst) {
  if (inst < 0 || inst > 15) return 0;
  return NetCANInit(&Net_can[inst],NetCANSocket[inst],NetCANInst[inst],NetMode,&q,&cmds);
}

void semdump() {
  union semun se;
  struct semid_ds ds;
  key_t semkey = 0x75000000;
  int i,semid;

  semid = SemaphoresGet(semkey,1);

  se.buf = &ds;
  i = semctl(semid,0,IPC_STAT,se);
  if (i < 0) {
    perror("semctl(1)");
    return;
  }
  {
    unsigned short arr[ds.sem_nsems];

    se.array = arr;
    i = semctl(semid, 0, GETALL,se);
    if (i < 0) { 
      perror("semctl(2)");
      return;
    }
    for (i=0;i<ds.sem_nsems;i++) {
      printf("%d ",arr[i]);
    }
    printf("\n");
  }
}

struct opt_txdata {
  int opt_txdat;
  int txdat_len;
  unsigned char txdat[8];
};

static int parse_txdat(char *s, unsigned char *txdat) {
	unsigned int val, len;
	char *e;

	len = 0;
	while (*s) {
		val = strtoul(s, &e, 0);
		if (s == e) return -1;
		else if (ispunct(*e)) s = e + 1; 
		else s = e;
		*txdat++ = val;
		len++;
		if (len == 8) break;
	}

	return len;	
}

int OptionOctets(char *arg,void *target0,int opt) {
  struct opt_txdata *target = target0;

  target->opt_txdat = 0;
  target->txdat_len = parse_txdat(optarg, target->txdat);
  if (target->txdat_len == -1) {
    fprintf(stderr, "txdat parse error\n");
    return 0;
  }
  target->opt_txdat = 1;
  return 1;
}

void CANMessageTxInit(CANMessage *msg,unsigned id,int flags,
		      int length,unsigned char *data) {
  memset(msg,0,sizeof(*msg));
  msg->flags = flags;
  msg->id = id;
  gettimeofday(&msg->timestamp,NULL);
  msg->length = length;
  memcpy(msg->data,data,length);
}

static volatile int alarmed = 0;
CAN *CANTestRxObject = 0;
static void alarmsig(int x) {
  alarmed = 1;
  if (CANTestRxObject) {
    CANTestRxObject->API->Fini(CANTestRxObject);
    // force-close the connection to any pending recv() will terminate
  }
}

int CANTestRx(CAN *can,int verbose) {
  struct sigaction sa;
  struct timeval tv1, tv2;
  unsigned char tmp[16];
  unsigned skipped = 0;
  int idseq = -1, err = 1, i, j, k, n;
  int corrupts = 0, badlen = 0;
  struct sched_param sched;
  struct rusage ru1, ru2;
  CANMessage msg[1];

  CANTestRxObject = can;
  nice(-20);
  sched.sched_priority = 50;
  sched_setscheduler(0, SCHED_FIFO, &sched);
  
  sa.sa_handler = alarmsig;
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = 0;
  sigaction(SIGALRM, &sa, NULL);
  if (getenv("TS_PRODUCTION") == NULL) alarm(10);
  else alarm(2);
  gettimeofday(&tv1, NULL);
  getrusage(RUSAGE_SELF, &ru1);
  i = 0;
  while (!alarmed) {
    if ((can->API->Rx(can,msg)) <= 0) continue;
    for (j=0;j<1;j++) {
      if (msg[j].flags & FLAG_BUS_ERROR) printf("errormsg=\"*** bus error\"\n");
      if (msg[j].flags & FLAG_ERROR_PASSIVE) printf("errormsg=\"*** error passive\"\n");
      if (msg[j].flags & FLAG_DATA_OVERRUN) printf("errormsg=\"*** data overrun error\"\n");
      if (msg[j].flags & FLAG_ERROR_WARNING) printf("errormsg=\"*** error warning\"\n");
      if (msg[j].length) {
	i++;
	if (idseq == -1) {
	  err = 0;
	  idseq = msg[j].id;
	} else {
	  int nid = (idseq + 1) & 0x1fffffff;
	  if (msg[j].id != nid) {
	    if (skipped > 0) {
	      if (verbose) fprintf(stderr, 
		      "errormsg=\"skipped %d\"\n",msg[j].id-nid);
	      //		    "ID 0x%x != 0x%x\"\n",
	      //		    msg[j].id, nid);
	      skipped += (msg[j].id-nid);
	    } else skipped = 1;
	    err++;
	  }
	  if (msg[j].length > 8 || msg[j].length < 0) {
	    if (verbose) fprintf(stderr,"errormsg=\"bad length %d\"\n",msg[j].length);
	    badlen++;
	  } else  for (k = 0; k < msg[j].length; k++) 
	    if (msg[j].data[k] != (((msg[j].id<<3)+k+1)&0xff)) {
	      if (verbose) fprintf(stderr,
		      "errormsg=\""
		      "data[%d] 0x%x != 0x%x\"\n", 
		      k, msg[j].data[k], 
		      (((msg[j].id<<3)+k+1)&0xff));
	      corrupts++;
	    }
	  idseq = msg[j].id;
	}
      }
    }
  }
  /* count packets in i */
  getrusage(RUSAGE_SELF, &ru2);
  gettimeofday(&tv2, NULL);
  j = (tv2.tv_sec - tv1.tv_sec) * 1000;
  j += tv2.tv_usec / 1000 - tv1.tv_usec / 1000;
  printf("total_packets=%d\n",i);
  printf("skipped_packets=%d\n",skipped?skipped-1:skipped);
  printf("bad_length=%d\n",badlen);
  printf("packets_per_sec=%d\n", (i * 1000) / j);
  i = (ru2.ru_utime.tv_sec + ru2.ru_stime.tv_sec) * 1000;
  i += ru2.ru_utime.tv_usec / 1000 + ru2.ru_stime.tv_usec / 1000;
  i -= (ru1.ru_utime.tv_sec + ru1.ru_utime.tv_sec) * 1000;
  i -= ru1.ru_utime.tv_usec / 1000 + ru1.ru_stime.tv_usec / 1000;
  printf("cpu_utilization_pct=%d\n", i * 100 / j);
  printf("test_sequence_errors=%d\n", err);
  printf("test_corruption_errors=%d\n", corrupts);
  return 0;
}

char *parse_hex_octets(char *buf,int *n) {
  char *ret,*s;
  int i,len = strlen(buf),count=0;
  *n = 0;
  for (i=0;i<len;i++) {
    if (buf[i] == ':') {
      count++;
    }
  }
  ret = malloc(2+count);
  while (buf) {
    s = strchr(buf,':');
    if (s) {
      *s++ = 0;
    }
    ret[n[0]] = strtoul(buf,0,16);
    n[0]++;
    buf = s;
  }
  return ret;
}

int OptSPIClient=-1,LastOptSPIClient=-1;
int OptionSPISeq(char *arg,void *target0,int opt) {
  unsigned *target = target0;
  char *buf;
  int i,n;
  static int adrs=1;
  static SPI *spi=0;
  static NetSPIctl nspi;

  if (OptSPIClient != LastOptSPIClient) {
    LastOptSPIClient = OptSPIClient;
    spi = NetSPIctlInit(&nspi,OptSPIClient,0);
  }
  if (!spi) {
    printf("Error: must specify port before SPI command\n");
    return 1;
  }

  target[0]++;
  switch(opt) {
  case 'c':
    i = atoi(arg);
    spi->API->ClockSet(spi,i);
    break;
  case 'e':
    i = atoi(arg);
    spi->API->EdgeSet(spi,i);
    break;
  case 'w':
    buf = parse_hex_octets(arg,&n);
    spi->API->Write(spi,adrs,buf,n);
    break;
  case 'd':
    buf = parse_hex_octets(arg,&n);
    {
      unsigned char buf2[n];
      spi->API->ReadWrite(spi,adrs,buf,buf2,n);
      for (i=0;i<n;i++) {
	printf("%02X%s",buf2[i],i+1<n?":":"");
      }
      printf("\n");
    }
    break;
  case 'r':
    n = atoi(arg);
    {
      unsigned char buf2[n];
      spi->API->Read(spi,adrs,buf2,n);
      for (i=0;i<n;i++) {
	printf("%02X%s",buf2[i],i+1<n?":":"");
      }
      printf("\n");
    }
    break;
  case 'l':
    if (adrs > 0) {
      adrs = 1+atoi(arg);
    } else {
      adrs = -1-atoi(arg);
    }
    break;
  case 'o':
    if (adrs > 0) adrs = -adrs;
    break;
  }
}

int spictl(int argc,char *argv[]) {
  int OptDoSeq=0, OptServer=0;
  struct option2 opts[] = {
    { 1, OptionSPISeq,  &OptDoSeq  ,"<c>lock", "frequency    SPI clock frequency" },
    { 1, OptionSPISeq,  &OptDoSeq  ,"<e>dge", "value         set clock edge (positive for > 0, negative for < 0)" },
    { 1, OptionSPISeq , &OptDoSeq , "<w>ritestream", "data   write colon delimited hex octets to SPI" },
    { 1, OptionSPISeq , &OptDoSeq , "rea<d>write", "data     write colon delimited hex octets to SPI while reading to stdout" },
    { 1, OptionSPISeq , &OptDoSeq , "<r>eadstream", "bytes   read specified number of bytes from SPI to stdout" },
    { 0, OptionBoolean, &OptDoSeq ,"h<o>ldcs", "            don't de-assert CS# when done" },
    { 1, OptionSPISeq,  &OptDoSeq  ,"<l>un", "id             Talk to specified chip number" },
    { 2, OptionUnsigned,&OptServer ,"<s>erver", "<port>      Daemonize and run as server listening on port" },
    { 1, OptionHost ,&OptSPIClient, "<p>ort", "<host><:port> Talk to spictl server" },
    { 0,0,0,"Technologic Systems SPI controller manipulation.\n\nGeneral options:\n",
      "hex octets are hexadecimal bytes. for example,\n"
      "this command reads 32 bytes of CS#1 SPI flash from address 8192:\n"
      "./spictl -l 1 -w 0B:00:20:00:00 -r 32\n"
    }
  };
  OptionHostSocketCreate = ClientSocketNew;
  OptionHostPortDefault = 5501;
  OptionsParse(argc,argv,opts);

  if (OptServer) {
    int tmp = ClientSocketNew("127.0.0.1",5001);
    char *s;
    if (tmp >= 0) {
      close(tmp);
      return 0; // already running
    }
    s = argv[0];
    while (*s) s++;
    s-= 6;
    s[0] = 't';
    s[1] = 's';
    s[2] = 'c';
    s[3] = 't';
    s[4] = 'l';
    s[5] = 0;
    return server();
  }

}

int canctl(int argc,char *argv[]) {
  int OptInst;
  unsigned OptAdrs=-1,OptBaud=0,OptPeek=-1,OptPoke=-1,OptTxId=0,OptRTR=0;
  unsigned OptServer=0,OptDump=0,OptBTR0=-1,OptBTR1=-1;
  unsigned OptTxTest=0,OptRxTest=0,OptExt=1,OptVerbose=0;
  int OptClient=-1;
  unsigned OptGoody=0;
  struct opt_txdata OptTxD = { .opt_txdat = 0 };
  CAN *can;

  struct option2 opts[] = {
    { 1, OptionUnsigned,&OptAdrs  ,"<a>ddress" , "ADR        CAN register address for write" },
    { 1, OptionUnsigned,&OptBaud  ,"<b>aud" , "BAUD          CAN baud rate (7500 to 1000000)" },
    { 1, OptionUnsigned ,&OptPeek  ,"peek8<R>" , "ADR         CAN register read" },
    { 1, OptionUnsigned,&OptPoke  ,"poke8<W>" , "VAL         CAN register write" },
    { 1, OptionUnsigned,&OptTxId  ,"tx<i>d" , "ID            CAN TX packet ID" },
    { 0, OptionBoolean ,&OptRTR   ,"txr<T>r" , "             TX RTR packet" },
    { 1, OptionOctets  ,&OptTxD   ,"tx<d>at" , "DAT          TX packet with data DAT" },
    { 2, OptionUnsigned,&OptServer,"<s>erver" , "=<port>     Daemonize, run as server (default port=7552+instance)" },
    { 0, OptionBoolean ,&OptDump  ,"<D>ump" , "              Receive and print all CAN packets" },
    { 1, OptionUnsigned,&OptBTR0  ,"btr<0>" , "BTR0          SJA1000 BTR0 bus timing reg val" },
    { 1, OptionUnsigned,&OptBTR1  ,"btr<1>" , "BTR1          SJA1000 BTR1 bus timing reg val" },
    { 0, OptionBoolean ,&OptTxTest,"<t>xtest" , "            Send TX test pattern (until CTRL+C pressed)" },
    { 0, OptionBoolean ,&OptRxTest,"<r>xtest" , "            Do RX test" },
    { 2, OptionHost    ,&OptClient,"<p>ort" , "<host><:port> Talk to canctl server" },
    { 1, OptionUnsigned ,&OptInst ,"<I>nstance" , "ADR       CAN instance (0,1,etc.)" },
    { 0, OptionBoolean0,&OptExt   ,"<S>td" , "               Send standard frame (not extended)" },
    //{ 0, OptionBoolean ,&OptRecover,"reco<v>er","             Automatically recover from bus-off" },
    { 0, OptionBoolean ,&OptVerbose,"<V>erbose" , "           verbose error messages during rxtest" },
    { 0, OptionBoolean ,&OptGoody ,"<G>oody" , "            Run CAN bus stress test" },
    { 0,0,0,"Technologic Systems CAN controller manipulation.\n","" }
  };
  pthread_t th1,th2,th0;
  int wait1=0,wait2=0,terminateWhenDone=0;

  OptionHostSocketCreate = ClientSocketNew;
  OptionHostPortDefault = 7552;
  OptionsParse(argc,argv,opts);

  // OptAdrs, OptPeek, OptPoke, OptBTR0, OptBTR1 talk directly to CAN registers
  if ((OptAdrs != -1 && OptPoke != -1) || OptPeek != -1 
      || OptBTR0 != -1 || OptBTR1 != -1 || OptGoody) {
    Bus *bus;
    System *sys;
    int peekval,i;

    NetSystemSocket[0] = NetBusSocket[0] = ClientSocketNew("127.0.0.1",5001);
    if (NetBusSocket[0]  < 0) {
      // Start a single NetLoop thread
      struct sockaddr_in csa;
      unsigned size_csa = sizeof(csa);
      int s,listen;
      THREAD_SPECIFIC_DATA_INIT;

      pthread_key_create(&tdata, thread_destructor);
      pthread_setspecific(tdata,&thread_data);
      ArchInit0();

      efd = epoll_create(8);
      assert(efd != -1);
      pthread_create(&th0, NULL, WatchLoop, 0);

      listen = ServerSocketNew(5001);
      if (listen < 0) {
	fprintf(stderr,"ServerSocketNew(%d): %d: %m\n",5001,listen);
	return 1;
      }
      NetSystemSocket[0] = NetBusSocket[0] = ClientSocketNew("127.0.0.1",5001);
      fcntl(NetBusSocket[0],F_SETFL,fcntl(NetBusSocket[0],F_GETFL,0)&~O_NONBLOCK);
      assert(NetBusSocket[0] >= 0);
      s = accept(listen, (struct sockaddr *)&csa, &size_csa);
      assert(s >= 0);
      close(listen);
      pthread_create(&th1, NULL, NetLoop, (void *)s);
      wait1 = 1;   
    }
    NetSystemInst[0] = 0;
    sys = ClientSystemInit(0);
    NetBusInst[0] = sys->API->CANBusGet(sys,OptInst);
    bus = ClientBusInit(0);
    bus->API->Lock(bus,WaitLock,0);
    if (!bus) {
      printf("Error: Bus %d not defined for this architecture\n",NetBusInst[0]);
      return 1;
    }
    if (bus->InitStatus <= 0) {
      printf("Error %d\n",-bus->InitStatus);
      return 1;
    }

    if (OptBTR0 != -1) { // note: user must put CAN into reset mode first
      bus->API->Poke8(bus,6, OptBTR0);
    }
    if (OptBTR1 != -1) { // note: user must put CAN into reset mode first
      bus->API->Poke8(bus,7, OptBTR1);
    }
    if (OptPoke != -1) {
      if (OptAdrs > 255) {
	fprintf(stderr,"Warning: Ignoring poke out of range address 0x%02X\n",OptAdrs);
      } else {
	bus->API->Poke8(bus, OptAdrs, OptPoke);
      }
    }
    if (OptPeek != -1) {
      if (OptPeek > 255) {
	fprintf(stderr,"Warning: Ignoring peek out of range address 0x%02X\n",OptAdrs);
      } else {
	//bus->Lock(bus,0,LOCK_NOW);
	peekval = bus->API->Peek8(bus,OptPeek);
	//bus->Unlock(bus);
	printf("[%d]=0x%02X\n",OptPeek,peekval);
      }
    }
    if (OptGoody) {
      int j,k;
      fprintf(stderr,"Beginning CAN stress test\n");
      bus->API->Poke8(bus,0,1);
      for (i=17;i<24;i++) {
	bus->API->Poke8(bus,i,i);
      }
      for (j=0;j<100000000;j++) {
	for (i=17;i<24;i++) {
	  if ((k=bus->API->Peek8(bus,i)) != i) {
	    fprintf(stderr,"%d:%d\n",k,bus->API->Peek8(bus,i));
	  }
	}
      }
    }
    bus->API->Lock(bus,DoRelease,0);
    if (wait1) {
      close(NetBusSocket[0]);
      close(s);
      pthread_join(th1,0);
      pthread_kill(th0,1);
    }
  }
  if (OptServer) {
    int tmp = ClientSocketNew("127.0.0.1",5001);
    char *s;
    if (tmp >= 0) {
      close(tmp);
      return 0; // already running
    }
    s = argv[0];
    while (*s) s++;
    s-= 6;
    s[0] = 't';
    s[1] = 's';
    s[2] = 'c';
    s[3] = 't';
    s[4] = 'l';
    s[5] = 0;
    return server();
  }

  //can = ClientCANInit(0);
  NetCANctl ncan;
  if (OptClient < 0 && (OptTxD.opt_txdat || OptRTR || OptDump || OptTxTest || OptRxTest)) {
    int retries=100;
    OptClient = ClientSocketNew("127.0.0.1",7552+OptInst);
    if (OptClient < 0) {
      // start a single can_listener thread
      THREAD_SPECIFIC_DATA_INIT;

      pthread_key_create(&tdata, thread_destructor);
      pthread_setspecific(tdata,&thread_data);
      ArchInit0();

      efd = epoll_create(8);
      assert(efd != -1);
      pthread_create(&th0, NULL, WatchLoop, 0);
      pthread_create(&th2, NULL, can_listener, (void *)OptInst);
      while (OptClient == -1 && retries-- > 0) {
	sched_yield();
	OptClient = ClientSocketNew("127.0.0.1",7552+OptInst);
      }
      assert(OptClient >= 0);
      wait2 = 1;
      terminateWhenDone = 1;
    }
  }
  can = NetCANctlInit(&ncan,OptClient);
  if (OptBaud) {
    can->API->BaudSet(can,OptBaud);
  }

  if (OptTxD.opt_txdat || OptRTR) { // send a single packet
    CANMessage msg;
    //fprintf(stderr,"dbug:%X %X %X\n",OptExt,OptRTR,(OptExt?FLAG_EXT_ID:0)|(OptRTR?FLAG_RTR:0));
    CANMessageTxInit(&msg,OptTxId,(OptExt?FLAG_EXT_ID:0)|(OptRTR?FLAG_RTR:0),
		     OptTxD.txdat_len,OptTxD.txdat);
    can->API->Tx(can,&msg);
    msg.flags = FLAG_CONTROL;
    msg.length = 666; // terminate server
    if (terminateWhenDone) can->API->Tx(can,&msg);
  }
  gotSigInt = 0;
  if (SignalCapture(2,SigInt) < 0) {
    fprintf(stderr,"capture %d:%m\n",15);
  }
  if (OptDump) {
    CANMessage msg;

    while (!gotSigInt) {
      if (can->API->Rx(can,&msg) <= 0) {
	fprintf(stderr,"Rx error\n");
	return 1;
      } else {
	CANMessagePrint(&msg);
      }
    }
  }
  if (OptTxTest) {
    CANMessage msg;
    int i,j;

    memset(&msg,0,sizeof(msg));
    msg.length = 8;
    msg.flags |= (OptExt ? FLAG_EXT_ID : 0);
    gettimeofday(&msg.timestamp,NULL);
    for (i=0;!gotSigInt;i++) {
      msg.id = i;
      for (j=0;j<8;j++) msg.data[j] = (i << 3) + j + 1;
      if (can->API->Tx(can,&msg) < 0) {
	break;
      }
    }
  }
  if (OptRxTest) {
    CANMessage msg;
    CANTestRx(can,OptVerbose);
    msg.flags = FLAG_CONTROL;
    msg.length = 666; // terminate server
    if (terminateWhenDone) {
      OptClient = ClientSocketNew("127.0.0.1",7552+OptInst); // alarm closed the connection
      can->API->Tx(can,&msg);
    }
  }
  NetCANctlFini(&ncan);
  if (wait2) {
    pthread_join(th2,0);
    pthread_kill(th0,1);  
  }
  return 0;
}

// ==================================================================
// dioctl client
// ==================================================================
FILE *dioctlF;
int netDIONameLookup(FILE *f,char *name);

int getDioNumFromNameOrNumber(char *nameOrNumber) {
  int j = atoi(nameOrNumber);
	
  if (j != 0 || !strcmp(nameOrNumber,"0")) return j;

  // Since we are only using these functions from the command line,
  // the extra overhead of looking up individual names is probably
  // less than fetching a copy of the entire name/number lookup map.
  return netDIONameLookup(dioctlF,nameOrNumber);
}

char *parse_dec_octets(char *buf,int *n) {
  char *ret,*s;
  int i,len = strlen(buf),count=0;
  *n = 0;
  for (i=0;i<len;i++) {
    if (buf[i] == ':') {
      count++;
    }
  }
  ret = malloc(2+count);
  while (buf) {
    s = strchr(buf,':');
    if (s) {
      *s++ = 0;
    }
//    ret[n[0]] = strtoul(buf,0,10);
    ret[n[0]] = getDioNumFromNameOrNumber(buf);
    n[0]++;
    buf = s;
  }
  return ret;
}

#define TxByte(val) fputc(val,f)
#define RxByte(ch) { fflush(f); int ch2 = fgetc(f); if (ch2 < 0) return FAILURE; ch=ch2; }
#define RxData(buf,len,dest) { fflush(f); if (fread(buf,len,1,f) != 1) { dest; return FAILURE; } }

#define FAILURE -1
int netBoardModelGet(FILE *f) {
  int ret;
  unsigned char byte[4];
	
  TxByte(22); // Get Model Id
  RxByte(byte[0]);
  RxByte(byte[1]);
  RxByte(byte[2]);
  RxByte(byte[3]);
  ret = (((int)byte[0]) << 24) + (((int)byte[1]) << 16) + (((int)byte[2]) << 8) + ((int)byte[3]);
  return ret;
}

int netDIONameLookup(FILE *f,char *name) {
  int i=0,val,len;
	
  len = strlen(name);
  if (len > 255) return -1;
  TxByte(23); // look up logical DIO name
  TxByte(len);
  while (name[i]) {
    TxByte(name[i++]);
  }
  RxByte(i);
  RxByte(val);
  if (i == 0) {
    return 0x7FFFFFFF;
  } else {
    return val;
  }
}

int netDIOGet(FILE *f,int num) {
  int ch;

  TxByte(4); // Get Single DIO State
  TxByte(num);
  RxByte(ch);
  return ch >> 6;
}

int netDIOGroupGet(FILE *f,int num,void (*callback)(int,int)) {
  int i,numdios,len,byte;

  TxByte(10); // Get DIO Group States
  TxByte(num << 4);
  RxByte(numdios);
  len = 1 + (numdios / 4) + ((numdios % 4) ? 1 : 0);
  for (i=0;i<numdios;i++) {
    if (i % 4 == 0) {
      RxByte(byte);
    }
    callback(i,byte >> 6);
    byte = (byte << 2) & 0xFF;
  }
  return 0;
}

int netDIOWait(FILE *f,int dionum,int isHigh) {
  int n;
	
  if (isHigh) {
    TxByte(16); // BlockUntilDIOHigh
  } else {
    TxByte(15); // BlockUntilDIOLow
  }
  TxByte(dionum);
  RxByte(n);
  return (n!=dionum) ?  -1 : 0;
}

#undef FAILURE
#define FAILURE 0
char *netDIOLookup(FILE *f,char *name,int value) {
  int i=0,val,len;
  char *result;

  len = strlen(name);
  if (len > 255) return 0;
  TxByte(27); // look up logical DIO name
  TxByte(len);
  while (name[i]) {
    TxByte(name[i++]);
  }
  TxByte(value);
  RxByte(len);
  result = malloc(len+1);
  for (i=0;i<len;i++) {
    RxByte(result[i]);
  }
  result[i] = 0;
  return result;
}

int netDIONamesList(FILE *f,void (*callback)(char *,char)) {
  unsigned char b[2];
  char byte;
  unsigned short count;
  int i,j,len;

  TxByte(26); // enumerate all logical DIO names
  RxByte(b[0]);
  RxByte(b[1]);
  count = (256 * (int)b[0] + b[1]);
  
  for (i=0;i<count;i++) {
    RxByte(byte);
    RxByte(len);
    {
      char name[len+1];

      for (j=0;j<len;j++) {
	RxByte(name[j]);
      }
      name[j] = 0;
      callback(name,byte);
    }
  }
  return 1;	
}

#undef FAILURE
#define FAILURE

void netDIONameAssign(FILE *f,int dionum,char *name) {
  int i=0,len;
	
  len = strlen(name);
  if (len > 255) return;
  TxByte(24); // assign logical DIO name
  TxByte(dionum);
  TxByte(len);
  while (name[i]) {
    TxByte(name[i++]);
  }
  fflush(f); // since change is global
}

void netDIONameRemove(FILE *f,char *name) {
  int i=0,len;
	
  len = strlen(name);
  if (len > 255) return;
  TxByte(25); // assign logical DIO name
  TxByte(len);
  while (name[i]) {
    TxByte(name[i++]);
  }
  fflush(f); // since change is global
}

// callback(int groupIndex,int value)
void netDIOGroupWaitOld(FILE *f,int groupNum,int numdios,int *val,void (*callback)(int,int)) {
  int i,n,byte;
  TxByte(14); // block until any DIO in Group is value specified for it
  TxByte(groupNum << 4);
  for (i=0;i<numdios;i++) {
    byte |= ((val[i]?1:0) << 7-(i%8));
    if (i%8 == 7 || i+1==numdios) TxByte(byte);
  }
  RxByte(n);
  for (i=0;i<n;i++) {
    RxByte(byte);
    callback(byte,val[byte]);
  }
}
#undef FAILURE

void netDIOGroupDefine(FILE *f,int groupNum,int numDios,int *dios) {
  int i;

  if (groupNum < 0 || groupNum > 15) return;
  if (numDios < 0 || numDios > 16) return;

  TxByte(5); // Define DIO Group Mapping
  TxByte((groupNum << 4) + (numDios-1));
  for (i=0;i<numDios;i++) {
    TxByte(dios[i]);
  }
}

void netDIOGroupSetAs(FILE *f,int groupNum,int asOutput,int drive) {
  TxByte(asOutput ? (drive ? 7 : 8) : 6);
  TxByte(groupNum << 4);
  fflush(f);
}

void netDIOSet(FILE *f,int num,int asOutput,int drive) {
  TxByte(asOutput ? (drive ? 2 : 3) : 1);  // Set Single DIO State
  TxByte(num);
  fflush(f);
}

int netDIOSleep(FILE *f,unsigned us) {
  TxByte(200);
  TxByte(0xFF & (us >> 16));
  TxByte(0xFF & (us >> 8));
  TxByte(0xFF & us);
}

int OptionHost2(char *arg,void *target0,int opt) {
  int OptClient;
  FILE **f = target0;

  if (!OptionHost(arg,&OptClient,opt)) {
    printf("%s:%m\n",arg);
    return 0;
  }
  *f = fdopen(OptClient,"a+");
  if (!*f) {
    fprintf(stderr,"fdopen:%m\n");
    exit(1);
  }
  return 1;
}

int OptionDIOSeq(char *arg,void *target0,int opt) {
  unsigned *target = target0;
  int i,j;
  void printDIOvalue(int index,int value) {
    printf("diodirG%d=%d\n",index,(value&2)?1:0);
    printf("diovalG%d=%d\n",index,(value&1)?1:0);
  }

  if (!dioctlF) {
    fprintf(stderr,"must specify port first!\n");
    return 0;
  }
  switch (opt) {
  case '0':
    j = getDioNumFromNameOrNumber(arg);
    if (j >= 1000) {
      netDIOGroupSetAs(dioctlF,j-1000,1,0);
    } else {
      netDIOSet(dioctlF,j,1,0);
    }
    break;
  case '1':
    j = getDioNumFromNameOrNumber(arg);
    if (j >= 1000) {
      netDIOGroupSetAs(dioctlF,j-1000,1,1);
    } else {
      netDIOSet(dioctlF,j,1,1);
    }
    break;
  case 'D': {
    char *pos = strstr(arg,":");
    if (!pos) {
	fprintf(stderr,"invalid format for --Define option\n");
    } else {
      *pos = 0; pos++;
      j = atoi(pos);
      netDIONameAssign(dioctlF,j,arg);
    }
    break;
  }
  case 'd': {
    void printEntry(char *name,char val) {
      printf("%s=%u\n",name,(unsigned char)val);
    }
    netDIONamesList(dioctlF,printEntry);
    break;
  }
  case 'e':
    j = atoi(arg);
    netDIOSleep(dioctlF,j);
    break;
  case 'G': {
    char *buf;
    int *dios;
    buf = parse_dec_octets(arg,&j);
    dios = malloc(sizeof(int) * j);
    for (i=0;i<j-1;i++) {
      dios[i] = buf[i+1];
    }
    netDIOGroupDefine(dioctlF,buf[0],j-1,dios);
    break;
  }
  case 'i':
    j = getDioNumFromNameOrNumber(arg);
    if (j >= 1000) {
      netDIOGroupSetAs(dioctlF,j-1000,0,0);
    } else {
      netDIOSet(dioctlF,j,0,0);
    }
    break;
  case 'L': {
    char *pos = strstr(arg,":");
    if (!pos) {
      fprintf(stderr,"Lookup reqires <strmatch>:<value> format\n");
      break;
    }
    *pos = 0; pos++;
    j = atoi(pos);
    pos=netDIOLookup(dioctlF,arg,j);
    if (pos) {
      printf("dioname=%s\n",pos);
      free(pos);
    }
    break;
  }
  case 'l':
    j=netDIONameLookup(dioctlF,arg);
    if (j != -1 && j != 0x7FFFFFFF) printf("dionum=%d\n",j);
    break;
  case 'm':
    printf("modelid=%X\n",netBoardModelGet(dioctlF));
    break;
  case 'r':
    j = getDioNumFromNameOrNumber(arg);
    if (j >= 1000) {
      netDIOGroupGet(dioctlF,j-1000,printDIOvalue);
    } else {
      i = netDIOGet(dioctlF,j);
      if (i < 0) {
	printf("server=0\n");
      } else {
	printf("server=1\n");
	printf("diodir%d=%d\n",j,(i & 2)?1:0);
	printf("dioval%d=%d\n",j,(i & 1)?1:0);
      }
    }
    break;
  case 'U':
    netDIONameRemove(dioctlF,arg);
    break;
  case 'W': {
    char *pos = strstr(arg,":");
    if (!pos) {
	fprintf(stderr,"invalid format for --Wait option\n");
    } else {
	*pos = 0; pos++;
	i = getDioNumFromNameOrNumber(arg);
	j = atoi(pos);
	netDIOWait(dioctlF,i,j);
    }
    break;
  }
  case 'w': {
    int *vals;
    char *buf;
    void printDIOvalue1(int index,int value) {
      printf("diovalG%d=%d\n",index,(value&1)?1:0);
    }

    buf = parse_dec_octets(arg,&j);
    vals = malloc(sizeof(int) * j);
    for (i=0;i<j-1;i++) {
	vals[i] = buf[i+1];
    }
    netDIOGroupWaitOld(dioctlF,buf[0],j-1,vals,printDIOvalue1);
    break;
  }
  default:
    break;
  }
  return 1;
  // <g>etcaps -> dioctlCapabilitiesGet (not implemented?)
}

int OptionConfig(char *arg,void *target0,int opt) {
  char **target = target0;
}

int dioctl(int argc,char *argv[]) {
  int h,i,j,modelDIO[200];
  int OptServer = 0, OptDoSeq, OptTest=0, OptArch = 0, OptLog = 0;

  struct option2 opts[] = {
    { 0, OptionBoolean ,&OptArch    ,"<a>rch", "              display what arches this binary supports" },
    { 2, OptionUnsigned,&OptServer  ,"<s>erver", "<port>      Daemonize and run as server listening on port" },
    { 1, OptionUnsigned,&OptPollRate,"p<o>ll", "rate          milliseconds between DIO polls (0 = never poll)" },
    //{ 1, OptionConfig  ,&OptDoSeq   ,"<f>ile"  ,"board        read the config definitions for the specified board" },
    { 2, OptionHost2   ,&dioctlF    ,"<p>ort", "<host><:port> Talk to dioctl server" },
    //{ 0, OptionDIOSeq  ,&OptDoSeq   ,"<g>etcaps", "           Get DIO capabilities" },
    { 0, OptionDIOSeq  ,&OptDoSeq   ,"<m>odel", "             Get model id of board" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<l>ookup", "name        Get logical DIO number from DIO name" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<L>ookup", "name:value  Get full DIO name from partial match and value" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<D>efine", "name:DIOnum Associate new name with logical DIO number" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<U>ndef", "name         De-Associate name with logical DIO number" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<r>ead", "DIOnum        Read the state of the specified DIO" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<1>", "DIOnum           set the specified DIO to output and drive high" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<0>", "DIOnum           set the specified DIO to output and drive low" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<i>nput", "DIOnum       set the specified DIO to an input" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<G>roup", "num:val1...  set the specified DIO Group number to the list of DIOs" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<w>ait", "num:val1...   wait for any DIO in the specified group number to be the given value" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"<W>ait", "num:val       wait for the specified DIO number to read as the given value" },
    { 1, OptionDIOSeq  ,&OptDoSeq   ,"d<e>lay", "us           server-side sleep (WARNING: use only on servers that support this!)" },
    //{ 0, OptionBoolean ,&OptTest    ,"<t>est", "              test macro-controller with base board" },
    //{ 1, OptionUnsigned   ,&BusCyclesLog,"<B>us", "debuglevel     log bus cycle accesses to stderr" },
    { 0, OptionDIOSeq  ,&OptDoSeq   ,"<d>ump", "              dump a table of the current server's DIO mappings" },
    { 0,0,0,"Technologic Systems DIO controller manipulation.\n\nGeneral options:\n",
      "To specify a DIO group where a DIO number is expected, add 1000 to the group number.\nOtherwise, DIOnum can be specified by an absolute DIO number, a DIO name, or\nthe connector name.\n"
    }
  };
  OptionHostSocketCreate = ClientSocketNew;
  OptionHostPortDefault = 4500;
  OptionsParse(argc,argv,opts);

  // OptArch
  // tsctl only supports on arch per binary, so just get modelid
  if (OptArch) {
    if (ModelId == 0) {
      printf("supported=client\n");
    } else {
      printf("supported=0x%04X\n",ModelId);
    }
  }

  // OptTest
  // lift code from dioctl
  
  if (OptServer) {
    int tmp = ClientSocketNew("127.0.0.1",5001);
    char *s;
    if (tmp >= 0) {
      close(tmp);
      return 0; // already running
    }
    s = argv[0];
    while (*s) s++;
    s-= 6;
    s[0] = 't';
    s[1] = 's';
    s[2] = 'c';
    s[3] = 't';
    s[4] = 'l';
    s[5] = 0;
    return server();
  }
}

int BaseBoardIdGet() {
  int adrs[3],inbit,extrabit;
  int saveCN2_6,saveCN2_8,saveCN1_98,saveCN1_83,saveCN1_85;
  int i,value=0,tmp;

  DIO *dio = dios[0];

  if (!dio) return 0;
  adrs[0] = DIOMapLookup("CN2_6");
  if (adrs[0] < 0) return 0;
  adrs[1] = DIOMapLookup("CN2_8");
  if (adrs[1] < 0) return 0;
  adrs[2] = DIOMapLookup("CN1_98");
  if (adrs[2] < 0) return 0;
  inbit = DIOMapLookup("CN1_83");
  if (inbit < 0) return 0;
  extrabit = DIOMapLookup("CN1_85");
  if (extrabit < 0) return 0;
  dio->API->Lock(dio,WaitLock,adrs[0]);
  dio->API->Lock(dio,WaitLock,adrs[1]);
  dio->API->Lock(dio,WaitLock,adrs[2]);
  dio->API->Lock(dio,WaitLock,inbit);
  dio->API->Lock(dio,WaitLock,extrabit);
  saveCN2_6 = dio->API->GetAsync(dio,adrs[0]);
  saveCN2_8 = dio->API->GetAsync(dio,adrs[1]);
  saveCN1_98 = dio->API->GetAsync(dio,adrs[2]);
  saveCN1_83 = dio->API->GetAsync(dio,inbit);
  saveCN1_85 = dio->API->GetAsync(dio,extrabit);
  dio->API->SetAsync(dio,inbit,INPUT);
  dio->API->SetAsync(dio,extrabit,INPUT);
  for (i=7;i>=0;i--) {
    dio->API->SetAsync(dio,adrs[0],i&1?LOW:HIGH);
    dio->API->SetAsync(dio,adrs[1],i&2?LOW:HIGH);
    dio->API->SetAsync(dio,adrs[2],i&4?HIGH:LOW);
    value <<= 1;
    tmp = dio->API->GetAsync(dio,inbit);
    value |= (3+dio->API->GetAsync(dio,inbit));
  }
  dio->API->SetAsync(dio,adrs[0],saveCN2_6);
  dio->API->SetAsync(dio,adrs[1],saveCN2_8);
  dio->API->SetAsync(dio,adrs[2],saveCN1_98);
  dio->API->SetAsync(dio,inbit,saveCN1_83);
  dio->API->SetAsync(dio,extrabit,saveCN1_85);
  dio->API->Lock(dio,DoRelease,adrs[0]);
  dio->API->Lock(dio,DoRelease,adrs[1]);
  dio->API->Lock(dio,DoRelease,adrs[2]);
  dio->API->Lock(dio,DoRelease,inbit);
  dio->API->Lock(dio,DoRelease,extrabit);
  fprintf(stderr,"%X\n",value);
  value = value & 0x3F;
  switch (value) {
  case 0x30: return 0x9490;
  case 0x01: return 0x8395;
  case 0x02: return 0x8390;
  case 0x03: return 0x8510;
  case 0x04: return 0x8500;
  case 0x06: return 0x8160;
  case 0x07: return 0x8100;
  case 0x00: case 0x3F: return 0x8200;
  default: return 0;
  }
}

typedef struct tsctlCommand tsctlCommand;
enum TypeEnum {
  T_Void, T_Integer32, T_VectorInteger32, T_BuildInfo, T_VectorInteger8,
  T_Pair
};
struct tsctlCommand {
  int Class, Inst;
  int Command,OpCode;
  int Status;
  union {
    System *sys;
    Bus *bus;
    Time *time;
    DIORaw *dioraw;
    DIO *dio;
    TWI *twi;
    SPI *spi;
    CAN *can;
  };
  enum TypeEnum rType;
  union {
    int rInteger32;
    char *rVectorInteger8;
    int *rVectorInteger32;
    struct BuildInfo *rBuildInfo;
    struct {
      char *key;
      int value;
    } rPair;
  };
};
enum {
  CommandErrorClass=-100, CommandErrorArgCount, CommandErrorCommandBad,
  CommandErrorArgsBad, CommandErrorNoSuchObject, CommandErrorObjectInit
};


int CommandParse(int argc,char **argv,tsctlCommand *cmd,int canlock);
int CommandExec(tsctlCommand *cmd,char **argv);
unsigned U32(char *str) {
  unsigned u32;
  OptionUnsigned(str,&u32,0);
  return u32;
}

int State(char *str) {
  if (!strcasecmp(str,"INPUT")) {
    return INPUT;
  } else if (!strcasecmp(str,"HIGH")) {
    return HIGH;
  } else if (!strcasecmp(str,"LOW")) {
    return LOW;
  } else return U32(str);
}

int DIONum(char *str) {
  int number = 1;
  char *s = str;
  while (*s) {
    if (!isdigit(*s) && !isspace(*s)) break;
    s++;
  }
  if (*s) { // contains non-numeric, non-whitespace character
    return LocalSystemMapLookup(&LocalSystem,str);
  } else { // contains only numbers and whitespace
    return U32(str);
  }
}

int main(int argc,char *argv[]) {
  tsctlCommand cmd;
  int ret;
  char *sep,*s;

  startuptime = time(0);
  s = argv[0];
  while (*s) s++;
  if (s - argv[0] >= 6) {
    s -= 6;
    if (s[0] == 'c' && s[1] == 'a' && s[2] == 'n') {
      return canctl(argc,argv);
    }
    if (s[0] == 'd' && s[1] == 'i' && s[2] == 'o') {
      return dioctl(argc,argv);
    }
    if (s[0] == 's' && s[1] == 'p' && s[2] == 'i') {
      return spictl(argc,argv);
    }
  }
  // query semaphores for debugging purposes
  if (argc == 2 && !strcmp(argv[1],"-q")) {
    semdump();
    return 0;
  }  
  // check for server invocation
  // gak! originally we didn't support any extra args
  // we added one for debug and tacked it on
  // then another and now it's starting to get ugly!
  if (argc >= 2 && !strcmp(argv[1],"--server")) {
    if (argc >= 3 && (!strcmp(argv[2],"--build") 
		      || argc >= 4 && !strcmp(argv[3],"--build")
		      || argc >= 5 && !strcmp(argv[4],"--build"))) {
      printf("%s\n",build);
    }
    if (argc >= 3) {
      int flags=0;
      if (argc >= 3 && !strcmp(argv[2],"--log")) {
	flags = strtoul(argv[3],0,0);
      } else if (argc >= 4 && !strcmp(argv[3],"--log")) {
	flags = strtoul(argv[4],0,0);
      }
      if (flags) {
	LC[0].socket = 2;
	LC[0].flags = flags;
	logflags |= flags;
	logconns++;
      }
    }
    if (SignalCapture(15,SignalHandle) < 0) 
      fprintf(stderr,"capture %d:%m\n",15);
    return server();
  }
  // check for minimum parms for client invocation
  if (argc < 3) {
    fprintf(stderr,"Usage:\n");
    fprintf(stderr,"\ttsctl --server\n");
    fprintf(stderr,"\ttsctl <Host> <Class[:Instance]> <Command> [Args...]\n");
    return 1;
  }

  ret = CommandParse(argc,argv,&cmd,0);
  switch (ret) {
  case CommandErrorClass:
    fprintf(stderr,"Error: unrecognized Class\n");
    fprintf(stderr,"Select from ");
    for (cmd.Class=0;Class[cmd.Class];cmd.Class++) {
      fprintf(stderr,"%s",Class[cmd.Class]);
      if (Class[cmd.Class+1]) fprintf(stderr,", ");
    }
    fprintf(stderr,"\n");
    return 1;
  case CommandErrorArgCount:
    fprintf(stderr,"Valid commands for %s are:\n",argv[2]);
    for (cmd.Command=0;CommandNames[cmd.Class][cmd.Command];cmd.Command++) {
      fprintf(stderr,"%s ",CommandNames[cmd.Class][cmd.Command]);
    }
    fprintf(stderr,"\n");
    return 1;
  case CommandErrorCommandBad:
    fprintf(stderr,"Error: unrecognized Command \"%s\"\n",argv[3]);
    fprintf(stderr,"Select from ");
    for (cmd.Command=0;CommandNames[cmd.Class][cmd.Command];cmd.Command++) {
      fprintf(stderr,"%s",CommandNames[cmd.Class][cmd.Command]);
      if (CommandNames[cmd.Class][cmd.Command+1]) fprintf(stderr,", ");
    }
    fprintf(stderr,"\n");
    return 1;    
  case CommandErrorArgsBad:
    fprintf(stderr,"Usage: tsctl %s %s %s %s\n",argv[1],argv[2],argv[3],
	    CommandDesc[cmd.Class][cmd.Command]);
    return 1;
  }
  NetBusInst[0] = NetTimeInst[0] = NetDIORawInst[0] =
    NetDIOInst[0] = NetTWIInst[0] = NetCANInst[0] =
    NetSystemInst[0] = NetPinInst[0] = cmd.Inst;
  
  // Open connection to server
  NetBusSocket[0] = NetTimeSocket[0] = NetDIORawSocket[0] = 
    NetDIOSocket[0] = NetTWISocket[0] = NetCANSocket[0] =
    NetSystemSocket[0] = NetPinSocket[0] = NetSPISocket[0] = 
    ClientSocketNew(argv[1],5001);
  if (NetPinSocket[0] <= 0) {
    printf("%s:%m\n",argv[1]);
    return 1;
  }

  ret = CommandExec(&cmd,argv);
  switch (ret) {
  case CommandErrorNoSuchObject:
    printf("Error: %s %d not defined for this architecture\n",
	   Class[cmd.Class],cmd.Inst);
    return 1;
  case CommandErrorObjectInit:
    printf("Error %d\n",-cmd.Status);
    return 1;
  }

  // Parse options and perform command
  switch (cmd.Class) { // TO DO: translate from enum to int?
  case NetSystemClass: {
    System *sys = ClientSystemInit(cmd.Inst);
    int class,count,i;

    if (!sys) {
      printf("Error: System %d not defined for this architecture\n",cmd.Inst);
      return 1;
    }
    if (sys->InitStatus <= 0) {
      printf("Error %d\n",-sys->InitStatus);
      return 1;
    }
    switch (cmd.OpCode) {
    case NetSystem_ClassCount:
      count = sys->API->ClassCount(sys);
      printf("%d\n",count);
      break;
    case NetSystem_InstanceCount:
      OptionUnsigned(argv[4],&class,0);
      count = sys->API->InstanceCount(sys,class);
      printf("%d\n",count);
      break;
    case NetSystem_APICount:
      OptionUnsigned(argv[4],&class,0);
      count = sys->API->APICount(sys,class);
      printf("%d\n",count);
      break;
    case NetSystem_LockCount:
      count = sys->API->LockCount(sys);
      printf("%d\n",count);
      break;
    case NetSystem_LockHolderInfo:
      count = sys->API->LockCount(sys);
      {
	int i;
	LockHolderInfo info[count];
	count = sys->API->LockHolderInfo(sys,info);
	for (i=0;i<count;i++) {
	  if (info[i].pid != -1) {
	    printf("%d:%d:%d\n",i,info[i].pid,info[i].count);
	  }
	}
      }
      break;
    case NetSystem_ConnWaitInfo: {
      ConnectionWaitInfo info[1024];
      int i;
      count = sys->API->ConnWaitInfo(sys,1024,info);
      for (i=0;i<count;i++) {
	printf("%d:%d:%d\n",i,info[i].pid,info[i].wait);
      }
      break;
    }
    case NetSystem_CANBusGet:
      OptionUnsigned(argv[4],&i,0);
      i = sys->API->CANBusGet(sys,i);
      printf("%d\n",i);
      break;
    case NetSystem_BuildInfo: {
      struct BuildInfo bi;
      sys->API->BuildInfo(sys,&bi);
      printf("BUILD=%u\n",bi.buildNumber);
      printf("BUILDTIME=%d\n",bi.buildTime);
      printf("UPTIME=%u\n",bi.uptime);
      printf("BUILDHOST=%s\n",bi.hostname);
      printf("ARCH=%s\n",bi.arch);
      break;
    }
    case NetSystem_ModelId:
      printf("ModelId=%04X\n",sys->API->ModelId(sys));
      break;
    case NetSystem_BaseBoardId:
      printf("ModelId=%04X\n",sys->API->BaseBoardId(sys));
      break;
    case NetSystem_MapLength:
      printf("MapLength=%d\n",sys->API->MapLength(sys));
      break;
    case NetSystem_MapGet: {
      char str[257];
      int val,len;
      len=sys->API->MapGet(sys,atoi(argv[4]),str,&val);
      str[len]=0;
      printf("%s=%d\n",str,val);
      break;
    }
    case NetSystem_MapLookup: {
      int val;
      val = sys->API->MapLookup(sys,argv[4]);
      printf("%s=%d\n",argv[4],val);
      break;
    }
    case NetSystem_MapLookupPartial: {
      char str[257];
      int len,v=atoi(argv[5]);
      len=sys->API->MapLookupPartial(sys,argv[4],v,str);
      str[len]=0;
      printf("%s=%d\n",str,v);
      break;
    }
    case NetSystem_MapAdd: {
      int ret;
      ret=sys->API->MapAdd(sys,argv[4],atoi(argv[5]));
      printf("%d\n",ret);
      break;
    }
    case NetSystem_MapDelete: {
      int ret;
      ret=sys->API->MapDelete(sys,argv[4]);
      printf("%d\n",ret);
      break;
    }
    }
    break;
  }
    break;
  case NetBusClass: {
    int OptAdrs = 0, OptVal = 0, OptCmd = 0, OptBit = 0, OptForce, OptBit1=0;
    struct opt_numlist nl;
    Bus *bus = ClientBusInit(cmd.Inst);
  
    if (!bus) {
      printf("Error: Bus %d not defined for this architecture\n",cmd.Inst);
      return 1;
    }
    if (bus->InitStatus <= 0) {
      printf("Error %d\n",-bus->InitStatus);
      return 1;
    }
    if (cmd.OpCode != NetBus_Refresh 
	&& cmd.OpCode != NetBus_Commit) {
      OptionUnsigned(argv[4],&OptAdrs,0);
    }
    bus->API->Lock(bus,WaitLock,0);
    switch (cmd.OpCode) {
    case NetBus_BitGet8: // BitGet8 address bit
    case NetBus_BitGet16: // BitGet16 address bit
    case NetBus_BitGet32: // BitGet32 address bit
      OptionUnsigned(argv[5],&OptBit,0);
      break;
    case NetBus_Poke8: // Poke8 address value
    case NetBus_Poke16: // Poke32 address value
    case NetBus_Poke32: // Poke32 address value
      OptionUnsigned(argv[5],&OptVal,0);
      break;
    case NetBus_BitSet8: // BitSet8 address bit
    case NetBus_BitSet16: // BitSet16 address bit
    case NetBus_BitSet32: // BitSet32 address bit
    case NetBus_BitClear8: // BitClear8 address bit
    case NetBus_BitClear16: // BitClear16 address bit
    case NetBus_BitClear32: // BitClear32 address bit
    case NetBus_BitToggle8: // BitToggle8 address bit
    case NetBus_BitToggle16: // BitToggle16 address bit
    case NetBus_BitToggle32: // BitToggle32 address bit
      OptionUnsigned(argv[5],&OptBit,0);
      break;
    case NetBus_BitAssign8: // BitAssign8 address bit value
    case NetBus_BitAssign16: // BitAssign16 address bit value
    case NetBus_BitAssign32: // BitAssign32 address bit value
      OptionUnsigned(argv[5],&OptBit,0);
      OptionUnsigned(argv[6],&OptVal,0);
      break;
    case NetBus_Commit: // Commit force
      OptionUnsigned(argv[4],&OptForce,0);
      break;
    case NetBus_PokeStream: // PokeStream address dir data
      OptionUnsigned(argv[5],&OptVal,0);
      OptionNumList(argv[6],&nl,0);
      break;
    case NetBus_PeekStream: // PokeStream address dir length
      OptionUnsigned(argv[5],&OptVal,0);
      OptionUnsigned(argv[6],&OptBit,0);
      break;
    case NetBus_Assign8X: case NetBus_Assign16X: case NetBus_Assign32X:
      OptionUnsigned(argv[5],&OptBit1,0);
      OptionUnsigned(argv[6],&OptBit,0);
      OptionUnsigned(argv[7],&OptVal,0);
      break;
    case NetBus_BitsGet8: case NetBus_BitsGet16: case NetBus_BitsGet32:
      OptionUnsigned(argv[5],&OptBit1,0);
      OptionUnsigned(argv[6],&OptBit,0);
      break;
    }
    switch (cmd.OpCode) {
    case NetBus_Peek8:
      printf("0x%02X\n",bus->API->Peek8(bus,OptAdrs));
      break;
    case NetBus_Peek16:
      printf("0x%04X\n",bus->API->Peek16(bus,OptAdrs));
      break;
    case NetBus_Peek32:
      printf("0x%08X\n",bus->API->Peek32(bus,OptAdrs));
      break;
    case NetBus_BitGet8:
      printf("%d\n",bus->API->BitGet8(bus,OptAdrs,OptBit));
      break;
    case NetBus_BitGet16:
      printf("%d\n",bus->API->BitGet16(bus,OptAdrs,OptBit));
      break;
    case NetBus_BitGet32:
      printf("%d\n",bus->API->BitGet32(bus,OptAdrs,OptBit));
      break;
    case NetBus_Poke8: // Poke8 address value
      bus->API->Poke8(bus,OptAdrs,OptVal);
      break;
    case NetBus_Poke16: // Poke32 address value
      bus->API->Poke16(bus,OptAdrs,OptVal);
      break;
    case NetBus_Poke32: // Poke32 address value
      bus->API->Poke32(bus,OptAdrs,OptVal);
      break;
    case NetBus_BitSet8: // BitSet8 address bit
      bus->API->BitSet8(bus,OptAdrs,OptBit);
      break;
    case NetBus_BitSet16: // BitSet16 address bit
      bus->API->BitSet16(bus,OptAdrs,OptBit);
      break;
    case NetBus_BitSet32: // BitSet32 address bit
      bus->API->BitSet32(bus,OptAdrs,OptBit);
      break;
    case NetBus_BitClear8: // BitClear8 address bit
      bus->API->BitClear8(bus,OptAdrs,OptBit);
      break;
    case NetBus_BitClear16: // BitClear16 address bit
      bus->API->BitClear16(bus,OptAdrs,OptBit);
      break;
    case NetBus_BitClear32: // BitClear32 address bit
      bus->API->BitClear32(bus,OptAdrs,OptBit);
      break;
    case NetBus_BitAssign8: // BitAssign8 address bit value
      bus->API->BitAssign8(bus,OptAdrs,OptBit,OptVal);
      break;
    case NetBus_BitAssign16: // BitAssign16 address bit value
      bus->API->BitAssign16(bus,OptAdrs,OptBit,OptVal);
      break;
    case NetBus_BitAssign32: // BitAssign32 address bit value
      bus->API->BitAssign32(bus,OptAdrs,OptBit,OptVal);
      break;
    case NetBus_Refresh: // Refresh
      bus->API->Refresh(bus);
      break;
    case NetBus_Commit: // Commit
      bus->API->Commit(bus,OptForce);
      break;
    case NetBus_BitToggle8: // BitToggle8 address bit
      printf("%d\n",bus->API->BitToggle8(bus,OptAdrs,OptBit));
      break;
    case NetBus_BitToggle16: // BitToggle16 address bit
      printf("%d\n",bus->API->BitToggle16(bus,OptAdrs,OptBit));
      break;
    case NetBus_BitToggle32: // BitToggle32 address bit
      printf("%d\n",bus->API->BitToggle32(bus,OptAdrs,OptBit));
      break;
    case NetBus_Assign8X: 
      printf("0x%02X\n",bus->API->Assign8X(bus,OptAdrs,OptBit1,OptBit,OptVal));
      break;
    case NetBus_Assign16X: 
      printf("0x%04X\n",bus->API->Assign16X(bus,OptAdrs,OptBit1,OptBit,OptVal));
      break;
    case NetBus_Assign32X:
      printf("0x%08X\n",bus->API->Assign32X(bus,OptAdrs,OptBit1,OptBit,OptVal));
      break;
    case NetBus_BitsGet8: 
      printf("0x%02X\n",bus->API->BitsGet8(bus,OptAdrs,OptBit1,OptBit));
      break;
    case NetBus_BitsGet16: 
      printf("0x%04X\n",bus->API->BitsGet16(bus,OptAdrs,OptBit1,OptBit));
      break;
    case NetBus_BitsGet32:
      printf("0x%08X\n",bus->API->BitsGet32(bus,OptAdrs,OptBit1,OptBit));
      break;
    case NetBus_PokeStream: // PokeStream address dir data
      bus->API->PokeStream(bus,OptAdrs,OptVal,nl.dat,nl.len);
      break;
    case NetBus_PeekStream: { // PokeStream address dir length
      char buffer[OptBit+1];
      int i;
      for (i=0;i<OptBit;i++) {
	buffer[i] = 0x55;
      }
      bus->API->PeekStream(bus,OptAdrs,OptVal,buffer,OptBit);
      for (i=0;i<OptBit;i++) {
	printf("%02X",buffer[i]);
	if (i+1 < OptBit) printf(":");
      }
      printf("\n");
      break;
    }
    }
    if (cmd.OpCode != NetBus_Refresh 
	&& cmd.OpCode != NetBus_Commit) { 
      bus->API->Lock(bus,DoRelease,0);
    }
    bus->API->Fini(bus);
    break;
  } // NetBusClass
    break;
  case NetTimeClass: {
    int OptUS,start,end;
    Time *t = ClientTimeInit(cmd.Inst);
    if (t->InitStatus <= 0) {
      printf("Time Error %d\n",-t->InitStatus);
      return 1;
    }
    switch (cmd.OpCode) {
    case NetTime_Wait:
      OptionUnsigned(argv[4],&OptUS,0);
      printf("%u\n",t->API->Wait(t,OptUS));
      break;
    case NetTime_Delay:
      OptionUnsigned(argv[4],&OptUS,0);
      t->API->Delay(t,OptUS);
      break;
    case NetTime_Tick:
      printf("%u\n",t->API->Tick(t));
      break;
    case NetTime_TPS:
      printf("%u\n",t->API->TPS(t));
      break;
    case NetTime_usElapsed:
      OptionUnsigned(argv[4],&start,0);
      printf("%u\n",t->API->usElapsed(t,start));
      break;
    case NetTime_usFuture:
      OptionUnsigned(argv[4],&start,0);
      OptionUnsigned(argv[5],&OptUS,0);
      printf("%u\n",t->API->usFuture(t,start,OptUS));
      break;
    case NetTime_TimeoutQ:
      OptionUnsigned(argv[4],&start,0);
      OptionUnsigned(argv[5],&end,0);
      printf("%u\n",t->API->TimeoutQ(t,start,end));
      break;
    }
    t->API->Fini(t);
    break;
  } // NetTimeClass
    break;
  case NetPinClass: {
    Pin *pin = ClientPinInit(cmd.Inst);
    int OptMode,OptPinNum;
    if (pin->InitStatus <= 0) {
      printf("Pin Error %d\n",-pin->InitStatus);
      return 1;
    }
    OptionUnsigned(argv[4],&OptPinNum,0);
    switch (cmd.OpCode) {
    case NetPin_ModeSet:
      if (!strcmp(argv[5],"DIO")) {
	OptMode = MODE_DIO;
      } else if (!strcmp(argv[5],"CAN")) {
	OptMode = MODE_CAN;
      } else if (!strcmp(argv[5],"SPI")) {
	OptMode = MODE_SPI;
      } else if (!strcmp(argv[5],"ADC")) {
	OptMode = MODE_ADC;
      } else if (!strcmp(argv[5],"TWI")) {
	OptMode = MODE_TWI;
      } else if (!strcmp(argv[5],"UART")) {
	OptMode = MODE_UART;
      } else if (!strcmp(argv[5],"TS")) {
	OptMode = MODE_TS;
      } else
	OptionUnsigned(argv[5],&OptMode,0);
      pin->API->Lock(pin,WaitLock,OptPinNum);
      printf("%d\n",pin->API->ModeSet(pin,OptPinNum,OptMode));
      pin->API->Lock(pin,DoRelease,OptPinNum);
      break;
    case NetPin_ModeGet:
      printf("%s\n",PinModeString(pin->API->ModeGet(pin,OptPinNum)));
      break;
    }
    pin->API->Fini(pin);
    break;
  } // NetPinClass
    break;
  case NetDIORawClass: {
    DIORaw *dio = ClientDIORawInit(cmd.Inst);
    int OptNum, OptValue;
    OptionUnsigned(argv[4],&OptNum,0);
    switch (cmd.OpCode) {
    case NetDIORaw_DirSet:
      OptionUnsigned(argv[5],&OptValue,0);
      dio->API->Lock(dio,WaitLock,OptNum);
      dio->API->DirSet(dio,OptNum,OptValue);
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIORaw_DataSet:
      OptionUnsigned(argv[5],&OptValue,0);
      dio->API->Lock(dio,WaitLock,OptNum);
      dio->API->DataSet(dio,OptNum,OptValue);
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIORaw_DirGet:
      dio->API->Lock(dio,WaitLock,OptNum);
      printf("%d\n",dio->API->DirGet(dio,OptNum));
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIORaw_DataGet:
      dio->API->Lock(dio,WaitLock,OptNum);
      printf("%d\n",dio->API->DataGet(dio,OptNum));
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIORaw_Count:
      // This command does not require that the DIORaw object be locked
      printf("%d\n",dio->API->Count(dio));
      break;
    }
    dio->API->Fini(dio);
    break;
  } // NetDIORawClass
    break;
  case NetDIOClass: {
    DIO *dio = ClientDIOInit(cmd.Inst);
    int OptNum, OptState, OptForce, OptFlags;
    switch (cmd.OpCode) {
    case NetDIO_Refresh:
      dio->API->Refresh(dio);
      break;
    case NetDIO_Commit:
      OptionUnsigned(argv[4],&OptForce,0);
      dio->API->Commit(dio,OptForce);
      break;
    case NetDIO_Set:
      OptionUnsigned(argv[4],&OptNum,0);
      if (!strcasecmp(argv[5],"INPUT")) {
	OptState = INPUT;
      } else if (!strcasecmp(argv[5],"HIGH")) {
	OptState = HIGH;
      } else if (!strcasecmp(argv[5],"LOW")) {
	OptState = LOW;
      } else OptionUnsigned(argv[5],&OptState,0);
      dio->API->Lock(dio,WaitLock,OptNum);
      dio->API->Set(dio,OptNum,OptState);
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIO_Get:
      OptionUnsigned(argv[4],&OptNum,0);
      dio->API->Lock(dio,WaitLock,OptNum);
      printf("%s\n",DIOValueString(dio->API->Get(dio,OptNum)));
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIO_SetAsync:
      OptionUnsigned(argv[4],&OptNum,0);
      if (!strcasecmp(argv[5],"INPUT")) {
	OptState = INPUT;
      } else if (!strcasecmp(argv[5],"HIGH")) {
	OptState = HIGH;
      } else if (!strcasecmp(argv[5],"LOW")) {
	OptState = LOW;
      } else OptionUnsigned(argv[5],&OptState,0);
      dio->API->Lock(dio,WaitLock,OptNum);
      dio->API->SetAsync(dio,OptNum,OptState);
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIO_GetAsync:
      OptionUnsigned(argv[4],&OptNum,0);
      dio->API->Lock(dio,WaitLock,OptNum);
      printf("%s\n",DIOValueString(dio->API->GetAsync(dio,OptNum)));
      dio->API->Lock(dio,DoRelease,OptNum);
      break;
    case NetDIO_Wait: {
      int min,max,i,n;
      struct diowaitlist wlh,wll;

      OptionUnsigned(argv[4],&min,0);
      OptionUnsigned(argv[5],&max,0);
      OptionDIOWaitList(argv[6],&wlh,0);
      OptionDIOWaitList(argv[7],&wll,0);
      {
	int matches[wll.n+wlh.n];
	
	n = dio->API->Wait(dio,matches,min,max,
			   wlh.n,wlh.dio,wll.n,wll.dio);
	for (i=0;i<n-1;i++) {
	  printf("%d:",matches[i]);
	}
	printf("%d\n",matches[i]);
      }
      break;
    }
    case NetDIO_Count:
      printf("%d\n",dio->API->Count(dio));
      break;
    case NetDIO_Capabilities:
      OptionUnsigned(argv[4],&OptNum,0);
      printf("%u\n",dio->API->Capabilities(dio,OptNum));
      break;
    }
  } // NetDIOClass
    break;
  case NetSPIClass: {
    SPI *spi = ClientSPIInit(cmd.Inst);
    struct opt_numlist nl;
    int ret,len,i,adrs;
    
    if (!spi) {
      printf("Error: SPI %d not defined for this architecture\n",cmd.Inst);
      return 1;
    }
    if (spi->InitStatus <= 0) {
      printf("Error %d\n",-spi->InitStatus);
      return 1;
    }
    switch (cmd.OpCode) {
    case NetSPI_Write:
      OptionUnsigned(argv[4],&adrs,0);
      OptionNumList(argv[5],&nl,0);
      spi->API->Lock(spi,WaitLock,0);
      ret = spi->API->Write(spi,adrs,nl.dat,nl.len);
      spi->API->Lock(spi,DoRelease,0);
      printf("%d\n",ret);
      break;
    case NetSPI_Read:
      OptionUnsigned(argv[4],&adrs,0);
      OptionUnsigned(argv[5],&len,0);
      {
	unsigned char buf[len];
	spi->API->Lock(spi,WaitLock,0);
	ret = spi->API->Read(spi,adrs,buf,len);
	spi->API->Lock(spi,DoRelease,0);
	printf("%d\n",ret);
	if (ret > 0) for (i=0;i<len;i++) {
	  printf("%02X",buf[i]);
	  if (i+1 < len) printf(":");
	}
	printf("\n");
      }
      break;
    case NetSPI_ReadWrite:
      OptionUnsigned(argv[4],&adrs,0);
      OptionNumList(argv[5],&nl,0);
      {
	unsigned char rbuf[nl.len];
	spi->API->Lock(spi,WaitLock,0);
	ret = spi->API->ReadWrite(spi,adrs,nl.dat,rbuf,nl.len);
	spi->API->Lock(spi,DoRelease,0);
	printf("%d\n",ret);
	if (ret > 0) for (i=0;i<nl.len;i++) {
	  printf("%02X",rbuf[i]);
	  if (i+1 < nl.len) printf(":");
	}
	printf("\n");
      }
      break;
    case NetSPI_ClockSet: {
	unsigned hz;
	OptionUnsigned(argv[4],&hz,0);
	spi->API->Lock(spi,WaitLock,0);
	ret = spi->API->ClockSet(spi,hz);
	spi->API->Lock(spi,DoRelease,0);
	printf("%d\n",ret);
	break;
    }
    case NetSPI_EdgeSet:
      OptionUnsigned(argv[4],&i,0);
      spi->API->Lock(spi,WaitLock,0);
      ret = spi->API->EdgeSet(spi,i);
      spi->API->Lock(spi,DoRelease,0);
      printf("%d\n",ret);
      break;
    }
  }
    break;
  case NetTWIClass: {
    int devadr, adrslen, adrs, len,i;
    struct opt_numlist nl;

    TWI *twi = ClientTWIInit(cmd.Inst);
    if (!twi) {
      printf("Error: TWI %d not defined for this architecture\n",cmd.Inst);
      return 1;
    }
    if (twi->InitStatus <= 0) {
      printf("Error %d\n",-twi->InitStatus);
      return 1;
    }
    switch (cmd.OpCode) {
    case NetTWI_Write:
      OptionUnsigned(argv[4],&devadr,0);
      OptionUnsigned(argv[5],&adrslen,0);
      OptionUnsigned(argv[6],&adrs,0);
      OptionNumList(argv[7],&nl,0);
      twi->API->Lock(twi,WaitLock,0);
      twi->API->Write(twi,devadr,adrslen,adrs,nl.len,nl.dat);
      twi->API->Lock(twi,DoRelease,0);
      break;
    case NetTWI_Read:
      OptionUnsigned(argv[4],&devadr,0);
      OptionUnsigned(argv[5],&adrslen,0);
      OptionUnsigned(argv[6],&adrs,0);
      OptionUnsigned(argv[7],&len,0);
      twi->API->Lock(twi,WaitLock,0);
      twi->API->Read(twi,devadr,adrslen,adrs,len,nl.dat);
      twi->API->Lock(twi,DoRelease,0);
      for (i=0;i<len;i++) {
	printf("%02X",nl.dat[i]);
	if (i+1 < len) printf(":");
      }
      printf("\n");
      break;
    }
  }
    break;
  case NetCANClass: {
    CAN *can = ClientCANInit(cmd.Inst);
    CANMessage msg,txmsg;
    int ret;
    unsigned baud;
    struct opt_numlist nl;

    if (!can) {
      printf("Error: CAN %d not defined for this architecture\n",cmd.Inst);
      return 1;
    }
    if (can->InitStatus <= 0) {
      printf("Error %d\n",-can->InitStatus);
      return 1;
    }
    switch (cmd.OpCode) {
    case NetCAN_Rx:
      if ((ret=can->API->Rx(can,&msg)) <= 0) {
	fprintf(stderr,"CAN Rx error %d\n",ret);
      } else {
	CANMessagePrint(&msg);
      }
      break;
    case NetCAN_Tx:
      // For transmitting CAN packets we need to handle our args a bit
      // differently.  We can have 1-4 arguments in any order:
      // F:ID1:ID2:ID3:ID4:D1:D2:D3:D4:D5:D6:D7
      // E|S        extended or standard ID (default is extended)
      // RTR        RTR flag
      // ID         integer, default is 0
      // D1[:...]   numlist containing data bytes
      // Our argument checking does not allow a variable number of
      // arguments, so we must fake it by quoting all the args into
      // a single argument, e.g.:
      // tsctl CAN 0 Tx "0x1234 1:2:3"
      OptionCAN(argv[4],&txmsg,0);
      if (txmsg.length != 0) {
	if ((ret=can->API->Tx(can,&txmsg)) <= 0) {
	  fprintf(stderr,"CAN Tx error %d\n",ret);
	}
      }
      break;
    case NetCAN_BaudSet:
      OptionUnsigned(argv[4],&baud,0);
      ret = can->API->BaudSet(can,baud);
      printf("baud=%u\n",ret);
      break;
    case NetCAN_BaudGet:
      ret = can->API->BaudGet(can);
      printf("baud=%u\n",ret);
      break;
    }  
  }
    break;
  }
}

int CommandParse(int argc,char **argv,tsctlCommand *cmd,int canlock) {
  char *sep;

  // split instance from class, if specified, otherwise assume 0
  if (sep = strchr(argv[2],':')) {
    *sep = 0;
    cmd->Inst = strtol(sep+1,NULL,0);
    if (cmd->Inst < 0) cmd->Inst = 0;
  } else {
    cmd->Inst = 0;
  }
  for (cmd->Class=0;Class[cmd->Class];cmd->Class++) {
    if (!strcmp(Class[cmd->Class],argv[2])) {
      break;
    }
  }
  if (!Class[cmd->Class]) return CommandErrorClass; 
  if (argc < 4) return CommandErrorArgCount;
  if (!strcmp(argv[3],"Lock")) {
    // Lock: Bus, Pin, DIORaw, DIO, TWI, SPI
    // Don't: System, Time, CAN
    if (cmd->Class == NetSystemClass || cmd->Class == NetTimeClass
	|| cmd->Class == NetCANClass) return CommandErrorCommandBad;
    cmd->OpCode = -1;
    if (argc != 6) return CommandErrorArgsBad;
  } else {
    // check for valid Command with correct number of args
    for (cmd->Command=0;CommandNames[cmd->Class][cmd->Command];cmd->Command++) {
      if (!strcmp(CommandNames[cmd->Class][cmd->Command],argv[3])) {
	break;
      }
    }
    if (!CommandNames[cmd->Class][cmd->Command]) return CommandErrorCommandBad;

    if (argc-4 != CommandOpts[cmd->Class][cmd->Command]) { // or <
      return CommandErrorArgsBad;
    }
    cmd->OpCode = CommandOpcode[cmd->Class][cmd->Command];
  }

}

int CommandExec(tsctlCommand *cmd,char **argv) {
  int i,j;
  Bus *bus; 
  DIO *dio;
  char **a = argv;
  const char *str;

  switch(cmd->Class) {
  case NetSystemClass:
    if (!cmd->sys) return CommandErrorNoSuchObject;
    if (cmd->sys->InitStatus <= 0) {
      cmd->Status = cmd->sys->InitStatus;
      return CommandErrorObjectInit;
    }
    switch (cmd->OpCode) {
    case NetSystem_ClassCount:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->ClassCount(cmd->sys);
      return 1;
    case NetSystem_InstanceCount:
      OptionUnsigned(argv[4],&i,0);
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->InstanceCount(cmd->sys,i);
      return 1;
    case NetSystem_APICount:
      OptionUnsigned(argv[4],&i,0);
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->APICount(cmd->sys,i);
      return 1;
    case NetSystem_LockCount:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->LockCount(cmd->sys);
      return 1;
    case NetSystem_LockHolderInfo:
      i = cmd->sys->API->LockCount(cmd->sys);
      cmd->rType = T_VectorInteger32;
      cmd->rVectorInteger32 = malloc((2*i+1)*sizeof(int));
      cmd->rVectorInteger32[0] = 2 * i;
      return cmd->sys->API->LockHolderInfo(cmd->sys,(void *)&cmd->rVectorInteger32[1]);
    case NetSystem_ConnWaitInfo:
      cmd->rType = T_VectorInteger32;
      cmd->rVectorInteger32 = malloc((1024+1)*sizeof(int));
      cmd->rVectorInteger32[0] = 
	cmd->sys->API->ConnWaitInfo(cmd->sys,
				   1024,(void *)&cmd->rVectorInteger32[1]);
      return 1;
    case NetSystem_CANBusGet:
      OptionUnsigned(argv[4],&i,0);
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->CANBusGet(cmd->sys,i);
      return 1;
    case NetSystem_BuildInfo:
      cmd->rType = T_BuildInfo;
      cmd->rBuildInfo = malloc(sizeof(struct BuildInfo));
      cmd->sys->API->BuildInfo(cmd->sys,cmd->rBuildInfo);
      return 1;
    case NetSystem_ModelId:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->ModelId(cmd->sys);
      return 1;
    case NetSystem_BaseBoardId:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->BaseBoardId(cmd->sys);
      return 1;
    case NetSystem_MapLength:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->MapLength(cmd->sys);
      return 1;
    case NetSystem_MapGet:
      cmd->rType = T_Pair;
      cmd->rPair.key = malloc(257);
      i=cmd->sys->API->MapGet(cmd->sys,atoi(argv[4]),cmd->rPair.key,&cmd->rPair.value);
      cmd->rPair.key[i]=0;
      return 1;
    case NetSystem_MapLookup:
      cmd->rType = T_Pair;
      cmd->rPair.key = argv[4];
      cmd->rPair.value = cmd->sys->API->MapLookup(cmd->sys,argv[4]);
      return 1;
    case NetSystem_MapLookupPartial:
      cmd->rType = T_Pair;
      cmd->rPair.key = malloc(257);
      j = atoi(argv[5]);
      i=cmd->sys->API->MapLookupPartial(cmd->sys,argv[4],j,cmd->rPair.key);
      cmd->rPair.key[i]=0;
      cmd->rPair.value = j;
      return 1;
    case NetSystem_MapAdd:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->MapAdd(cmd->sys,argv[4],atoi(argv[5]));
      return 1;
    case NetSystem_MapDelete:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = cmd->sys->API->MapDelete(cmd->sys,argv[4]);
      return 1;
      
    }
    break; // case NetSystemClass of switch(cmd->Class)
  case NetBusClass:
    bus = cmd->bus;
    if (!bus) return CommandErrorNoSuchObject;
    if (bus->InitStatus <= 0) {
      cmd->Status = bus->InitStatus;
      return CommandErrorObjectInit;
    }
    switch (cmd->OpCode) {
    case -1: // Lock
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Lock(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_Peek8:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Peek8(bus,U32(a[4]));
      return 1;
    case NetBus_Peek16:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Peek16(bus,U32(a[4]));
      return 1;
    case NetBus_Peek32:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Peek32(bus,U32(a[4]));
      return 1;
    case NetBus_BitGet8:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitGet8(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitGet16:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitGet16(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitGet32:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitGet32(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_Poke8: // Poke8 address value
      cmd->rType = T_Void;
      bus->API->Poke8(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_Poke16: // Poke32 address value
      cmd->rType = T_Void;
      bus->API->Poke16(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_Poke32: // Poke32 address value
      cmd->rType = T_Void;
      bus->API->Poke32(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitSet8: // BitSet8 address bit
      cmd->rType = T_Void;
      bus->API->BitSet8(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitSet16: // BitSet16 address bit
      cmd->rType = T_Void;
      bus->API->BitSet16(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitSet32: // BitSet32 address bit
      cmd->rType = T_Void;
      bus->API->BitSet32(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitClear8: // BitClear8 address bit
      cmd->rType = T_Void;
      bus->API->BitClear8(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitClear16: // BitClear16 address bit
      cmd->rType = T_Void;
      bus->API->BitClear16(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitClear32: // BitClear32 address bit
      cmd->rType = T_Void;
      bus->API->BitClear32(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitAssign8: // BitAssign8 address bit value
      cmd->rType = T_Void;
      bus->API->BitAssign8(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      return 1;
    case NetBus_BitAssign16: // BitAssign16 address bit value
      cmd->rType = T_Void;
      bus->API->BitAssign16(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      return 1;
    case NetBus_BitAssign32: // BitAssign32 address bit value
      cmd->rType = T_Void;
      bus->API->BitAssign32(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      return 1;
    case NetBus_Refresh: // Refresh
      cmd->rType = T_Void;
      bus->API->Refresh(bus);
      return 1;
    case NetBus_Commit: // Commit
      cmd->rType = T_Void;
      bus->API->Commit(bus,U32(a[4]));
      return 1;
    case NetBus_BitToggle8: // BitToggle8 address bit
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitToggle8(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitToggle16: // BitToggle16 address bit
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitToggle16(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_BitToggle32: // BitToggle32 address bit
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitToggle32(bus,U32(a[4]),U32(a[5]));
      return 1;
    case NetBus_Assign8X: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Assign8X(bus,U32(a[4]),U32(a[5]),
					   U32(a[6]),U32(a[7]));
      return 1;
    case NetBus_Assign16X: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Assign16X(bus,U32(a[4]),U32(a[5]),
					    U32(a[6]),U32(a[7]));
      return 1;
    case NetBus_Assign32X:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->Assign32X(bus,U32(a[4]),U32(a[5]),
					    U32(a[6]),U32(a[7]));
      return 1;
    case NetBus_BitsGet8: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitsGet8(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      return 1;
    case NetBus_BitsGet16: 
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitsGet16(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      return 1;
    case NetBus_BitsGet32:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = bus->API->BitsGet32(bus,U32(a[4]),U32(a[5]),U32(a[6]));
      return 1;

    }
    break; // case NetBusClass of switch(cmd->Class)
  case NetDIOClass:
    dio = cmd->dio;
    if (!dio) return CommandErrorNoSuchObject;
    if (dio->InitStatus <= 0) {
      cmd->Status = dio->InitStatus;
      return CommandErrorObjectInit;
    }
    switch (cmd->OpCode) {
    case -1: // Lock
      cmd->rType = T_Integer32;
      cmd->rInteger32 = dio->API->Lock(dio,U32(a[4]),U32(a[5]));
      return 1;
     case NetDIO_Refresh:
      cmd->rType = T_Void;
      dio->API->Refresh(dio);
      return 1;
    case NetDIO_Commit:
      cmd->rType = T_Void;
      dio->API->Commit(dio,U32(a[4]));
      return 1;
    case NetDIO_Set:
      cmd->rType = T_Void;
      dio->API->Set(dio,DIONum(argv[4]),State(argv[5]));
      return 1;
    case NetDIO_Get:
      cmd->rType = T_VectorInteger8; 
      str = DIOValueString(dio->API->Get(dio,DIONum(argv[4])));
      i = strlen(str);
      cmd->rVectorInteger8 = malloc(i+2);
      cmd->rVectorInteger8[0] = i;
      strcpy(cmd->rVectorInteger8+1,str);
      return 1;
    case NetDIO_SetAsync:
      cmd->rType = T_Void;
      dio->API->SetAsync(dio,DIONum(argv[4]),State(argv[5]));
      return 1;
    case NetDIO_GetAsync:
      cmd->rType = T_VectorInteger8; 
      str = DIOValueString(dio->API->GetAsync(dio,DIONum(argv[4])));
      i = strlen(str);
      cmd->rVectorInteger8 = malloc(i+2);
      cmd->rVectorInteger8[0] = i;
      strcpy(cmd->rVectorInteger8+1,str);
      return 1;
    case NetDIO_Wait:

      return 1;
    case NetDIO_Count:
      cmd->rType = T_Integer32;
      cmd->rInteger32 = dio->API->Count(dio); 
      return 1;
    case NetDIO_Capabilities:
      return 1;
    }
    break; // case NetDIOClass of switch(cmd->Class)
  }
}

void httpCommand(int socket,char *cmd) {
  int i,len = strlen(cmd);
  char *str=cmd,*argv[16];
  int argc=2,quoting=0,ret;
  tsctlCommand tsctlcmd;

  if (cmd[len-1] == '\n') cmd[len-1] = 0;
  argv[argc++] = str++;
  while (*str && argc < 16) {
    if (*str == ' ' && !quoting) {
      argv[argc++] = str+1;
      *str = 0;
    } else if (*str == '"' || *str == '\'') {
      quoting = !quoting;
    }
    str++;
  }
  ret = CommandParse(argc,argv,&tsctlcmd,1);
  switch (ret) {
  case CommandErrorClass:
    zprintf(socket,"{ \"error\": \"unrecognized Class\" }");
    return;
  case CommandErrorArgCount:
    zprintf(socket,"{ \"error\": \"insufficient arguments\" }");
    return; 
  case CommandErrorCommandBad:
    zprintf(socket,"{ \"error\": \"bad command\" }");
    return; 
  case CommandErrorArgsBad:
    zprintf(socket,"{ \"error\": \"wrong arguments\" }");
    return; 
  }
  switch (tsctlcmd.Class) {
  case NetSystemClass: tsctlcmd.sys = &LocalSystem; break;
  case NetBusClass: 
    tsctlcmd.bus = (tsctlcmd.Inst < numBusses) ? busses[tsctlcmd.Inst] : 0; 
    break;
  case NetTimeClass: 
    tsctlcmd.time = (tsctlcmd.Inst < numBusses) ? times[tsctlcmd.Inst] : 0; 
    break;
  case NetDIORawClass: 
    tsctlcmd.dioraw = (tsctlcmd.Inst<numBusses) ? dioraws[tsctlcmd.Inst] : 0; 
    break;
  case NetDIOClass: 
    tsctlcmd.dio = (tsctlcmd.Inst < numBusses) ? dios[tsctlcmd.Inst] : 0; 
    break;
  case NetTWIClass: 
    tsctlcmd.twi = (tsctlcmd.Inst < numBusses) ? twis[tsctlcmd.Inst] : 0; 
    break;
  case NetSPIClass: 
    tsctlcmd.spi = (tsctlcmd.Inst < numBusses) ? spis[tsctlcmd.Inst] : 0; 
    break;
  case NetCANClass: 
    tsctlcmd.can = (tsctlcmd.Inst < numBusses) ? cans[tsctlcmd.Inst] : 0; 
    break;
  }
  ret = CommandExec(&tsctlcmd,argv);
  switch (ret) {
  case CommandErrorNoSuchObject:
    zprintf(socket,"{ \"error\": \"no such instance\" }");
    return;
  case CommandErrorObjectInit:
    zprintf(socket,"{ \"error\": \"instance init error %d\" }",tsctlcmd.Status);
    return;
  }

  switch (tsctlcmd.rType) {
  case T_Void:
    zprintf(socket,"null");
    break;
  case T_Integer32:
    zprintf(socket,"%d",tsctlcmd.rInteger32);
    break;
  case T_VectorInteger32:
    zprintf(socket,"[");
    for (i=1;i<=tsctlcmd.rVectorInteger32[0];i++) {
      if (i == tsctlcmd.rVectorInteger32[0]) {
	zprintf(socket,"%d",tsctlcmd.rVectorInteger32[i]);
      } else {
	zprintf(socket,"%d,",tsctlcmd.rVectorInteger32[i]);
      }
    }
    zprintf(socket,"]");
    free(tsctlcmd.rVectorInteger32);
    break;
  case T_BuildInfo: 
    break;
  case T_VectorInteger8:
    zprintf(socket,"\"");
    write(socket,tsctlcmd.rVectorInteger8+1,tsctlcmd.rVectorInteger8[0]);
    zprintf(socket,"\"");
    free(tsctlcmd.rVectorInteger8);
    break;
  case T_Pair:
    zprintf(socket,"{ \"%s\": %d }",tsctlcmd.rPair.key,tsctlcmd.rPair.value);
    free(tsctlcmd.rPair.key);
    break;
  }
  //zprintf(socket,"args: %d:",args);
  //for (i=0;i<args;i++) zprintf(socket,"%s:",arg[i]);
  //zprintf(socket,"\n");
}
// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2011, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
